@@ -275,7 +275,7 @@ mgmt_get_volinfo_cbk(struct rpc_req *req, struct iovec *iov, int count,
         goto out;
     }
 
-    fs = ((xlator_t *)ctx->master)->private;
+    fs = ((xlator_t *)ctx->root)->private;
 
     if (-1 == req->rpc_status) {
         gf_smsg(frame->this->name, GF_LOG_ERROR, EINVAL,
@@ -364,6 +364,7 @@ out:
     return ret;
 }
 
+GFAPI_SYMVER_PUBLIC_DEFAULT(glfs_get_volumeid, 3.5.0)
 int
 pub_glfs_get_volumeid(struct glfs *fs, char *volid, size_t size)
 {
@@ -418,8 +419,6 @@ invalid_fs:
     return -1;
 }
 
-GFAPI_SYMVER_PUBLIC_DEFAULT(glfs_get_volumeid, 3.5.0);
-
 int
 glfs_get_volume_info(struct glfs *fs)
 {
@@ -567,7 +566,7 @@ glfs_mgmt_getspec_cbk(struct rpc_req *req, struct iovec *iov, int count,
         goto out;
     }
 
-    fs = ((xlator_t *)ctx->master)->private;
+    fs = ((xlator_t *)ctx->root)->private;
 
     if (-1 == req->rpc_status) {
         ret = -1;
@@ -829,7 +828,7 @@ mgmt_rpc_notify(struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
     if (!ctx)
         goto out;
 
-    fs = ((xlator_t *)ctx->master)->private;
+    fs = ((xlator_t *)ctx->root)->private;
 
     switch (event) {
         case RPC_CLNT_DISCONNECT:
@@ -1014,11 +1013,6 @@ glfs_mgmt_init(struct glfs *fs)
     if (ret)
         goto out;
 
-    if (sys_access(SECURE_ACCESS_FILE, F_OK) == 0) {
-        ctx->secure_mgmt = 1;
-        ctx->ssl_cert_depth = glusterfs_read_secure_access_file();
-    }
-
     rpc = rpc_clnt_new(options, THIS, THIS->name, 8);
     if (!rpc) {
         ret = -1;
