@@ -140,6 +140,9 @@ out:
     }
 
     CLI_STACK_DESTROY(frame);
+    if (options)
+        dict_unref(options);
+
     return ret;
 }
 
@@ -190,6 +193,8 @@ out:
     }
 
     CLI_STACK_DESTROY(frame);
+    if (options)
+        dict_unref(options);
 
     return ret;
 }
