@@ -33,12 +33,6 @@
 #include <malloc.h>
 #endif
 
-#ifdef HAVE_MALLOC_STATS
-#ifdef DEBUG
-#include <mcheck.h>
-#endif
-#endif
-
 #include "cli.h"
 #include "cli-quotad-client.h"
 #include "cli-cmd.h"
@@ -61,7 +55,6 @@
 
 #include "xdr-generic.h"
 
-extern int connected;
 /* using argp for command line parsing */
 
 const char *argp_program_version =
@@ -77,13 +70,13 @@ const char *argp_program_version =
     "in all cases as published by the Free Software Foundation.";
 const char *argp_program_bug_address = "<" PACKAGE_BUGREPORT ">";
 
-struct rpc_clnt *global_quotad_rpc;
-struct rpc_clnt *global_rpc;
-
 rpc_clnt_prog_t *cli_rpc_prog;
 
 extern struct rpc_clnt_program cli_prog;
 
+time_t cli_default_conn_timeout = 120;
+time_t cli_ten_minutes_timeout = 600;
+
 static int
 glusterfs_ctx_defaults_init(glusterfs_ctx_t *ctx)
 {
@@ -280,8 +273,11 @@ cli_submit_request(struct rpc_clnt *rpc, void *req, call_frame_t *frame,
         count = 1;
     }
 
-    if (!rpc)
-        rpc = global_rpc;
+    if (!rpc) {
+        rpc = ((cli_state_t *)(frame->this->private))->rpc;
+        GF_ASSERT(rpc);
+    }
+
     /* Send the msg */
     ret = rpc_clnt_submit(rpc, prog, procnum, cbkfn, &iov, count, NULL, 0,
                           iobref, frame, NULL, 0, NULL, 0, NULL);
@@ -299,22 +295,25 @@ int
 cli_rpc_notify(struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
                void *data)
 {
+    struct cli_state *state = NULL;
     xlator_t *this = NULL;
     int ret = 0;
 
     this = mydata;
+    state = this->private;
+    GF_ASSERT(state);
 
     switch (event) {
         case RPC_CLNT_CONNECT: {
-            cli_cmd_broadcast_connected();
+            cli_cmd_broadcast_connected(_gf_true);
             gf_log(this->name, GF_LOG_TRACE, "got RPC_CLNT_CONNECT");
             break;
         }
 
         case RPC_CLNT_DISCONNECT: {
+            cli_cmd_broadcast_connected(_gf_false);
             gf_log(this->name, GF_LOG_TRACE, "got RPC_CLNT_DISCONNECT");
-            connected = 0;
-            if (!global_state->prompt && global_state->await_connected) {
+            if (!state->prompt && state->await_connected) {
                 ret = 1;
                 cli_out(
                     "Connection failed. Please check if gluster "
@@ -334,26 +333,6 @@ cli_rpc_notify(struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
     return ret;
 }
 
-static gf_boolean_t
-is_valid_int(char *str)
-{
-    if (*str == '-')
-        ++str;
-
-    /* Handle empty string or just "-".*/
-    if (!*str)
-        return _gf_false;
-
-    /* Check for non-digit chars in the rest of the string */
-    while (*str) {
-        if (!isdigit(*str))
-            return _gf_false;
-        else
-            ++str;
-    }
-    return _gf_true;
-}
-
 /*
  * ret: 0: option successfully processed
  *      1: signalling end of option list
@@ -452,12 +431,16 @@ cli_opt_parse(char *opt, struct cli_state *state)
     }
     oarg = strtail(opt, "timeout=");
     if (oarg) {
-        if (!is_valid_int(oarg) || atoi(oarg) <= 0) {
+        time_t val;
+        char *endptr = NULL;
+
+        val = strtol(oarg, &endptr, 10);
+        if (*endptr != '\0' || val <= 0) {
             cli_err("timeout value should be a positive integer");
-            return -2; /* -2 instead of -1 to avoid unknown option
-                          error */
+            /* Use -2 to not confuse with unknown option error. */
+            return -2;
         }
-        cli_default_conn_timeout = atoi(oarg);
+        cli_default_conn_timeout = val;
         return 0;
     }
 
@@ -590,7 +573,8 @@ _cli_err(const char *fmt, ...)
     va_list ap;
     int ret = 0;
 #ifdef HAVE_READLINE
-    struct cli_state *state = global_state;
+    struct cli_state *state = THIS->private;
+    GF_ASSERT(state);
 #endif
 
     va_start(ap, fmt);
@@ -616,7 +600,8 @@ _cli_out(const char *fmt, ...)
     va_list ap;
     int ret = 0;
 #ifdef HAVE_READLINE
-    struct cli_state *state = global_state;
+    struct cli_state *state = THIS->private;
+    GF_ASSERT(state);
 #endif
 
     va_start(ap, fmt);
@@ -635,8 +620,8 @@ _cli_out(const char *fmt, ...)
     return ret;
 }
 
-struct rpc_clnt *
-cli_quotad_clnt_rpc_init(void)
+static int
+cli_quotad_clnt_rpc_init(cli_state_t *state)
 {
     struct rpc_clnt *rpc = NULL;
     dict_t *rpc_opts = NULL;
@@ -662,18 +647,19 @@ cli_quotad_clnt_rpc_init(void)
         goto out;
 
     rpc = cli_quotad_clnt_init(THIS, rpc_opts);
-    if (!rpc)
+    if (!rpc) {
+        ret = -1;
         goto out;
-
-    global_quotad_rpc = rpc;
-out:
-    if (rpc_opts) {
-        dict_unref(rpc_opts);
     }
-    return rpc;
+    state->quotad_rpc = rpc;
+    ret = 0;
+out:
+    if (rpc_opts)
+        dict_unref(rpc_opts);
+    return ret;
 }
 
-struct rpc_clnt *
+static int
 cli_rpc_init(struct cli_state *state)
 {
     struct rpc_clnt *rpc = NULL;
@@ -761,8 +747,10 @@ out:
         if (rpc)
             rpc_clnt_unref(rpc);
         rpc = NULL;
-    }
-    return rpc;
+    } else
+        state->rpc = rpc;
+
+    return ret;
 }
 
 cli_local_t *
@@ -790,8 +778,6 @@ cli_local_wipe(cli_local_t *local)
     return;
 }
 
-struct cli_state *global_state;
-
 int
 main(int argc, char *argv[])
 {
@@ -829,7 +815,7 @@ main(int argc, char *argv[])
         goto out;
 
     state.ctx = ctx;
-    global_state = &state;
+    THIS->private = &state;
 
     ret = parse_cmdline(argc, argv, &state);
     if (ret)
@@ -842,19 +828,19 @@ main(int argc, char *argv[])
     gf_log("cli", GF_LOG_INFO, "Started running %s with version %s", argv[0],
            PACKAGE_VERSION);
 
-    global_rpc = cli_rpc_init(&state);
-    if (!global_rpc)
-        goto out;
-
-    global_quotad_rpc = cli_quotad_clnt_rpc_init();
-    if (!global_quotad_rpc)
-        goto out;
-
-    ret = cli_cmds_register(&state);
+    ret = cli_rpc_init(&state);
     if (ret)
         goto out;
 
-    ret = cli_cmd_cond_init();
+    /* Do not initialize quota RPC context unless and until quota is enabled. */
+    ret = sys_access(QUOTAD_PID_PATH, F_OK);
+    if (!ret) {
+        ret = cli_quotad_clnt_rpc_init(&state);
+        if (ret)
+            goto out;
+    }
+
+    ret = cli_cmds_register(&state);
     if (ret)
         goto out;
 
