@@ -316,7 +316,12 @@ glusterfs_handle_terminate(rpcsvc_request_t *req)
     xlator_list_t **trav_p = NULL;
     gf_boolean_t lockflag = _gf_false;
     gf_boolean_t still_bricks_attached = _gf_false;
+    dict_t *dict = NULL;
+    xlator_t *this = NULL;
+    char *value = NULL;
+    gf_boolean_t graceful_cleanup = _gf_false;
 
+    this = THIS;
     ret = xdr_to_generic(req->msg[0], &xlator_req,
                          (xdrproc_t)xdr_gd1_mgmt_brick_op_req);
     if (ret < 0) {
@@ -325,6 +330,28 @@ glusterfs_handle_terminate(rpcsvc_request_t *req)
     }
     ctx = glusterfsd_ctx;
 
+    dict = dict_new();
+    if (!dict) {
+        return -1;
+    }
+    if (xlator_req.dict.dict_len) {
+        ret = dict_unserialize(xlator_req.dict.dict_val,
+                               xlator_req.dict.dict_len, &dict);
+        if (ret < 0) {
+            gf_log(this->name, GF_LOG_ERROR,
+                   "Failed to unserialize "
+                   "req-buffer to dictionary");
+            goto err;
+        }
+    }
+
+    ret = dict_get_str(dict, GLUSTER_BRICK_GRACEFUL_CLEANUP, &value);
+    if (!ret) {
+        ret = gf_string2boolean(value, &graceful_cleanup);
+        if (ret)
+            graceful_cleanup = _gf_false;
+    }
+
     LOCK(&ctx->volfile_lock);
     {
         /* Find the xlator_list_t that points to our victim. */
@@ -343,7 +370,7 @@ glusterfs_handle_terminate(rpcsvc_request_t *req)
             goto err;
     }
     if (!*trav_p) {
-        gf_log(THIS->name, GF_LOG_ERROR, "can't terminate %s - not found",
+        gf_log(this->name, GF_LOG_ERROR, "can't terminate %s - not found",
                xlator_req.name);
         /*
          * Used to be -ENOENT.  However, the caller asked us to
@@ -366,12 +393,16 @@ glusterfs_handle_terminate(rpcsvc_request_t *req)
             break;
         }
     }
-    if (!still_bricks_attached) {
-        gf_log(THIS->name, GF_LOG_INFO,
+    /* Cleanup brick resource gracefully if enabled is true */
+    if (!still_bricks_attached && !graceful_cleanup) {
+        gf_log(this->name, GF_LOG_INFO,
                "terminating after loss of last child %s", xlator_req.name);
         rpc_clnt_mgmt_pmap_signout(glusterfsd_ctx, xlator_req.name);
         kill(getpid(), SIGTERM);
     } else {
+        /* Check if detach brick is a last brick */
+        if (!still_bricks_attached && graceful_cleanup)
+            ctx->cleanup_starting = 1;
         /* TODO cleanup sequence needs to be done properly for
            Quota and Changelog
         */
@@ -384,19 +415,21 @@ glusterfs_handle_terminate(rpcsvc_request_t *req)
         UNLOCK(&ctx->volfile_lock);
         lockflag = _gf_true;
 
-        gf_log(THIS->name, GF_LOG_INFO,
-               "detaching not-only"
-               " child %s",
-               xlator_req.name);
+        gf_log(this->name, GF_LOG_INFO,
+               "detaching not-only child %s "
+               " graceful_cleanup %d",
+               xlator_req.name, graceful_cleanup);
         top->notify(top, GF_EVENT_CLEANUP, victim);
     }
 err:
     if (!lockflag)
         UNLOCK(&ctx->volfile_lock);
-    if (xlator_req.input.input_val)
-        free(xlator_req.input.input_val);
     if (xlator_req.dict.dict_val)
         free(xlator_req.dict.dict_val);
+    if (xlator_req.input.input_val)
+        free(xlator_req.input.input_val);
+    if (dict)
+        dict_unref(dict);
     free(xlator_req.name);
     xlator_req.name = NULL;
     return 0;
@@ -2337,6 +2370,7 @@ post_unlock:
            restart the process */
         gf_log("mgmt", GF_LOG_ERROR, "failed to fetch volume file (key:%s)",
                ctx->cmd_args.volfile_id);
+        emancipate(ctx, ret);
         cleanup_and_exit(0);
     }
 
@@ -2653,6 +2687,23 @@ mgmt_rpc_notify(struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
                 }
             }
             server = ctx->cmd_args.curr_server;
+
+            if (ctx->cmd_args.brick_port && ctx->cmd_args.brick_name) {
+                /* This process requires a portmap signin with glusterd.
+                 * Currently the glusterd portmaps are local to each glusterd.
+                 * Hence connecting the process to a different volfile server
+                 * won't work well with such process, so don't try to connect
+                 * to backup volfile server here.
+                 */
+                if (!ctx->active) {
+                    need_term = 1;
+                }
+                emval = ENOTCONN;
+                GF_LOG_OCCASIONALLY(log_ctr2, "glusterfsd-mgmt", GF_LOG_INFO,
+                                    "Port-mapper is active, Giving up on the "
+                                    "backup volfile servers");
+                break;
+            }
             if (server->list.next == &ctx->cmd_args.volfile_servers) {
                 if (!ctx->active) {
                     need_term = 1;
