@@ -25,8 +25,8 @@
 #include "config.h"
 #endif
 
-#include "xlator.h"
-#include "defaults.h"
+#include "glusterfs/xlator.h"
+#include "glusterfs/defaults.h"
 
 #pragma generate
 
@@ -84,6 +84,7 @@ struct xlator_fops _default_fops = {
     .put = default_put,
     .icreate = default_icreate,
     .namelink = default_namelink,
+    .copy_file_range = default_copy_file_range,
 };
 struct xlator_fops *default_fops = &_default_fops;
 
@@ -124,13 +125,24 @@ int
 default_notify(xlator_t *this, int32_t event, void *data, ...)
 {
     GF_UNUSED int ret = 0;
+    xlator_t *victim = data;
+
+    glusterfs_graph_t *graph = NULL;
+
+    GF_VALIDATE_OR_GOTO("notify", this, out);
+    graph = this->graph;
+    GF_VALIDATE_OR_GOTO(this->name, graph, out);
+
     switch (event) {
         case GF_EVENT_PARENT_UP:
         case GF_EVENT_PARENT_DOWN: {
             xlator_list_t *list = this->children;
 
             while (list) {
-                xlator_notify(list->xlator, event, this);
+                if (victim && victim->cleanup_starting)
+                    xlator_notify(list->xlator, event, victim);
+                else
+                    xlator_notify(list->xlator, event, this);
                 list = list->next;
             }
         } break;
@@ -144,8 +156,8 @@ default_notify(xlator_t *this, int32_t event, void *data, ...)
              * Handle case of CHILD_* & AUTH_FAILED event specially, send
              * it to fuse.
              */
-            if (!parent && this->ctx && this->ctx->master) {
-                xlator_notify(this->ctx->master, event, this->graph, NULL);
+            if (!parent && this->ctx && this->ctx->root) {
+                xlator_notify(this->ctx->root, event, this->graph, NULL);
             }
 
             while (parent) {
@@ -153,12 +165,26 @@ default_notify(xlator_t *this, int32_t event, void *data, ...)
                     xlator_notify(parent->xlator, event, this, NULL);
                 parent = parent->next;
             }
+
+            if (event == GF_EVENT_CHILD_DOWN &&
+                !(this->ctx && this->ctx->root) && (graph->top == this)) {
+                /* Make sure this is not a daemon with root xlator */
+                pthread_mutex_lock(&graph->mutex);
+                {
+                    if (graph->parent_down ==
+                        graph_total_client_xlator(graph)) {
+                        graph->used = 0;
+                        pthread_cond_broadcast(&graph->child_down_cond);
+                    }
+                }
+                pthread_mutex_unlock(&graph->mutex);
+            }
         } break;
         case GF_EVENT_UPCALL: {
             xlator_list_t *parent = this->parents;
 
-            if (!parent && this->ctx && this->ctx->master)
-                xlator_notify(this->ctx->master, event, data, NULL);
+            if (!parent && this->ctx && this->ctx->root)
+                xlator_notify(this->ctx->root, event, data, NULL);
 
             while (parent) {
                 if (parent->xlator->init_succeeded)
@@ -199,7 +225,7 @@ default_notify(xlator_t *this, int32_t event, void *data, ...)
              * nothing to do with readability.
              */
     }
-
+out:
     return 0;
 }
 
