@@ -16,15 +16,26 @@
 #include <limits.h>
 #include <fnmatch.h>
 
+
+/* dict_t is always initialized with hash_size = 1
+ * with this usage it's implemented as a doubly-linked list
+ * and the hash calculation done per operation is not necessary.
+ * using this macro to delimit blocks related to hash imp */
+#define DICT_LIST_IMP 1
+
+
 #include "glusterfs/dict.h"
+#if !DICT_LIST_IMP
 #define XXH_INLINE_ALL
 #include "xxhash.h"
+#endif
 #include "glusterfs/compat.h"
 #include "glusterfs/compat-errno.h"
 #include "glusterfs/byte-order.h"
 #include "glusterfs/statedump.h"
 #include "glusterfs/libglusterfs-messages.h"
 
+
 struct dict_cmp {
     dict_t *dict;
     gf_boolean_t (*value_ignore)(char *k);
@@ -77,31 +88,31 @@ get_new_dict_full(int size_hint)
     }
 
     dict->hash_size = size_hint;
-    if (size_hint == 1) {
-        /*
-         * This is the only case we ever see currently.  If we ever
-         * need to support resizing the hash table, the resize function
-         * will have to take into account the possibility that
-         * "members" is not separately allocated (i.e. don't just call
-         * realloc() blindly.
-         */
-        dict->members = &dict->members_internal;
-    } else {
-        /*
-         * We actually need to allocate space for size_hint *pointers*
-         * but we actually allocate space for one *structure*.  Since
-         * a data_pair_t consists of five pointers, we're wasting four
-         * pointers' worth for N=1, and will overrun what we allocated
-         * for N>5.  If anybody ever starts using size_hint, we'll need
-         * to fix this.
-         */
-        GF_ASSERT(size_hint <= (sizeof(data_pair_t) / sizeof(data_pair_t *)));
-        dict->members = mem_get0(THIS->ctx->dict_pair_pool);
-        if (!dict->members) {
-            mem_put(dict);
-            return NULL;
-        }
+#if DICT_LIST_IMP
+    /*
+     * This is the only case we ever see currently.  If we ever
+     * need to support resizing the hash table, the resize function
+     * will have to take into account the possibility that
+     * "members" is not separately allocated (i.e. don't just call
+     * realloc() blindly.
+     */
+    dict->members = &dict->members_internal;
+#else
+    /*
+     * We actually need to allocate space for size_hint *pointers*
+     * but we actually allocate space for one *structure*.  Since
+     * a data_pair_t consists of five pointers, we're wasting four
+     * pointers' worth for N=1, and will overrun what we allocated
+     * for N>5.  If anybody ever starts using size_hint, we'll need
+     * to fix this.
+     */
+    GF_ASSERT(size_hint <= (sizeof(data_pair_t) / sizeof(data_pair_t *)));
+    dict->members = mem_get0(THIS->ctx->dict_pair_pool);
+    if (!dict->members) {
+        mem_put(dict);
+        return NULL;
     }
+#endif
 
     dict->free_pair.key = NULL;
     dict->totkvlen = 0;
@@ -113,7 +124,7 @@ get_new_dict_full(int size_hint)
 dict_t *
 dict_new(void)
 {
-    dict_t *dict = get_new_dict_full(1);
+    dict_t *dict = get_new_dict_full(DICT_LIST_IMP);
 
     if (dict)
         dict_ref(dict);
@@ -351,8 +362,9 @@ dict_lookup_common(const dict_t *this, const char *key, const uint32_t hash)
     /* If the divisor is 1, the modulo is always 0,
      * in such case avoid hash calculation.
      */
-    if (this->hash_size != 1)
-        hashval = hash % this->hash_size;
+#if !DICT_LIST_IMP
+    hashval = hash % this->hash_size;
+#endif
 
     for (pair = this->members[hashval]; pair != NULL; pair = pair->hash_next) {
         if (pair->key && (hash == pair->key_hash) && !strcmp(pair->key, key))
@@ -374,7 +386,11 @@ dict_lookup(dict_t *this, char *key, data_t **data)
 
     data_pair_t *tmp = NULL;
 
-    uint32_t hash = (uint32_t)XXH64(key, strlen(key), 0);
+    uint32_t hash = 0;
+
+#if !DICT_LIST_IMP
+    hash = (uint32_t)XXH64(key, strlen(key), 0);
+#endif
 
     LOCK(&this->lock);
     {
@@ -396,7 +412,7 @@ dict_set_lk(dict_t *this, char *key, const int key_len, data_t *value,
     int hashval = 0;
     data_pair_t *pair;
     int key_free = 0;
-    uint32_t key_hash;
+    uint32_t key_hash = 0;
     int keylen;
 
     if (!key) {
@@ -405,7 +421,9 @@ dict_set_lk(dict_t *this, char *key, const int key_len, data_t *value,
             return -1;
         }
         key_free = 1;
+#if !DICT_LIST_IMP
         key_hash = (uint32_t)XXH64(key, keylen, 0);
+#endif
     } else {
         keylen = key_len;
         key_hash = hash;
@@ -458,9 +476,9 @@ dict_set_lk(dict_t *this, char *key, const int key_len, data_t *value,
     /* If the divisor is 1, the modulo is always 0,
      * in such case avoid hash calculation.
      */
-    if (this->hash_size != 1) {
-        hashval = (key_hash % this->hash_size);
-    }
+#if !DICT_LIST_IMP
+    hashval = (key_hash % this->hash_size);
+#endif
     pair->hash_next = this->members[hashval];
     this->members[hashval] = pair;
 
@@ -502,9 +520,11 @@ dict_setn(dict_t *this, char *key, const int keylen, data_t *value)
         return -1;
     }
 
+#if !DICT_LIST_IMP
     if (key) {
         key_hash = (uint32_t)XXH64(key, keylen, 0);
     }
+#endif
 
     LOCK(&this->lock);
 
@@ -536,9 +556,11 @@ dict_addn(dict_t *this, char *key, const int keylen, data_t *value)
         return -1;
     }
 
+#if !DICT_LIST_IMP
     if (key) {
         key_hash = (uint32_t)XXH64(key, keylen, 0);
     }
+#endif
 
     LOCK(&this->lock);
 
@@ -557,15 +579,18 @@ dict_get(dict_t *this, char *key)
                          "!this || key=%s", (key) ? key : "()");
         return NULL;
     }
-
+#if !DICT_LIST_IMP
     return dict_getn(this, key, strlen(key));
+#else   
+    return dict_getn(this, key, 0);
+#endif
 }
 
 data_t *
 dict_getn(dict_t *this, char *key, const int keylen)
 {
     data_pair_t *pair;
-    uint32_t hash;
+    uint32_t hash = 0;
 
     if (!this || !key) {
         gf_msg_callingfn("dict", GF_LOG_DEBUG, EINVAL, LG_MSG_INVALID_ARG,
@@ -573,7 +598,9 @@ dict_getn(dict_t *this, char *key, const int keylen)
         return NULL;
     }
 
+#if !DICT_LIST_IMP
     hash = (uint32_t)XXH64(key, keylen, 0);
+#endif
 
     LOCK(&this->lock);
     {
@@ -607,39 +634,46 @@ dict_key_count(dict_t *this)
     return ret;
 }
 
-void
+gf_boolean_t
 dict_del(dict_t *this, char *key)
 {
     if (!this || !key) {
         gf_msg_callingfn("dict", GF_LOG_WARNING, EINVAL, LG_MSG_INVALID_ARG,
                          "!this || key=%s", key);
-        return;
+        return _gf_false;
     }
-
+#if !DICT_LIST_IMP
     return dict_deln(this, key, strlen(key));
+#else
+    return dict_deln(this, key, 0);
+#endif
 }
 
-void
+gf_boolean_t
 dict_deln(dict_t *this, char *key, const int keylen)
 {
     int hashval = 0;
-    uint32_t hash;
+    uint32_t hash = 0;
+    gf_boolean_t rc = _gf_false;
 
     if (!this || !key) {
         gf_msg_callingfn("dict", GF_LOG_WARNING, EINVAL, LG_MSG_INVALID_ARG,
                          "!this || key=%s", key);
-        return;
+        return rc;
     }
 
+#if !DICT_LIST_IMP
     hash = (uint32_t)XXH64(key, keylen, 0);
+#endif
 
     LOCK(&this->lock);
 
     /* If the divisor is 1, the modulo is always 0,
      * in such case avoid hash calculation.
      */
-    if (this->hash_size != 1)
-        hashval = hash % this->hash_size;
+#if !DICT_LIST_IMP
+    hashval = hash % this->hash_size;
+#endif
 
     data_pair_t *pair = this->members[hashval];
     data_pair_t *prev = NULL;
@@ -670,6 +704,7 @@ dict_deln(dict_t *this, char *key, const int keylen)
                 mem_put(pair);
             }
             this->count--;
+            rc = _gf_true;
             break;
         }
 
@@ -679,10 +714,32 @@ dict_deln(dict_t *this, char *key, const int keylen)
 
     UNLOCK(&this->lock);
 
-    return;
+    return rc;
 }
 
-void
+/* removes and free all data_pair_t elements.
+ * has to be called with this->lock held */
+static void
+dict_clear_data(dict_t *this)
+{
+    data_pair_t *curr = this->members_list;
+    data_pair_t *next = NULL;
+
+    while (curr != NULL) {
+        next = curr->next;
+        data_unref(curr->value);
+        GF_FREE(curr->key);
+        if (curr == &this->free_pair) {
+            this->free_pair.key = NULL;
+        } else {
+            mem_put(curr);
+        }
+        curr = next;
+    }
+    this->count = this->totkvlen = 0;
+}
+
+static void
 dict_destroy(dict_t *this)
 {
     if (!this) {
@@ -691,28 +748,13 @@ dict_destroy(dict_t *this)
         return;
     }
 
-    data_pair_t *pair = this->members_list;
-    data_pair_t *prev = this->members_list;
     glusterfs_ctx_t *ctx = NULL;
     uint64_t current_max = 0;
-    uint32_t total_pairs = 0;
+    uint32_t total_pairs = this->count;
 
     LOCK_DESTROY(&this->lock);
 
-    while (prev) {
-        pair = pair->next;
-        data_unref(prev->value);
-        GF_FREE(prev->key);
-        if (prev != &this->free_pair) {
-            mem_put(prev);
-        } else {
-            this->free_pair.key = NULL;
-        }
-        total_pairs++;
-        prev = pair;
-    }
-
-    this->totkvlen = 0;
+    dict_clear_data(this);
     if (this->members != &this->members_internal) {
         mem_put(this->members);
     }
@@ -1466,7 +1508,16 @@ dict_reset(dict_t *dict)
                          "dict is NULL");
         goto out;
     }
-    dict_foreach(dict, dict_remove_foreach_fn, NULL);
+
+    int i;
+    LOCK(&dict->lock);
+
+    dict_clear_data(dict);
+    for (i = 0; i < dict->hash_size; i++)
+        dict->members[i] = NULL;
+    dict->members_list = NULL;
+
+    UNLOCK(&dict->lock);
     ret = 0;
 out:
     return ret;
@@ -1506,9 +1557,11 @@ dict_get_with_refn(dict_t *this, char *key, const int keylen, data_t **data)
 {
     data_pair_t *pair = NULL;
     int ret = -ENOENT;
-    uint32_t hash;
+    uint32_t hash = 0;
 
+#if !DICT_LIST_IMP
     hash = (uint32_t)XXH64(key, keylen, 0);
+#endif
 
     LOCK(&this->lock);
     {
@@ -1527,13 +1580,26 @@ dict_get_with_refn(dict_t *this, char *key, const int keylen, data_t **data)
 int
 dict_get_with_ref(dict_t *this, char *key, data_t **data)
 {
-    if (!this || !key || !data) {
-        gf_msg_callingfn("dict", GF_LOG_WARNING, EINVAL, LG_MSG_INVALID_ARG,
-                         "dict OR key (%s) is NULL", key);
+    if (caa_unlikely(!this)) {
+        /* There are possibilities where dict can be NULL. so not so critical
+         * for users to know */
+        gf_msg_callingfn("dict", GF_LOG_DEBUG, EINVAL, LG_MSG_INVALID_ARG,
+                         "dict is NULL: %s", key);
         return -EINVAL;
     }
+    if (caa_unlikely(!key || !data)) {
+        /* is there a chance like this? then it can be a coding error, but in
+         * any case, this should be caught early, so good to log it out. */
+        gf_msg_callingfn("dict", GF_LOG_WARNING, EINVAL, LG_MSG_INVALID_ARG,
+                         "key (%s) or data ptr is NULL", key);
 
+        return -EINVAL;
+    }
+#if !DICT_LIST_IMP
     return dict_get_with_refn(this, key, strlen(key), data);
+#else
+    return dict_get_with_refn(this, key, 0, data);
+#endif
 }
 
 static int
@@ -2108,7 +2174,7 @@ _dict_modify_flag(dict_t *this, char *key, int flag, int op)
     data_pair_t *pair = NULL;
     char *ptr = NULL;
     int hashval = 0;
-    uint32_t hash;
+    uint32_t hash = 0;
 
     if (!this || !key) {
         gf_msg_callingfn("dict", GF_LOG_WARNING, EINVAL, LG_MSG_INVALID_ARG,
@@ -2123,7 +2189,10 @@ _dict_modify_flag(dict_t *this, char *key, int flag, int op)
      */
     GF_ASSERT(flag >= 0 && flag < DICT_MAX_FLAGS);
 
+#if !DICT_LIST_IMP
     hash = (uint32_t)XXH64(key, strlen(key), 0);
+#endif
+
     LOCK(&this->lock);
     {
         pair = dict_lookup_common(this, key, hash);
@@ -2181,7 +2250,9 @@ _dict_modify_flag(dict_t *this, char *key, int flag, int op)
             pair->key_hash = hash;
             pair->value = data_ref(data);
             this->totkvlen += (strlen(key) + 1 + data->len);
+#if !DICT_LIST_IMP
             hashval = hash % this->hash_size;
+#endif
             pair->hash_next = this->members[hashval];
             this->members[hashval] = pair;
 
@@ -2847,9 +2918,9 @@ dict_rename_key(dict_t *this, char *key, char *replace_key)
 {
     data_pair_t *pair = NULL;
     int ret = -EINVAL;
-    uint32_t hash;
-    uint32_t replacekey_hash;
-    int replacekey_len;
+    uint32_t hash = 0;
+    uint32_t replacekey_hash = 0;
+    int replacekey_len = 0;
 
     /* replacing a key by itself is a NO-OP */
     if (strcmp(key, replace_key) == 0)
@@ -2861,9 +2932,11 @@ dict_rename_key(dict_t *this, char *key, char *replace_key)
         return ret;
     }
 
-    hash = (uint32_t)XXH64(key, strlen(key), 0);
     replacekey_len = strlen(replace_key);
+#if !DICT_LIST_IMP
+    hash = (uint32_t)XXH64(key, strlen(key), 0);
     replacekey_hash = (uint32_t)XXH64(replace_key, replacekey_len, 0);
+#endif
 
     LOCK(&this->lock);
     {
@@ -3481,7 +3554,9 @@ dict_has_key_from_array(dict_t *dict, char **strings, gf_boolean_t *result)
     LOCK(&dict->lock);
     {
         for (i = 0; strings[i]; i++) {
+#if !DICT_LIST_IMP
             hash = (uint32_t)XXH64(strings[i], strlen(strings[i]), 0);
+#endif
             if (dict_lookup_common(dict, strings[i], hash)) {
                 *result = _gf_true;
                 goto unlock;
