@@ -462,22 +462,26 @@ fd_destroy(fd_t *fd, gf_boolean_t bound)
         for (i = 0; i < fd->xl_count; i++) {
             if (fd->_ctx[i].key) {
                 xl = fd->_ctx[i].xl_key;
-                old_THIS = THIS;
-                THIS = xl;
-                if (!xl->call_cleanup && xl->cbks->releasedir)
+                if (!xl->call_cleanup && xl->cbks->releasedir) {
+                    if (!old_THIS)
+                        old_THIS = THIS;
+                    THIS = xl;
                     xl->cbks->releasedir(xl, fd);
-                THIS = old_THIS;
+                    THIS = old_THIS;
+                }
             }
         }
     } else {
         for (i = 0; i < fd->xl_count; i++) {
             if (fd->_ctx[i].key) {
                 xl = fd->_ctx[i].xl_key;
-                old_THIS = THIS;
-                THIS = xl;
-                if (!xl->call_cleanup && xl->cbks->release)
+                if (!xl->call_cleanup && xl->cbks->release) {
+                    if (!old_THIS)
+                        old_THIS = THIS;
+                    THIS = xl;
                     xl->cbks->release(xl, fd);
-                THIS = old_THIS;
+                    THIS = old_THIS;
+                }
             }
         }
     }
@@ -501,6 +505,32 @@ out:
     return;
 }
 
+void
+fd_close(fd_t *fd)
+{
+    xlator_t *xl, *old_THIS;
+
+    old_THIS = THIS;
+
+    for (xl = fd->inode->table->xl->graph->first; xl != NULL; xl = xl->next) {
+        if (!xl->call_cleanup) {
+            THIS = xl;
+
+            if (IA_ISDIR(fd->inode->ia_type)) {
+                if (xl->cbks->fdclosedir != NULL) {
+                    xl->cbks->fdclosedir(xl, fd);
+                }
+            } else {
+                if (xl->cbks->fdclose != NULL) {
+                    xl->cbks->fdclose(xl, fd);
+                }
+            }
+        }
+    }
+
+    THIS = old_THIS;
+}
+
 void
 fd_unref(fd_t *fd)
 {
