@@ -43,6 +43,9 @@
 #define GF_YES 1
 #define GF_NO 0
 
+#define IS_ERROR(ret) ((ret) < 0)
+#define IS_SUCCESS(ret) ((ret) >= 0)
+
 #ifndef O_LARGEFILE
 /* savannah bug #20053, patch for compiling on darwin */
 #define O_LARGEFILE 0100000 /* from bits/fcntl.h */
@@ -78,7 +81,8 @@
 #define GLUSTERD_MAX_SNAP_NAME 255
 #define GLUSTERFS_SOCKET_LISTEN_BACKLOG 1024
 #define GLUSTERD_BRICK_SERVERS "cluster.brick-vol-servers"
-#define SLEN(str) (sizeof(str) - 1)
+#define GLUSTER_STR_MUST_BE_LITERAL(__x) "" __x
+#define SLEN(str) (sizeof(GLUSTER_STR_MUST_BE_LITERAL(str)) - 1)
 
 #define ZR_MOUNTPOINT_OPT "mountpoint"
 #define ZR_ATTR_TIMEOUT_OPT "attribute-timeout"
@@ -210,6 +214,13 @@ enum gf_internal_fop_indicator {
 #define GF_XATTR_XTIME_PATTERN "trusted.glusterfs.*.xtime"
 #define GF_XATTR_TRIGGER_SYNC "glusterfs.geo-rep.trigger-sync"
 
+#define GLUSTERFS_ENTRYLK_COUNT_BIT 0
+#define GLUSTERFS_INODELK_COUNT_BIT 1
+#define GLUSTERFS_INODELK_DOM_COUNT_BIT 2
+#define GLUSTERFS_POSIXLK_COUNT_BIT 3
+#define GLUSTERFS_PARENT_ENTRYLK_BIT 4
+#define GLUSTERFS_MULTIPLE_DOM_LK_CNT_REQUESTS_BIT 5
+
 /* quota xattrs */
 #define QUOTA_SIZE_KEY "trusted.glusterfs.quota.size"
 #define QUOTA_LIMIT_KEY "trusted.glusterfs.quota.limit-set"
@@ -247,12 +258,7 @@ enum gf_internal_fop_indicator {
 #define GF_XATTROP_PURGE_INDEX "glusterfs.xattrop-purge-index"
 
 #define GF_GFIDLESS_LOOKUP "gfidless-lookup"
-/* replace-brick and pump related internal xattrs */
-#define RB_PUMP_CMD_START "glusterfs.pump.start"
-#define RB_PUMP_CMD_PAUSE "glusterfs.pump.pause"
-#define RB_PUMP_CMD_COMMIT "glusterfs.pump.commit"
-#define RB_PUMP_CMD_ABORT "glusterfs.pump.abort"
-#define RB_PUMP_CMD_STATUS "glusterfs.pump.status"
+#define GF_UNLINKED_LOOKUP "unlinked-lookup"
 
 #define GLUSTERFS_MARKER_DONT_ACCOUNT_KEY "glusters.marker.dont-account"
 #define GLUSTERFS_RDMA_INLINE_THRESHOLD (2048)
@@ -309,6 +315,7 @@ enum gf_internal_fop_indicator {
 #define DHT_SKIP_OPEN_FD_UNLINK "dont-unlink-for-open-fd"
 #define DHT_IATT_IN_XDATA_KEY "dht-get-iatt-in-xattr"
 #define DHT_MODE_IN_XDATA_KEY "dht-get-mode-in-xattr"
+#define GF_FORCE_REPLACE_KEY "force-replace"
 #define GET_LINK_COUNT "get-link-count"
 #define GF_GET_SIZE "get-size"
 #define GF_PRESTAT "virt-gf-prestat"
@@ -350,7 +357,7 @@ enum gf_internal_fop_indicator {
 #define GF_CHECK_XATTR_KEY_AND_GOTO(key, cmpkey, errval, lbl)                  \
     do {                                                                       \
         if (key && strcmp(key, cmpkey) == 0) {                                 \
-            errval = -EINVAL;                                                  \
+            errval = EINVAL;                                                   \
             goto lbl;                                                          \
         }                                                                      \
     } while (0)
@@ -422,7 +429,7 @@ static const char *const FOP_PRI_STRINGS[] = {"HIGH", "NORMAL", "LOW", "LEAST"};
 static inline const char *
 fop_pri_to_string(gf_fop_pri_t pri)
 {
-    if (pri < 0)
+    if (IS_ERROR(pri))
         return "UNSPEC";
 
     if (pri >= GF_FOP_PRI_MAX)
@@ -463,6 +470,8 @@ typedef struct _server_cmdline server_cmdline_t;
 #define GF_OPTION_DISABLE _gf_false
 #define GF_OPTION_DEFERRED 2
 
+typedef enum { _gf_none, _gf_memcheck, _gf_drd } gf_valgrind_tool;
+
 struct _cmd_args {
     /* basic options */
     char *volfile_server;
@@ -555,13 +564,15 @@ struct _cmd_args {
     /* Run this process with valgrind? Might want to prevent calling
      * functions that prevent valgrind from working correctly, like
      * dlclose(). */
-    int valgrind;
+    gf_valgrind_tool vgtool;
+
     int localtime_logging;
 
     /* For the subdir mount */
     char *subdir_mount;
 
     char *process_name;
+    char *fs_display_name;
     char *event_history;
     int thin_client;
     uint32_t reader_thread_count;
@@ -577,6 +588,8 @@ struct _cmd_args {
     bool brick_mux;
 
     uint32_t fuse_dev_eperm_ratelimit_ns;
+
+    char *io_engine;
 };
 typedef struct _cmd_args cmd_args_t;
 
@@ -639,7 +652,7 @@ struct _glusterfs_ctx {
     glusterfs_graph_t *active;
 
     /* fuse or nfs (but not protocol/server) */
-    void *master;
+    void *root;
 
     /* xlator implementing MOPs for centralized logging, volfile server */
     void *mgmt;
@@ -727,7 +740,24 @@ struct _glusterfs_ctx {
     } stats;
 
     struct list_head volfile_list;
+    /* Add members to manage janitor threads for cleanup fd */
+    struct list_head janitor_fds;
+    pthread_cond_t fd_cond;
+    pthread_mutex_t fd_lock;
+    pthread_t janitor;
+    /* The variable is use to save total posix xlator count */
+    uint32_t pxl_count;
+    uint32_t diskxl_count;
 
+    /* List of posix xlator use by disk thread*/
+    struct list_head diskth_xl;
+    pthread_mutex_t xl_lock;
+    pthread_cond_t xl_cond;
+    pthread_t disk_space_check;
+
+    gf_boolean_t cleanup_starting;
+    gf_boolean_t destroy_ctx;
+    char *hostname;
     char volume_id[GF_UUID_BUF_SIZE]; /* Used only in protocol/client */
 };
 typedef struct _glusterfs_ctx glusterfs_ctx_t;
@@ -822,4 +852,5 @@ int
 glusterfs_read_secure_access_file(void);
 int
 glusterfs_graph_fini(glusterfs_graph_t *graph);
+
 #endif /* _GLUSTERFS_H */
