@@ -11,6 +11,14 @@
 #include "glusterfs/syncop.h"
 #include "glusterfs/libglusterfs-messages.h"
 
+#ifdef HAVE_TSAN_API
+#include <sanitizer/tsan_interface.h>
+#endif
+
+#ifdef HAVE_VALGRIND_API
+#include <valgrind/valgrind.h>
+#endif
+
 int
 syncopctx_setfsuid(void *uid)
 {
@@ -262,6 +270,11 @@ synctask_yield(struct synctask *task, struct timespec *delta)
     if (task->state != SYNCTASK_DONE) {
         task->state = SYNCTASK_SUSPEND;
     }
+
+#ifdef HAVE_TSAN_API
+    __tsan_switch_to_fiber(task->proc->tsan.fiber, 0);
+#endif
+
     if (swapcontext(&task->ctx, &task->proc->sched) < 0) {
         gf_msg("syncop", GF_LOG_ERROR, errno, LG_MSG_SWAPCONTEXT_FAILED,
                "swapcontext failed");
@@ -288,6 +301,24 @@ synctask_sleep(int32_t secs)
     }
 }
 
+void
+synctask_usleep(int32_t usecs)
+{
+    struct timespec delta;
+    struct synctask *task;
+
+    task = synctask_get();
+
+    if (task == NULL) {
+        usleep(usecs);
+    } else {
+        delta.tv_sec = usecs / 1000000;
+        delta.tv_nsec = (usecs % 1000000) * 1000;
+
+        synctask_yield(task, &delta);
+    }
+}
+
 static void
 __synctask_wake(struct synctask *task)
 {
@@ -349,7 +380,7 @@ synctask_destroy(struct synctask *task)
 
     GF_FREE(task->stack);
 
-    if (task->opframe)
+    if (task->opframe && (task->opframe != task->frame))
         STACK_DESTROY(task->opframe->root);
 
     if (task->synccbk == NULL) {
@@ -357,6 +388,14 @@ synctask_destroy(struct synctask *task)
         pthread_cond_destroy(&task->cond);
     }
 
+#ifdef HAVE_TSAN_API
+    __tsan_destroy_fiber(task->tsan.fiber);
+#endif
+
+#ifdef HAVE_VALGRIND_API
+    VALGRIND_STACK_DEREGISTER(task->stackid);
+#endif
+
     GF_FREE(task);
 }
 
@@ -428,7 +467,7 @@ synctask_create(struct syncenv *env, size_t stacksize, synctask_fn_t fn,
         set_lk_owner_from_ptr(&newtask->opframe->root->lk_owner,
                               newtask->opframe->root);
     } else {
-        newtask->opframe = copy_frame(frame);
+        newtask->opframe = frame;
     }
     if (!newtask->opframe)
         goto err;
@@ -467,6 +506,19 @@ synctask_create(struct syncenv *env, size_t stacksize, synctask_fn_t fn,
 
     makecontext(&newtask->ctx, (void (*)(void))synctask_wrap, 0);
 
+#ifdef HAVE_TSAN_API
+    newtask->tsan.fiber = __tsan_create_fiber(0);
+    snprintf(newtask->tsan.name, TSAN_THREAD_NAMELEN, "<synctask of %s>",
+             this->name);
+    __tsan_set_fiber_name(newtask->tsan.fiber, newtask->tsan.name);
+#endif
+
+#ifdef HAVE_VALGRIND_API
+    newtask->stackid = VALGRIND_STACK_REGISTER(
+        newtask->ctx.uc_stack.ss_sp,
+        newtask->ctx.uc_stack.ss_sp + newtask->ctx.uc_stack.ss_size);
+#endif
+
     newtask->state = SYNCTASK_INIT;
 
     newtask->slept = 1;
@@ -483,7 +535,7 @@ synctask_create(struct syncenv *env, size_t stacksize, synctask_fn_t fn,
 err:
     if (newtask) {
         GF_FREE(newtask->stack);
-        if (newtask->opframe)
+        if (newtask->opframe && (newtask->opframe != newtask->frame))
             STACK_DESTROY(newtask->opframe->root);
         GF_FREE(newtask);
     }
@@ -573,7 +625,7 @@ syncenv_task(struct syncproc *proc)
 
             env->procs_idle++;
 
-            sleep_till.tv_sec = time(NULL) + SYNCPROC_IDLE_TIME;
+            sleep_till.tv_sec = gf_time() + SYNCPROC_IDLE_TIME;
             ret = pthread_cond_timedwait(&env->cond, &env->mutex, &sleep_till);
 
             env->procs_idle--;
@@ -638,6 +690,10 @@ synctask_switchto(struct synctask *task)
     task->ctx.uc_flags &= ~_UC_TLSBASE;
 #endif
 
+#ifdef HAVE_TSAN_API
+    __tsan_switch_to_fiber(task->tsan.fiber, 0);
+#endif
+
     if (swapcontext(&task->proc->sched, &task->ctx) < 0) {
         gf_msg("syncop", GF_LOG_ERROR, errno, LG_MSG_SWAPCONTEXT_FAILED,
                "swapcontext failed");
@@ -667,6 +723,27 @@ synctask_switchto(struct synctask *task)
     pthread_mutex_unlock(&env->mutex);
 }
 
+#ifdef HAVE_VALGRIND_API
+
+static unsigned
+__valgrind_register_current_stack(void)
+{
+    pthread_attr_t attr;
+    size_t stacksize;
+    void *stack;
+    int ret;
+
+    ret = pthread_getattr_np(pthread_self(), &attr);
+    GF_ASSERT(ret == 0);
+
+    ret = pthread_attr_getstack(&attr, &stack, &stacksize);
+    GF_ASSERT(ret == 0);
+
+    return VALGRIND_STACK_REGISTER(stack, stack + stacksize);
+}
+
+#endif /* HAVE_VALGRIND_API */
+
 void *
 syncenv_processor(void *thdata)
 {
@@ -675,10 +752,29 @@ syncenv_processor(void *thdata)
 
     proc = thdata;
 
+#ifdef HAVE_TSAN_API
+    proc->tsan.fiber = __tsan_create_fiber(0);
+    snprintf(proc->tsan.name, TSAN_THREAD_NAMELEN, "<sched of syncenv@%p>",
+             proc);
+    __tsan_set_fiber_name(proc->tsan.fiber, proc->tsan.name);
+#endif
+
+#ifdef HAVE_VALGRIND_API
+    proc->stackid = __valgrind_register_current_stack();
+#endif
+
     while ((task = syncenv_task(proc)) != NULL) {
         synctask_switchto(task);
     }
 
+#ifdef HAVE_TSAN_API
+    __tsan_destroy_fiber(proc->tsan.fiber);
+#endif
+
+#ifdef HAVE_VALGRIND_API
+    VALGRIND_STACK_DEREGISTER(proc->stackid);
+#endif
+
     return NULL;
 }
 
@@ -3055,12 +3151,13 @@ syncop_seek(xlator_t *subvol, fd_t *fd, off_t offset, gf_seek_what_t what,
     SYNCOP(subvol, (&args), syncop_seek_cbk, subvol->fops->seek, fd, offset,
            what, xdata_in);
 
-    if (*off)
-        *off = args.offset;
-
-    if (args.op_ret == -1)
+    if (args.op_ret < 0) {
         return -args.op_errno;
-    return args.op_ret;
+    } else {
+        if (off)
+            *off = args.offset;
+        return args.op_ret;
+    }
 }
 
 int
