@@ -97,7 +97,7 @@ call_bail(void *data)
     struct saved_frame *saved_frame = NULL;
     struct saved_frame *trav = NULL;
     struct saved_frame *tmp = NULL;
-    char frame_sent[256] = {
+    char frame_sent[GF_TIMESTR_SIZE] = {
         0,
     };
     struct timespec timeout = {
@@ -105,7 +105,6 @@ call_bail(void *data)
     };
     char peerid[UNIX_PATH_MAX] = {0};
     gf_boolean_t need_unref = _gf_false;
-    int len;
 
     GF_VALIDATE_OR_GOTO("client", data, out);
 
@@ -165,15 +164,14 @@ call_bail(void *data)
 
     list_for_each_entry_safe(trav, tmp, &list, list)
     {
-        gf_time_fmt(frame_sent, sizeof frame_sent, trav->saved_at.tv_sec,
-                    gf_timefmt_FT);
-        len = strlen(frame_sent);
-        snprintf(frame_sent + len, sizeof(frame_sent) - len,
-                 ".%" GF_PRI_SUSECONDS, trav->saved_at.tv_usec);
+        gf_time_fmt_tv(frame_sent, sizeof frame_sent, &trav->saved_at,
+                       gf_timefmt_FT);
 
         gf_log(conn->name, GF_LOG_ERROR,
                "bailing out frame type(%s), op(%s(%d)), xid = 0x%x, "
-               "unique = %" PRIu64 ", sent = %s, timeout = %d for %s",
+               "unique = %" PRIu64
+               ", sent = %s, timeout = %ld "
+               "for %s",
                trav->rpcreq->prog->progname,
                (trav->rpcreq->prog->procnames)
                    ? trav->rpcreq->prog->procnames[trav->rpcreq->procnum]
@@ -317,20 +315,15 @@ saved_frames_unwind(struct saved_frames *saved_frames)
 {
     struct saved_frame *trav = NULL;
     struct saved_frame *tmp = NULL;
-    char timestr[1024] = {
+    char timestr[GF_TIMESTR_SIZE] = {
         0,
     };
-    int len;
 
     list_splice_init(&saved_frames->lk_sf.list, &saved_frames->sf.list);
 
     list_for_each_entry_safe(trav, tmp, &saved_frames->sf.list, list)
     {
-        gf_time_fmt(timestr, sizeof timestr, trav->saved_at.tv_sec,
-                    gf_timefmt_FT);
-        len = strlen(timestr);
-        snprintf(timestr + len, sizeof(timestr) - len, ".%" GF_PRI_SUSECONDS,
-                 trav->saved_at.tv_usec);
+        gf_time_fmt_tv(timestr, sizeof timestr, &trav->saved_at, gf_timefmt_FT);
 
         if (!trav->rpcreq || !trav->rpcreq->prog)
             continue;
@@ -376,19 +369,20 @@ rpc_clnt_reconnect(void *conn_ptr)
     struct timespec ts = {0, 0};
     struct rpc_clnt *clnt = NULL;
     gf_boolean_t need_unref = _gf_false;
+    gf_boolean_t canceled_unref = _gf_false;
 
     conn = conn_ptr;
     clnt = conn->rpc_clnt;
-
     pthread_mutex_lock(&conn->lock);
     {
         trans = conn->trans;
-        if (!trans) {
-            pthread_mutex_unlock(&conn->lock);
-            return;
+        if (!trans)
+            goto out_unlock;
+
+        if (conn->reconnect) {
+            if (!gf_timer_call_cancel(clnt->ctx, conn->reconnect))
+                canceled_unref = _gf_true;
         }
-        if (conn->reconnect)
-            gf_timer_call_cancel(clnt->ctx, conn->reconnect);
         conn->reconnect = 0;
 
         if ((conn->connected == 0) && !clnt->disabled) {
@@ -409,11 +403,14 @@ rpc_clnt_reconnect(void *conn_ptr)
             gf_log(conn->name, GF_LOG_TRACE, "breaking reconnect chain");
         }
     }
+out_unlock:
     pthread_mutex_unlock(&conn->lock);
 
     rpc_clnt_unref(clnt);
     if (need_unref)
         rpc_clnt_unref(clnt);
+    if (canceled_unref)
+        rpc_clnt_unref(clnt);
     return;
 }
 
@@ -924,7 +921,7 @@ rpc_clnt_notify(rpc_transport_t *trans, void *mydata,
         }
 
         case RPC_TRANSPORT_MSG_RECEIVED: {
-            clock_gettime(CLOCK_REALTIME, &conn->last_received);
+            timespec_now_realtime(&conn->last_received);
 
             pollin = data;
             if (pollin->is_reply)
@@ -938,8 +935,7 @@ rpc_clnt_notify(rpc_transport_t *trans, void *mydata,
         }
 
         case RPC_TRANSPORT_MSG_SENT: {
-            clock_gettime(CLOCK_REALTIME, &conn->last_sent);
-
+            timespec_now_realtime(&conn->last_sent);
             ret = 0;
             break;
         }
@@ -956,6 +952,7 @@ rpc_clnt_notify(rpc_transport_t *trans, void *mydata,
                 conn->config.remote_port = 0;
                 conn->connected = 1;
                 conn->disconnected = 0;
+                pthread_cond_broadcast(&conn->cond);
             }
             pthread_mutex_unlock(&conn->lock);
 
@@ -1001,6 +998,7 @@ rpc_clnt_connection_init(struct rpc_clnt *clnt, glusterfs_ctx_t *ctx,
 
     conn = &clnt->conn;
     pthread_mutex_init(&clnt->conn.lock, NULL);
+    pthread_cond_init(&clnt->conn.cond, NULL);
 
     conn->name = gf_strdup(name);
     if (!conn->name) {
@@ -1008,9 +1006,9 @@ rpc_clnt_connection_init(struct rpc_clnt *clnt, glusterfs_ctx_t *ctx,
         goto out;
     }
 
-    ret = dict_get_int32(options, "frame-timeout", &conn->frame_timeout);
+    ret = dict_get_time(options, "frame-timeout", &conn->frame_timeout);
     if (ret >= 0) {
-        gf_log(name, GF_LOG_INFO, "setting frame-timeout to %d",
+        gf_log(name, GF_LOG_INFO, "setting frame-timeout to %ld",
                conn->frame_timeout);
     } else {
         gf_log(name, GF_LOG_DEBUG, "defaulting frame-timeout to 30mins");
@@ -1018,9 +1016,9 @@ rpc_clnt_connection_init(struct rpc_clnt *clnt, glusterfs_ctx_t *ctx,
     }
     conn->rpc_clnt = clnt;
 
-    ret = dict_get_int32(options, "ping-timeout", &conn->ping_timeout);
+    ret = dict_get_time(options, "ping-timeout", &conn->ping_timeout);
     if (ret >= 0) {
-        gf_log(name, GF_LOG_DEBUG, "setting ping-timeout to %d",
+        gf_log(name, GF_LOG_DEBUG, "setting ping-timeout to %ld",
                conn->ping_timeout);
     } else {
         /*TODO: Once the epoll thread model is fixed,
@@ -1615,6 +1613,7 @@ rpc_clnt_submit(struct rpc_clnt *rpc, rpc_clnt_prog_t *prog, int procnum,
         0,
     };
     struct rpc_req *rpcreq = NULL;
+    struct rpc_req rpcreq_static;
     rpc_transport_req_t req;
     int ret = -1;
     int proglen = 0;
@@ -1640,6 +1639,7 @@ rpc_clnt_submit(struct rpc_clnt *rpc, rpc_clnt_prog_t *prog, int procnum,
     if (!iobref) {
         iobref = iobref_new();
         if (!iobref) {
+            ret = -1;
             goto out;
         }
 
@@ -1754,9 +1754,15 @@ out:
     }
 
     if (frame && (ret == -1)) {
-        if (rpcreq) {
-            rpcreq->rpc_status = -1;
-            cbkfn(rpcreq, NULL, 0, frame);
+        if (!rpcreq) {
+            memset(&rpcreq_static, 0,
+                   sizeof(rpcreq_static)); /* To handle frame destroy in case
+                                              rpcreq was not defined */
+            rpcreq = &rpcreq_static;
+        }
+        rpcreq->rpc_status = -1;
+        cbkfn(rpcreq, NULL, 0, frame);
+        if (rpcreq != &rpcreq_static) {
             mem_put(rpcreq);
         }
     }
@@ -1826,6 +1832,7 @@ rpc_clnt_destroy(struct rpc_clnt *rpc)
     saved_frames_destroy(saved_frames);
     pthread_mutex_destroy(&rpc->lock);
     pthread_mutex_destroy(&rpc->conn.lock);
+    pthread_cond_destroy(&rpc->conn.cond);
 
     /* mem-pool should be destroyed, otherwise,
        it will cause huge memory leaks */
@@ -1858,7 +1865,7 @@ rpc_clnt_unref(struct rpc_clnt *rpc)
     return rpc;
 }
 
-void
+int
 rpc_clnt_disable(struct rpc_clnt *rpc)
 {
     rpc_clnt_connection_t *conn = NULL;
@@ -1902,8 +1909,9 @@ rpc_clnt_disable(struct rpc_clnt *rpc)
     }
     pthread_mutex_unlock(&conn->lock);
 
+    ret = -1;
     if (trans) {
-        rpc_transport_disconnect(trans, _gf_true);
+        ret = rpc_transport_disconnect(trans, _gf_true);
         /* The auth_value was being reset to AUTH_GLUSTERFS_v2.
          *    if (clnt->auth_value)
          *           clnt->auth_value = AUTH_GLUSTERFS_v2;
@@ -1919,7 +1927,6 @@ rpc_clnt_disable(struct rpc_clnt *rpc)
          * on a connected transport and hence its strictly serialized.
          */
     }
-
     if (unref)
         rpc_clnt_unref(rpc);
 
@@ -1930,7 +1937,7 @@ rpc_clnt_disable(struct rpc_clnt *rpc)
         rpc_clnt_unref(rpc);
 
 out:
-    return;
+    return ret;
 }
 
 void
@@ -1939,7 +1946,7 @@ rpc_clnt_reconfig(struct rpc_clnt *rpc, struct rpc_clnt_config *config)
     if (config->ping_timeout) {
         if (config->ping_timeout != rpc->conn.ping_timeout)
             gf_log(rpc->conn.name, GF_LOG_INFO,
-                   "changing ping timeout to %d (from %d)",
+                   "changing ping timeout to %ld (from %ld)",
                    config->ping_timeout, rpc->conn.ping_timeout);
 
         pthread_mutex_lock(&rpc->conn.lock);
@@ -1952,7 +1959,7 @@ rpc_clnt_reconfig(struct rpc_clnt *rpc, struct rpc_clnt_config *config)
     if (config->rpc_timeout) {
         if (config->rpc_timeout != rpc->conn.config.rpc_timeout)
             gf_log(rpc->conn.name, GF_LOG_INFO,
-                   "changing timeout to %d (from %d)", config->rpc_timeout,
+                   "changing timeout to %ld (from %ld)", config->rpc_timeout,
                    rpc->conn.config.rpc_timeout);
         rpc->conn.config.rpc_timeout = config->rpc_timeout;
     }
