@@ -79,7 +79,7 @@ function tar_logs()
         then
                 if [[ $skip_preserve_logs == "yes" ]]; then
                         savetarname=$(get_next_filename "${logdir}" \
-                                "${basetarname}-iteration" "tar" \
+                                "${basetarname}-iteration" "tar.gz" \
                                 | tail -1)
                 else
                         savetarname="$basetarname"
@@ -88,15 +88,15 @@ function tar_logs()
                 # Can't use --exclude here because NetBSD doesn't have it.
                 # However, both it and Linux have -X to take patterns from
                 # a file, so use that.
-                (echo '*.tar'; echo .notar) > "${logdir}"/.notar \
+                (echo '*.tar.gz'; echo .notar) > "${logdir}"/.notar \
                         && \
-                tar -cf "${logdir}"/"${savetarname}".tar -X "${logdir}"/.notar \
+                tar -czf "${logdir}"/"${savetarname}".tar.gz -X "${logdir}"/.notar \
                         "${logdir}"/* 2> /dev/null \
                         && \
                 find "$logdir"/* -maxdepth 0 -name '*.tar' -prune \
                                         -o -exec rm -rf '{}' ';'
 
-                echo "Logs preserved in tarball $savetarname.tar"
+                echo "Logs preserved in tarball ${savetarname}.tar.gz"
         else
                 echo "Logs not preserved, as logdir is not set"
         fi
@@ -170,6 +170,12 @@ function check_dependencies()
         MISSING="$MISSING netstat"
     fi
 
+    # Check for killall
+    env killall --version > /dev/null 2>&1
+    if [ $? -ne 0 ]; then
+        MISSING="$MISSING killall"
+    fi
+
     # check for psutil python package
     test `uname -s` == "Darwin" || test `uname -s` == "FreeBSD" && {
         pip show psutil | grep -q psutil >/dev/null 2>&1
@@ -265,6 +271,7 @@ function match()
 # G_TESTDEF_TEST_STATUS_NETBSD7
 # Some examples:
 # G_TESTDEF_TEST_STATUS_CENTOS6=BAD_TEST,BUG=123456
+# G_TESTDEF_TEST_STATUS_CENTOS6=BRICK_MUX_BAD_TEST,BUG=123456
 # G_TESTDEF_TEST_STATUS_NETBSD7=KNOWN_ISSUE,BUG=4444444
 # G_TESTDEF_TEST_STATUS_CENTOS6=BAD_TEST,BUG=123456;555555
 # G_TESTDEF_TEST_STATUS_CENTOS6=NFS_TESTS,BUG=1385758
@@ -327,6 +334,7 @@ function get_bug_list_for_disabled_test ()
 function run_tests()
 {
     RES=0
+    FLAKY=''
     FAILED=''
     TESTS_NEEDED_RETRY=''
     GENERATED_CORE=''
@@ -348,15 +356,16 @@ function run_tests()
         timeout_cmd_exists="no"
     fi
 
-    for t in $(find ${regression_testsdir}/tests -name '*.t' \
-               | LC_COLLATE=C sort) ; do
+    all_tests=($(find ${regression_testsdir}/tests -name '*.t' | sort))
+    all_tests_cnt=${#all_tests[@]}
+    for t in "${all_tests[@]}" ; do
         old_cores=$(ls /*-*.core 2> /dev/null | wc -l)
         total_tests=$((total_tests+1))
         if match $t "$@" ; then
             selected_tests=$((selected_tests+1))
             echo
-            echo $section_separator$section_separator
-            if [[ $(get_test_status $t) == "BAD_TEST" ]] && \
+            echo $section_separator "(${total_tests} / ${all_tests_cnt})" $section_separator
+            if [[ $(get_test_status $t) =~ "BAD_TEST" ]] && \
                [[ $skip_bad_tests == "yes" ]]
             then
                 skipped_bad_tests=$((skipped_bad_tests+1))
@@ -394,7 +403,7 @@ function run_tests()
                     cmd_timeout=$(grep "SCRIPT_TIMEOUT=" ${t} | cut -f2 -d'=');
                     echo "Timeout set is ${cmd_timeout}, default ${run_timeout}"
                 fi
-                timeout -k ${kill_after_time} ${cmd_timeout} prove -vmfe '/bin/bash' ${t}
+                timeout --foreground -k ${kill_after_time} ${cmd_timeout} prove -vmfe '/bin/bash' ${t}
             else
                 prove -vmfe '/bin/bash' ${t}
             fi
@@ -418,7 +427,7 @@ function run_tests()
                 echo ""
 
                 if [ ${timeout_cmd_exists} == "yes" ]; then
-                    timeout -k ${kill_after_time} ${cmd_timeout} prove -vmfe '/bin/bash' ${t}
+                    timeout --foreground -k ${kill_after_time} ${cmd_timeout} prove -vmfe '/bin/bash' ${t}
                 else
                     prove -vmfe '/bin/bash' ${t}
                 fi
@@ -431,9 +440,17 @@ function run_tests()
 
                 TESTS_NEEDED_RETRY="${TESTS_NEEDED_RETRY}${t} "
             fi
+
+
             if [ ${TMP_RES} -ne 0 ] ; then
-                RES=${TMP_RES}
-                FAILED="${FAILED}${t} "
+		if [[ "$t" == *"tests/000-flaky/"* ]]; then
+                    FLAKY="${FLAKY}${t} "
+		    echo "FAILURE -> SUCCESS: Flaky test"
+		    TMP_RES=0
+		else
+                    RES=${TMP_RES}
+                    FAILED="${FAILED}${t} "
+		fi
             fi
 
             new_cores=$(ls /*-*.core 2> /dev/null | wc -l)
@@ -468,8 +485,10 @@ function run_tests()
         echo "$key  -  ${ELAPSEDTIMEMAP["$key"]} second"
     done | sort -rn -k3
 
-    # Output the errors into a file
+    # initialize the output file
     echo > "${result_output}"
+
+    # Output the errors into a file
     if [ ${RES} -ne 0 ] ; then
         FAILED=$( echo ${FAILED} | tr ' ' '\n' | sort -u )
         FAILED_COUNT=$( echo -n "${FAILED}" | grep -c '^' )
@@ -482,7 +501,13 @@ function run_tests()
     TESTS_NEEDED_RETRY=$( echo ${TESTS_NEEDED_RETRY} | tr ' ' '\n' | sort -u )
     RETRY_COUNT=$( echo -n "${TESTS_NEEDED_RETRY}" | grep -c '^' )
     if [ ${RETRY_COUNT} -ne 0 ] ; then
-        echo -e "\n${RETRY_COUNT} test(s) needed retry \n${TESTS_NEEDED_RETRY}"
+        echo -e "\n${RETRY_COUNT} test(s) needed retry \n${TESTS_NEEDED_RETRY}" >> "${result_output}"
+    fi
+
+    FLAKY_TESTS_FAILED=$( echo ${FLAKY} | tr ' ' '\n' | sort -u )
+    RETRY_COUNT=$( echo -n "${FLAKY_TESTS_FAILED}" | grep -c '^' )
+    if [ ${RETRY_COUNT} -ne 0 ] ; then
+        echo -e "\n${RETRY_COUNT} flaky test(s) marked as success even though they failed \n${FLAKY_TESTS_FAILED}" >> "${result_output}"
     fi
 
     echo
@@ -511,8 +536,38 @@ function run_head_tests()
     run_tests "$htests"
 }
 
-function parse_args () {
-    args=`getopt frcbkphHno:t: "$@"`
+function show_usage ()
+{
+    cat <<EOF
+Usage: $0 <opts> [<glob>|<bzid>]...
+
+Options:
+
+-f  force
+-h  skip tests altering from HEAD
+-H  run only tests altering from HEAD
+-r  retry failed tests
+-R  do not retry failed tests
+-c  dont't exit on failure
+-b  don't skip bad tests
+-k  don't skip known bugs
+-p  don't keep logs from preceding runs
+-o  OUTPUT
+-t  TIMEOUT
+-n  skip NFS tests
+--help
+EOF
+}
+
+usage="no"
+
+function parse_args ()
+{
+    args=`getopt -u -l help frRcbkphHno:t: "$@"`
+    if ! [ $? -eq 0 ]; then
+	show_usage
+	exit 1
+    fi
     set -- $args
     while [ $# -gt 0 ]; do
         case "$1" in
@@ -520,6 +575,7 @@ function parse_args () {
         -h)    head="no" ;;
         -H)    head="only" ;;
         -r)    retry="yes" ;;
+        -R)    retry="no" ;;
         -c)    exit_on_failure="no" ;;
         -b)    skip_bad_tests="no" ;;
         -k)    skip_known_bugs="no" ;;
@@ -527,6 +583,7 @@ function parse_args () {
         -o)    result_output="$2"; shift;;
         -t)    run_timeout="$2"; shift;;
         -n)    nfs_tests="no";;
+        --help) usage="yes" ;;
         --)    shift; break;;
         esac
         shift
@@ -541,6 +598,10 @@ echo
 
 # Get user options
 parse_args "$@"
+if [ x"$usage" == x"yes" ]; then
+    show_usage
+    exit 0
+fi
 
 # Make sure we're running as the root user
 check_user
