@@ -225,24 +225,43 @@ __afr_selfheal_data_read_write(call_frame_t *frame, xlator_t *this, fd_t *fd,
     return ret;
 }
 
+static gf_boolean_t
+afr_source_sinks_locked(xlator_t *this, unsigned char *locked_on, int source,
+                        unsigned char *healed_sinks)
+{
+    afr_private_t *priv = this->private;
+    int i = 0;
+
+    if (!locked_on[source])
+        return _gf_false;
+
+    for (i = 0; i < priv->child_count; i++) {
+        if (healed_sinks[i] && locked_on[i])
+            return _gf_true;
+    }
+
+    return _gf_false;
+}
+
 static int
 afr_selfheal_data_block(call_frame_t *frame, xlator_t *this, fd_t *fd,
                         int source, unsigned char *healed_sinks, off_t offset,
                         size_t size, int type, struct afr_reply *replies)
 {
     int ret = -1;
-    int sink_count = 0;
     afr_private_t *priv = NULL;
     unsigned char *data_lock = NULL;
 
     priv = this->private;
-    sink_count = AFR_COUNT(healed_sinks, priv->child_count);
     data_lock = alloca0(priv->child_count);
 
+    gf_msg_debug(this->name, 0, "gfid:%s, offset=%jd, size=%zu",
+                 uuid_utoa(fd->inode->gfid), offset, size);
+
     ret = afr_selfheal_inodelk(frame, this, fd->inode, this->name, offset, size,
                                data_lock);
     {
-        if (ret < sink_count) {
+        if (!afr_source_sinks_locked(this, data_lock, source, healed_sinks)) {
             ret = -ENOTCONN;
             goto unlock;
         }
@@ -334,6 +353,12 @@ afr_selfheal_data_do(call_frame_t *frame, xlator_t *this, fd_t *fd, int source,
     }
 
     block = 128 * 1024 * priv->data_self_heal_window_size;
+    if (HAS_HOLES((&replies[source].poststat))) {
+        /*Reduce the possibility of data-block allocations in case of files
+         * with holes. Correct way to fix it would be to use seek fop while
+         * healing data*/
+        block = 128 * 1024;
+    }
 
     type = afr_data_self_heal_type_get(priv, healed_sinks, source, replies);
 
@@ -386,7 +411,7 @@ __afr_selfheal_truncate_sinks(call_frame_t *frame, xlator_t *this, fd_t *fd,
     /* This will send truncate on the arbiter brick as well if it is marked as
      * sink. If changelog is enabled on the volume it captures truncate as a
      * data transactions on the arbiter brick. This will help geo-rep to
-     * properly sync the data from master to slave if arbiter is the ACTIVE
+     * properly sync the data from primary to secondary if arbiter is the ACTIVE
      * brick during syncing and which had got some entries healed for data as
      * part of self heal.
      */
