@@ -710,7 +710,7 @@ dht_deitransform(xlator_t *this, uint64_t y, xlator_t **subvol_p)
 }
 
 void
-dht_local_wipe(xlator_t *this, dht_local_t *local)
+dht_local_wipe(dht_local_t *local)
 {
     int i = 0;
 
@@ -728,7 +728,7 @@ dht_local_wipe(xlator_t *this, dht_local_t *local)
         inode_unref(local->inode);
 
     if (local->layout) {
-        dht_layout_unref(this, local->layout);
+        dht_layout_unref(local->layout);
         local->layout = NULL;
     }
 
@@ -758,12 +758,12 @@ dht_local_wipe(xlator_t *this, dht_local_t *local)
         dict_unref(local->xdata);
 
     if (local->selfheal.layout) {
-        dht_layout_unref(this, local->selfheal.layout);
+        dht_layout_unref(local->selfheal.layout);
         local->selfheal.layout = NULL;
     }
 
     if (local->selfheal.refreshed_layout) {
-        dht_layout_unref(this, local->selfheal.refreshed_layout);
+        dht_layout_unref(local->selfheal.refreshed_layout);
         local->selfheal.refreshed_layout = NULL;
     }
 
@@ -950,7 +950,7 @@ dht_subvol_get_hashed(xlator_t *this, loc_t *loc)
 
 out:
     if (layout) {
-        dht_layout_unref(this, layout);
+        dht_layout_unref(layout);
     }
 
     return subvol;
@@ -975,7 +975,7 @@ dht_subvol_get_cached(xlator_t *this, inode_t *inode)
 
 out:
     if (layout) {
-        dht_layout_unref(this, layout);
+        dht_layout_unref(layout);
     }
 
     return subvol;
@@ -1380,7 +1380,7 @@ dht_migration_complete_check_task(void *data)
 
     /* update local. A layout is set in inode-ctx in lookup already */
 
-    dht_layout_unref(this, local->layout);
+    dht_layout_unref(local->layout);
 
     local->layout = dht_layout_get(frame->this, inode);
     local->cached_subvol = dst_node;
@@ -2083,6 +2083,7 @@ dht_heal_full_path_done(int op_ret, call_frame_t *heal_frame, void *data)
     dht_local_t *local = NULL;
     xlator_t *this = NULL;
     int ret = -1;
+    int op_errno = 0;
 
     local = heal_frame->local;
     main_frame = local->main_frame;
@@ -2092,11 +2093,12 @@ dht_heal_full_path_done(int op_ret, call_frame_t *heal_frame, void *data)
     dht_set_fixed_dir_stat(&local->postparent);
     if (local->need_xattr_heal) {
         local->need_xattr_heal = 0;
-        ret = dht_dir_xattr_heal(this, local);
-        if (ret)
-            gf_smsg(this->name, GF_LOG_ERROR, ret,
+        ret = dht_dir_xattr_heal(this, local, &op_errno);
+        if (ret) {
+            gf_smsg(this->name, GF_LOG_ERROR, op_errno,
                     DHT_MSG_DIR_XATTR_HEAL_FAILED, "path=%s", local->loc.path,
                     NULL);
+        }
     }
 
     DHT_STACK_UNWIND(lookup, main_frame, 0, 0, local->inode, &local->stbuf,
@@ -2265,6 +2267,7 @@ dht_dir_set_heal_xattr(xlator_t *this, dht_local_t *local, dict_t *dst,
     int luret = -1;
     int luflag = -1;
     int i = 0;
+    char **xattrs_to_heal;
 
     if (!src || !dst) {
         gf_smsg(this->name, GF_LOG_WARNING, EINVAL, DHT_MSG_DST_NULL_SET_FAILED,
@@ -2279,6 +2282,9 @@ dht_dir_set_heal_xattr(xlator_t *this, dht_local_t *local, dict_t *dst,
        and set it to dst dict, here index start from 1 because
        user xattr already checked in previous statement
     */
+
+    xattrs_to_heal = get_xattrs_to_heal();
+
     for (i = 1; xattrs_to_heal[i]; i++) {
         keyval = dict_get(src, xattrs_to_heal[i]);
         if (keyval) {
