@@ -87,7 +87,7 @@ dht_layout_set(xlator_t *this, inode_t *inode, dht_layout_t *layout)
     UNLOCK(&conf->layout_lock);
 
     if (!oldret) {
-        dht_layout_unref(this, old_layout);
+        dht_layout_unref(old_layout);
     }
     if (ret)
         GF_ATOMIC_DEC(layout->ref);
@@ -97,11 +97,11 @@ out:
 }
 
 void
-dht_layout_unref(xlator_t *this, dht_layout_t *layout)
+dht_layout_unref(dht_layout_t *layout)
 {
     int ref = 0;
 
-    if (!layout || layout->preset || !this->private)
+    if (!layout || layout->preset)
         return;
 
     ref = GF_ATOMIC_DEC(layout->ref);
@@ -131,9 +131,8 @@ dht_layout_search(xlator_t *this, dht_layout_t *layout, const char *name)
 
     ret = dht_hash_compute(this, layout->type, name, &hash);
     if (ret != 0) {
-        gf_msg(this->name, GF_LOG_WARNING, 0, DHT_MSG_COMPUTE_HASH_FAILED,
-               "hash computation failed for type=%d name=%s", layout->type,
-               name);
+        gf_smsg(this->name, GF_LOG_WARNING, 0, DHT_MSG_COMPUTE_HASH_FAILED,
+                "type=%d", layout->type, "name=%s", name, NULL);
         goto out;
     }
 
@@ -145,8 +144,8 @@ dht_layout_search(xlator_t *this, dht_layout_t *layout, const char *name)
     }
 
     if (!subvol) {
-        gf_msg(this->name, GF_LOG_WARNING, 0, DHT_MSG_HASHED_SUBVOL_GET_FAILED,
-               "no subvolume for hash (value) = %u", hash);
+        gf_smsg(this->name, GF_LOG_WARNING, 0, DHT_MSG_HASHED_SUBVOL_GET_FAILED,
+                "hash-value=0x%x", hash, NULL);
     }
 
 out:
@@ -255,7 +254,7 @@ dht_disk_layout_extract_for_subvol(xlator_t *this, dht_layout_t *layout,
     return dht_disk_layout_extract(this, layout, i, disk_layout_p);
 }
 
-int
+static int
 dht_disk_layout_merge(xlator_t *this, dht_layout_t *layout, int pos,
                       void *disk_layout_raw, int disk_layout_len)
 {
@@ -266,8 +265,8 @@ dht_disk_layout_merge(xlator_t *this, dht_layout_t *layout, int pos,
     int disk_layout[4];
 
     if (!disk_layout_raw) {
-        gf_msg(this->name, GF_LOG_CRITICAL, 0, DHT_MSG_LAYOUT_MERGE_FAILED,
-               "error no layout on disk for merge");
+        gf_smsg(this->name, GF_LOG_CRITICAL, 0, DHT_MSG_LAYOUT_MERGE_FAILED,
+                NULL);
         return -1;
     }
 
@@ -284,10 +283,8 @@ dht_disk_layout_merge(xlator_t *this, dht_layout_t *layout, int pos,
         case DHT_HASH_TYPE_DM:
             break;
         default:
-            gf_msg(this->name, GF_LOG_CRITICAL, 0, DHT_MSG_INVALID_DISK_LAYOUT,
-                   "Invalid disk layout: "
-                   "Catastrophic error layout with unknown type found %d",
-                   disk_layout[1]);
+            gf_smsg(this->name, GF_LOG_CRITICAL, 0, DHT_MSG_INVALID_DISK_LAYOUT,
+                    "layout=%d", disk_layout[1], NULL);
             return -1;
     }
 
@@ -299,9 +296,10 @@ dht_disk_layout_merge(xlator_t *this, dht_layout_t *layout, int pos,
     layout->list[pos].start = start_off;
     layout->list[pos].stop = stop_off;
 
-    gf_msg_trace(
-        this->name, 0, "merged to layout: %u - %u (type %d, hash %d) from %s",
-        start_off, stop_off, commit_hash, type, layout->list[pos].xlator->name);
+    gf_msg_trace(this->name, 0,
+                 "merged to layout: 0x%x - 0x%x (hash 0x%x, type %d) from %s",
+                 start_off, stop_off, commit_hash, type,
+                 layout->list[pos].xlator->name);
 
     return 0;
 }
@@ -354,8 +352,8 @@ dht_layout_merge(xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
     ret = dht_disk_layout_merge(this, layout, i, disk_layout_raw,
                                 disk_layout_len);
     if (ret != 0) {
-        gf_msg(this->name, GF_LOG_WARNING, 0, DHT_MSG_LAYOUT_MERGE_FAILED,
-               "layout merge from subvolume %s failed", subvol->name);
+        gf_smsg(this->name, GF_LOG_WARNING, 0, DHT_MSG_LAYOUT_MERGE_FAILED,
+                "subvolume=%s", subvol->name, NULL);
         goto out;
     }
 
@@ -371,34 +369,6 @@ out:
     return ret;
 }
 
-void
-dht_layout_entry_swap(dht_layout_t *layout, int i, int j)
-{
-    uint32_t start_swap = 0;
-    uint32_t stop_swap = 0;
-    uint32_t commit_hash_swap = 0;
-    xlator_t *xlator_swap = 0;
-    int err_swap = 0;
-
-    start_swap = layout->list[i].start;
-    stop_swap = layout->list[i].stop;
-    xlator_swap = layout->list[i].xlator;
-    err_swap = layout->list[i].err;
-    commit_hash_swap = layout->list[i].commit_hash;
-
-    layout->list[i].start = layout->list[j].start;
-    layout->list[i].stop = layout->list[j].stop;
-    layout->list[i].xlator = layout->list[j].xlator;
-    layout->list[i].err = layout->list[j].err;
-    layout->list[i].commit_hash = layout->list[j].commit_hash;
-
-    layout->list[j].start = start_swap;
-    layout->list[j].stop = stop_swap;
-    layout->list[j].xlator = xlator_swap;
-    layout->list[j].err = err_swap;
-    layout->list[j].commit_hash = commit_hash_swap;
-}
-
 void
 dht_layout_range_swap(dht_layout_t *layout, int i, int j)
 {
@@ -415,12 +385,6 @@ dht_layout_range_swap(dht_layout_t *layout, int i, int j)
     layout->list[j].stop = stop_swap;
 }
 
-int64_t
-dht_layout_entry_cmp_volname(dht_layout_t *layout, int i, int j)
-{
-    return (strcmp(layout->list[i].xlator->name, layout->list[j].xlator->name));
-}
-
 gf_boolean_t
 dht_is_subvol_in_layout(dht_layout_t *layout, xlator_t *xlator)
 {
@@ -438,60 +402,40 @@ dht_is_subvol_in_layout(dht_layout_t *layout, xlator_t *xlator)
     return _gf_false;
 }
 
-int64_t
-dht_layout_entry_cmp(dht_layout_t *layout, int i, int j)
+static int
+dht_layout_entry_cmp(const void *p, const void *q)
 {
+    const dht_layout_entry_t *x = p, *y = q;
     int64_t diff = 0;
 
-    /* swap zero'ed out layouts to front, if needed */
-    if (!layout->list[j].start && !layout->list[j].stop) {
-        diff = (int64_t)layout->list[i].stop - (int64_t)layout->list[j].stop;
-        goto out;
-    }
-    diff = (int64_t)layout->list[i].start - (int64_t)layout->list[j].start;
+    if (!y->start && !y->stop)
+        diff = (int64_t)x->stop - (int64_t)y->stop;
+    else
+        diff = (int64_t)x->start - (int64_t)y->start;
 
-out:
-    return diff;
+    return (diff == 0 ? 0 : (diff < 0 ? -1 : 1));
 }
 
-int
+static int
+dht_layout_entry_cmp_volname(const void *p, const void *q)
+{
+    const dht_layout_entry_t *x = p, *y = q;
+
+    return strcmp(x->xlator->name, y->xlator->name);
+}
+
+void
 dht_layout_sort(dht_layout_t *layout)
 {
-    int i = 0;
-    int j = 0;
-    int64_t ret = 0;
-
-    /* TODO: O(n^2) -- bad bad */
-
-    for (i = 0; i < layout->cnt - 1; i++) {
-        for (j = i + 1; j < layout->cnt; j++) {
-            ret = dht_layout_entry_cmp(layout, i, j);
-            if (ret > 0)
-                dht_layout_entry_swap(layout, i, j);
-        }
-    }
-
-    return 0;
+    qsort(layout->list, layout->cnt, sizeof(dht_layout_entry_t),
+          dht_layout_entry_cmp);
 }
 
-int
+void
 dht_layout_sort_volname(dht_layout_t *layout)
 {
-    int i = 0;
-    int j = 0;
-    int64_t ret = 0;
-
-    /* TODO: O(n^2) -- bad bad */
-
-    for (i = 0; i < layout->cnt - 1; i++) {
-        for (j = i + 1; j < layout->cnt; j++) {
-            ret = dht_layout_entry_cmp_volname(layout, i, j);
-            if (ret > 0)
-                dht_layout_entry_swap(layout, i, j);
-        }
-    }
-
-    return 0;
+    qsort(layout->list, layout->cnt, sizeof(dht_layout_entry_t),
+          dht_layout_entry_cmp_volname);
 }
 
 void
@@ -622,12 +566,7 @@ dht_layout_normalize(xlator_t *this, loc_t *loc, dht_layout_t *layout)
     uint32_t misc = 0, missing_dirs = 0;
     char gfid[GF_UUID_BUF_SIZE] = {0};
 
-    ret = dht_layout_sort(layout);
-    if (ret == -1) {
-        gf_msg(this->name, GF_LOG_WARNING, 0, DHT_MSG_LAYOUT_SORT_FAILED,
-               "sort failed?! how the ....");
-        goto out;
-    }
+    dht_layout_sort(layout);
 
     gf_uuid_unparse(loc->gfid, gfid);
 
@@ -641,10 +580,9 @@ dht_layout_normalize(xlator_t *this, loc_t *loc, dht_layout_t *layout)
                          " gfid = %s",
                          loc->path, gfid);
         } else {
-            gf_msg(this->name, GF_LOG_INFO, 0, DHT_MSG_ANOMALIES_INFO,
-                   "Found anomalies in %s (gfid = %s). "
-                   "Holes=%d overlaps=%d",
-                   loc->path, gfid, holes, overlaps);
+            gf_smsg(this->name, GF_LOG_INFO, 0, DHT_MSG_ANOMALIES_INFO,
+                    "path=%s", loc->path, "gfid=%s", gfid, "holes=%d", holes,
+                    "overlaps=%d", overlaps, NULL);
         }
         ret = -1;
     }
@@ -658,7 +596,6 @@ dht_layout_normalize(xlator_t *this, loc_t *loc, dht_layout_t *layout)
             ret += missing_dirs;
     }
 
-out:
     return ret;
 }
 
@@ -711,12 +648,11 @@ dht_layout_dir_mismatch(xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
     if (!xattr) {
         if (err == 0) {
             if (loc) {
-                gf_msg(this->name, GF_LOG_INFO, 0, DHT_MSG_DICT_GET_FAILED,
-                       "%s: xattr dictionary is NULL", loc->path);
+                gf_smsg(this->name, GF_LOG_INFO, 0, DHT_MSG_XATTR_DICT_NULL,
+                        "path=%s", loc->path, NULL);
             } else {
-                gf_msg(this->name, GF_LOG_INFO, 0, DHT_MSG_DICT_GET_FAILED,
-                       "path not found: "
-                       "xattr dictionary is NULL");
+                gf_smsg(this->name, GF_LOG_INFO, 0, DHT_MSG_XATTR_DICT_NULL,
+                        "path not found", NULL);
             }
             ret = -1;
         }
@@ -728,13 +664,13 @@ dht_layout_dir_mismatch(xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
     if (dict_ret < 0) {
         if (err == 0 && layout->list[pos].stop) {
             if (loc) {
-                gf_msg(this->name, GF_LOG_INFO, 0, DHT_MSG_DISK_LAYOUT_MISSING,
-                       "%s: Disk layout missing, gfid = %s", loc->path, gfid);
+                gf_smsg(this->name, GF_LOG_INFO, 0, DHT_MSG_DISK_LAYOUT_MISSING,
+                        "path=%s", loc->path, "gfid=%s", gfid, NULL);
             } else {
-                gf_msg(this->name, GF_LOG_INFO, 0, DHT_MSG_DISK_LAYOUT_MISSING,
-                       "path not found: "
-                       "Disk layout missing, gfid = %s",
-                       gfid);
+                gf_smsg(this->name, GF_LOG_INFO, 0, DHT_MSG_DISK_LAYOUT_MISSING,
+                        "path not found"
+                        "gfid=%s",
+                        gfid, NULL);
             }
             ret = -1;
         }
@@ -750,13 +686,13 @@ dht_layout_dir_mismatch(xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
     if ((layout->list[pos].start != start_off) ||
         (layout->list[pos].stop != stop_off) ||
         (layout->list[pos].commit_hash != commit_hash)) {
-        gf_msg(this->name, GF_LOG_INFO, 0, DHT_MSG_LAYOUT_INFO,
-               "subvol: %s; inode layout - %" PRIu32 " - %" PRIu32 " - %" PRIu32
-               "; "
-               "disk layout - %" PRIu32 " - %" PRIu32 " - %" PRIu32,
-               layout->list[pos].xlator->name, layout->list[pos].start,
-               layout->list[pos].stop, layout->list[pos].commit_hash, start_off,
-               stop_off, commit_hash);
+        gf_smsg(this->name, GF_LOG_INFO, 0, DHT_MSG_LAYOUT_INFO, "subvol=%s",
+                layout->list[pos].xlator->name, "inode-layout:start=0x%x",
+                layout->list[pos].start, "inode-layout:stop=0x%x",
+                layout->list[pos].stop, "layout-commit-hash=0x%x; ",
+                layout->list[pos].commit_hash, "disk-layout:start-off=0x%x",
+                start_off, "disk-layout:top-off=0x%x", stop_off,
+                "commit-hash=0x%x", commit_hash, NULL);
         ret = 1;
     } else {
         ret = 0;
@@ -778,9 +714,8 @@ dht_layout_preset(xlator_t *this, xlator_t *subvol, inode_t *inode)
 
     layout = dht_layout_for_subvol(this, subvol);
     if (!layout) {
-        gf_msg(this->name, GF_LOG_INFO, 0, DHT_MSG_SUBVOL_NO_LAYOUT_INFO,
-               "no pre-set layout for subvolume %s",
-               subvol ? subvol->name : "<nil>");
+        gf_smsg(this->name, GF_LOG_INFO, 0, DHT_MSG_SUBVOL_NO_LAYOUT_INFO,
+                "subvolume=%s", subvol ? subvol->name : "<nil>", NULL);
         ret = -1;
         goto out;
     }
