@@ -123,7 +123,9 @@ dht_rename_dir_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
     local = frame->local;
     prev = cookie;
     subvol_cnt = dht_subvol_cnt(this, prev);
-    local->ret_cache[subvol_cnt] = op_ret;
+
+    if (subvol_cnt >= 0)
+        local->ret_cache[subvol_cnt] = op_ret;
 
     if (op_ret == -1) {
         gf_uuid_unparse(local->loc.inode->gfid, gfid);
@@ -1210,8 +1212,8 @@ dht_rename_link_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
     prev = cookie;
 
     if (op_ret == -1) {
-        gf_msg_debug(this->name, 0, "link/file on %s failed (%s)", prev->name,
-                     strerror(op_errno));
+        gf_msg_debug(this->name, op_errno, "link/file on %s failed",
+                     prev->name);
         local->op_ret = -1;
         local->op_errno = op_errno;
         local->added_link = _gf_false;
@@ -1248,8 +1250,8 @@ dht_rename_linkto_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
     src_cached = local->src_cached;
 
     if (op_ret == -1) {
-        gf_msg_debug(this->name, 0, "link/file on %s failed (%s)", prev->name,
-                     strerror(op_errno));
+        gf_msg_debug(this->name, op_errno, "link/file on %s failed",
+                     prev->name);
         local->op_ret = -1;
         local->op_errno = op_errno;
     }
@@ -1266,6 +1268,18 @@ dht_rename_linkto_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
         DHT_MARKER_DONT_ACCOUNT(xattr);
     }
 
+    /* problem: link creation fails if a stale linkto exists for dst in
+     * the src_cached.
+     * handling: populate the FORCE_REPLACE key for "forcing" posix
+     * to unlink the stale file and repeat the link */
+    if (dict_set_str(xattr, GF_FORCE_REPLACE_KEY, "yes")) {
+        gf_msg(this->name, GF_LOG_WARNING, 0, DHT_MSG_DICT_SET_FAILED,
+               "Failed to set dictionary value: key = %s,"
+               " path = %s",
+               GF_FORCE_REPLACE_KEY, local->loc.path);
+        goto cleanup;
+    }
+
     local->added_link = _gf_true;
 
     STACK_WIND_COOKIE(frame, dht_rename_link_cbk, src_cached, src_cached,
@@ -1298,8 +1312,8 @@ dht_rename_unlink_links_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
     prev = cookie;
 
     if ((op_ret == -1) && (op_errno != ENOENT)) {
-        gf_msg_debug(this->name, 0, "unlink of %s on %s failed (%s)",
-                     local->loc2.path, prev->name, strerror(op_errno));
+        gf_msg_debug(this->name, op_errno, "unlink of %s on %s failed ",
+                     local->loc2.path, prev->name);
         local->op_ret = -1;
         local->op_errno = op_errno;
     }
@@ -1328,6 +1342,7 @@ dht_rename_create_links(call_frame_t *frame)
     xlator_t *dst_cached = NULL;
     int call_cnt = 0;
     dict_t *xattr = NULL;
+    int ret = 0;
 
     local = frame->local;
     this = frame->this;
@@ -1363,12 +1378,37 @@ dht_rename_create_links(call_frame_t *frame)
         return 0;
     }
 
+    dict_t *xattr_new = NULL;
+
     if (src_cached != dst_hashed) {
         /* needed to create the link file */
         call_cnt++;
         if (dst_hashed != src_hashed)
             /* needed to create the linkto file */
             call_cnt++;
+
+        /* handle cases of stale linkto files that cause failure to create
+         * linkto/link during rename.
+         * set the FORCE_REPLACE key for unlinking the stale file if it exists.
+         * possible cases of stale linkto:
+         * 1. stale linkto for src is exists in the dst_hash
+         * 2. stale linkto for dst is exists in the src_cache */
+
+        xattr_new = dict_copy_with_ref(xattr, NULL);
+        if (!xattr_new) {
+            ret = -1;
+            goto cleanup;
+        }
+        ret = dict_set_str(xattr_new, GF_FORCE_REPLACE_KEY, "yes");
+        if (ret < 0) {
+            gf_msg(this->name, GF_LOG_WARNING, 0, DHT_MSG_DICT_SET_FAILED,
+                   "Failed to set dictionary value: key = %s,"
+                   " path = %s",
+                   GF_FORCE_REPLACE_KEY, local->loc.path);
+            dict_unref(xattr_new);
+            ret = -1;
+            goto cleanup;
+        }
     }
 
     /* We should not have any failures post the link creation, as this
@@ -1383,17 +1423,14 @@ dht_rename_create_links(call_frame_t *frame)
      * unlink the newname we created, we would have effectively lost the
      * file to rename operations. */
     if (dst_hashed != src_hashed && src_cached != dst_hashed) {
-        gf_msg_trace(this->name, 0, "linkfile %s @ %s => %s", local->loc.path,
-                     dst_hashed->name, src_cached->name);
-
+        gf_msg_trace(this->name, 0, "linkto-file %s @ %s => %s",
+                     local->loc.path, dst_hashed->name, src_cached->name);
+        local->params = xattr_new;
         memcpy(local->gfid, local->loc.inode->gfid, 16);
         dht_linkfile_create(frame, dht_rename_linkto_cbk, this, src_cached,
                             dst_hashed, &local->loc);
+
     } else if (src_cached != dst_hashed) {
-        dict_t *xattr_new = NULL;
-
-        xattr_new = dict_copy_with_ref(xattr, NULL);
-
         gf_msg_trace(this->name, 0, "link %s => %s (%s)", local->loc.path,
                      local->loc2.path, src_cached->name);
         if (gf_uuid_compare(local->loc.pargfid, local->loc2.pargfid) == 0) {
@@ -1414,10 +1451,12 @@ nolinks:
         /* skip to next step */
         dht_do_rename(frame);
     }
+
+cleanup:
     if (xattr)
         dict_unref(xattr);
 
-    return 0;
+    return ret;
 }
 
 int
@@ -1550,7 +1589,11 @@ dht_rename_lookup_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
             goto fail;
         }
 
-        dht_rename_create_links(frame);
+        local->op_ret = dht_rename_create_links(frame);
+        if (local->op_ret < 0) {
+            local->op_errno = ENOMEM;
+            goto fail;
+        }
     }
 
     return 0;
