@@ -27,7 +27,7 @@ delay_gen(xlator_t *this, int fop)
         return 0;
 
     if ((rand() % DELAY_GRANULARITY) < dg->delay_ppm)
-        usleep(dg->delay_duration);
+        gf_nanosleep(((uint64_t)dg->delay_duration) * GF_US_IN_NS);
 
     return 0;
 }
@@ -530,7 +530,7 @@ init(xlator_t *this)
 
     GF_OPTION_INIT("delay-percentage", delay_percent, percent, out);
     GF_OPTION_INIT("enable", delay_enable_fops, str, out);
-    GF_OPTION_INIT("delay-duration", dg->delay_duration, int32, out);
+    GF_OPTION_INIT("delay-duration", dg->delay_duration, uint32, out);
 
     delay_gen_set_delay_ppm(dg, delay_percent);
 
@@ -561,7 +561,7 @@ mem_acct_init(xlator_t *this)
     if (!this)
         return ret;
 
-    ret = xlator_mem_acct_init(this, gf_delay_gen_mt_end + 1);
+    ret = xlator_mem_acct_init(this, gf_delay_gen_mt_end);
 
     if (ret != 0) {
         gf_log(this->name, GF_LOG_ERROR,
@@ -679,4 +679,19 @@ struct volume_options options[] = {
         .default_value = "",
     },
 
-    {.key = {NULL}}};
+    {.key = {NULL}},
+};
+
+xlator_api_t xlator_api = {
+    .init = init,
+    .fini = fini,
+    .notify = notify,
+    .reconfigure = reconfigure,
+    .mem_acct_init = mem_acct_init,
+    .op_version = {GD_OP_VERSION_3_12_0},
+    .fops = &fops,
+    .cbks = &cbks,
+    .options = options,
+    .identifier = "delay-gen",
+    .category = GF_TECH_PREVIEW,
+};
