@@ -31,9 +31,9 @@
 sys_error_t error_no_list[] = {
     [GF_FOP_LOOKUP] = {.error_no_count = 4,
                        .error_no = {ENOENT, ENOTDIR, ENAMETOOLONG, EAGAIN}},
-    [GF_FOP_STAT] = {.error_no_count = 7,
-                     .error_no = {EACCES, EBADF, EFAULT, ENAMETOOLONG, ENOENT,
-                                  ENOMEM, ENOTDIR}},
+    [GF_FOP_STAT] = {.error_no_count = 6,
+                     .error_no = {EACCES, EFAULT, ENAMETOOLONG, ENOENT, ENOMEM,
+                                  ENOTDIR}},
     [GF_FOP_READLINK] = {.error_no_count = 8,
                          .error_no = {EACCES, EFAULT, EINVAL, EIO, ENAMETOOLONG,
                                       ENOENT, ENOMEM, ENOTDIR}},
@@ -79,21 +79,20 @@ sys_error_t error_no_list[] = {
     [GF_FOP_WRITE] = {.error_no_count = 7,
                       .error_no = {EINVAL, EBADF, EFAULT, EISDIR, ENAMETOOLONG,
                                    ENOSPC, GF_ERROR_SHORT_WRITE}},
-    [GF_FOP_STATFS] = {.error_no_count = 10,
-                       .error_no = {EACCES, EBADF, EFAULT, EINTR, EIO,
-                                    ENAMETOOLONG, ENOENT, ENOMEM, ENOSYS,
-                                    ENOTDIR}},
+    [GF_FOP_STATFS] = {.error_no_count = 9,
+                       .error_no = {EACCES, EFAULT, EINTR, EIO, ENAMETOOLONG,
+                                    ENOENT, ENOMEM, ENOSYS, ENOTDIR}},
     [GF_FOP_FLUSH] = {.error_no_count = 5,
                       .error_no = {EACCES, EFAULT, ENAMETOOLONG, ENOSYS,
                                    ENOENT}},
     [GF_FOP_FSYNC] = {.error_no_count = 4,
                       .error_no = {EBADF, EIO, EROFS, EINVAL}},
-    [GF_FOP_SETXATTR] = {.error_no_count = 4,
-                         .error_no = {EACCES, EBADF, EINTR, ENAMETOOLONG}},
-    [GF_FOP_GETXATTR] = {.error_no_count = 4,
-                         .error_no = {EACCES, EBADF, ENAMETOOLONG, EINTR}},
-    [GF_FOP_REMOVEXATTR] = {.error_no_count = 4,
-                            .error_no = {EACCES, EBADF, ENAMETOOLONG, EINTR}},
+    [GF_FOP_SETXATTR] = {.error_no_count = 3,
+                         .error_no = {EACCES, EINTR, ENAMETOOLONG}},
+    [GF_FOP_GETXATTR] = {.error_no_count = 3,
+                         .error_no = {EACCES, ENAMETOOLONG, EINTR}},
+    [GF_FOP_REMOVEXATTR] = {.error_no_count = 3,
+                            .error_no = {EACCES, ENAMETOOLONG, EINTR}},
     [GF_FOP_FSETXATTR] = {.error_no_count = 4,
                           .error_no = {EACCES, EBADF, EINTR, ENAMETOOLONG}},
     [GF_FOP_FGETXATTR] = {.error_no_count = 4,
@@ -125,26 +124,25 @@ sys_error_t error_no_list[] = {
                                      ENOENT}},
     [GF_FOP_FXATTROP] = {.error_no_count = 4,
                          .error_no = {EBADF, EIO, EROFS, EINVAL}},
-    [GF_FOP_INODELK] = {.error_no_count = 4,
-                        .error_no = {EACCES, EBADF, EINTR, ENAMETOOLONG}},
+    [GF_FOP_INODELK] = {.error_no_count = 3,
+                        .error_no = {EACCES, EINTR, ENAMETOOLONG}},
     [GF_FOP_FINODELK] = {.error_no_count = 4,
                          .error_no = {EACCES, EBADF, EINTR, ENAMETOOLONG}},
-    [GF_FOP_ENTRYLK] = {.error_no_count = 4,
-                        .error_no = {EACCES, EBADF, ENAMETOOLONG, EINTR}},
+    [GF_FOP_ENTRYLK] = {.error_no_count = 3,
+                        .error_no = {EACCES, ENAMETOOLONG, EINTR}},
     [GF_FOP_FENTRYLK] = {.error_no_count = 10,
                          .error_no = {EACCES, EEXIST, EFAULT, EISDIR, EMFILE,
                                       ENAMETOOLONG, ENFILE, ENODEV, ENOENT,
                                       ENOMEM}},
-    [GF_FOP_SETATTR] = {.error_no_count = 11,
+    [GF_FOP_SETATTR] = {.error_no_count = 10,
                         .error_no = {EACCES, EFAULT, EIO, ENAMETOOLONG, ENOENT,
-                                     ENOMEM, ENOTDIR, EPERM, EROFS, EBADF,
-                                     EIO}},
+                                     ENOMEM, ENOTDIR, EPERM, EROFS, EIO}},
     [GF_FOP_FSETATTR] = {.error_no_count = 11,
                          .error_no = {EACCES, EFAULT, EIO, ENAMETOOLONG, ENOENT,
                                       ENOMEM, ENOTDIR, EPERM, EROFS, EBADF,
                                       EIO}},
-    [GF_FOP_GETSPEC] = {.error_no_count = 4,
-                        .error_no = {EACCES, EBADF, ENAMETOOLONG, EINTR}}};
+    [GF_FOP_GETSPEC] = {.error_no_count = 3,
+                        .error_no = {EACCES, ENAMETOOLONG, EINTR}}};
 
 int
 generate_rand_no(int op_no)
@@ -1413,7 +1411,7 @@ mem_acct_init(xlator_t *this)
     if (!this)
         return ret;
 
-    ret = xlator_mem_acct_init(this, gf_error_gen_mt_end + 1);
+    ret = xlator_mem_acct_init(this, gf_error_gen_mt_end);
 
     if (ret != 0) {
         gf_log(this->name, GF_LOG_ERROR,
@@ -1509,8 +1507,8 @@ init(xlator_t *this)
 
     this->private = pvt;
 
-    /* Give some seed value here */
-    srand(time(NULL));
+    /* Give some seed value here. */
+    srand(gf_time());
 
     ret = 0;
 out:
