@@ -216,7 +216,7 @@ mem_acct_init(xlator_t *this)
 
     GF_VALIDATE_OR_GOTO("dht", this, out);
 
-    ret = xlator_mem_acct_init(this, gf_libaws_mt_end + 1);
+    ret = xlator_mem_acct_init(this, gf_libaws_mt_end);
 
     if (ret != 0) {
         gf_msg(this->name, GF_LOG_ERROR, 0, 0, "Memory accounting init failed");
@@ -237,7 +237,7 @@ aws_form_request(char *resource, char **date, char *reqtype, char *bucketid,
     int date_len = -1;
     int res_len = -1;
 
-    ctime = time(NULL);
+    ctime = gf_time();
     gtime = gmtime(&ctime);
 
     date_len = strftime(httpdate, sizeof(httpdate),
