@@ -8,16 +8,16 @@
    cases as published by the Free Software Foundation.
 */
 
-#ifndef __QUIESCE_MEM_TYPES_H__
-#define __QUIESCE_MEM_TYPES_H__
+#ifndef __INDEX_MEM_TYPES_H__
+#define __INDEX_MEM_TYPES_H__
 
-#include "mem-types.h"
+#include <glusterfs/mem-types.h>
 
 enum gf_index_mem_types_ {
     gf_index_mt_priv_t = gf_common_mt_end + 1,
-    gf_index_inode_ctx_t = gf_common_mt_end + 2,
-    gf_index_fd_ctx_t = gf_common_mt_end + 3,
-    gf_index_mt_local_t = gf_common_mt_end + 4,
+    gf_index_inode_ctx_t,
+    gf_index_fd_ctx_t,
+    gf_index_mt_local_t,
     gf_index_mt_end
 };
 #endif
