@@ -3068,6 +3068,7 @@ marker_readdirp_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
     int ret = -1;
     char *resolvedpath = NULL;
     quota_inode_ctx_t *ctx = NULL;
+    int no_found_entry = 1;
 
     if (op_ret <= 0)
         goto unwind;
@@ -3085,6 +3086,9 @@ marker_readdirp_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
             (strcmp(entry->d_name, "..") == 0) || entry->inode == NULL)
             continue;
 
+        if (no_found_entry)
+            no_found_entry = 0;
+
         loc.parent = inode_ref(local->loc.inode);
         loc.inode = inode_ref(entry->inode);
         ret = inode_path(loc.parent, entry->d_name, &resolvedpath);
@@ -3118,6 +3122,9 @@ marker_readdirp_cbk(call_frame_t *frame, void *cookie, xlator_t *this,
         }
     }
 
+    if (no_found_entry && loc_is_root(&local->loc))
+        mq_create_xattrs_txn(this, &local->loc, NULL);
+
 unwind:
     MARKER_STACK_UNWIND(readdirp, frame, op_ret, op_errno, entries, xdata);
 
@@ -3178,7 +3185,7 @@ mem_acct_init(xlator_t *this)
     if (!this)
         return ret;
 
-    ret = xlator_mem_acct_init(this, gf_marker_mt_end + 1);
+    ret = xlator_mem_acct_init(this, gf_marker_mt_end);
 
     if (ret != 0) {
         gf_log(this->name, GF_LOG_ERROR,
