@@ -10,7 +10,7 @@
 
 #ifndef __META_HOOKS_H
 #define __META_HOOKS_H
-#include "xlator.h"
+#include <glusterfs/xlator.h>
 
 #define DECLARE_HOOK(name)                                                     \
     int meta_##name##_hook(call_frame_t *frame, xlator_t *this, loc_t *loc,    \
@@ -40,7 +40,7 @@ DECLARE_HOOK(name_file);
 DECLARE_HOOK(private_file);
 DECLARE_HOOK(mallinfo_file);
 DECLARE_HOOK(history_file);
-DECLARE_HOOK(master_dir);
+DECLARE_HOOK(root);
 DECLARE_HOOK(meminfo_file);
 DECLARE_HOOK(measure_file);
 DECLARE_HOOK(profile_file);
