@@ -51,12 +51,10 @@ get_snap_volname_and_volinfo(const char *volpath, char **volname,
     char *volname_token = NULL;
     char *vol = NULL;
     glusterd_snap_t *snap = NULL;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     char *tmp_str_token = NULL;
     char *volfile_token = NULL;
 
-    this = THIS;
-    GF_ASSERT(this);
     GF_ASSERT(volpath);
     GF_ASSERT(volinfo);
 
@@ -111,6 +109,8 @@ get_snap_volname_and_volinfo(const char *volpath, char **volname,
         volfile_token = strtok_r(NULL, "/", &save_ptr);
         *volname = gf_strdup(volfile_token);
         if (NULL == *volname) {
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_STRDUP_FAILED,
+                    "Volname=%s", volfile_token, NULL);
             ret = -1;
             goto out;
         }
@@ -187,7 +187,6 @@ glusterd_get_client_per_brick_volfile(glusterd_volinfo_t *volinfo,
     glusterd_conf_t *priv = NULL;
     int32_t ret = -1;
 
-    GF_VALIDATE_OR_GOTO("glusterd", THIS, out);
     priv = THIS->private;
     GF_VALIDATE_OR_GOTO(THIS->name, priv, out);
 
@@ -220,13 +219,11 @@ build_volfile_path(char *volume_id, char *path, size_t path_len,
     char path_prefix[PATH_MAX] = {
         0,
     };
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_volinfo_t *volinfo = NULL;
     glusterd_conf_t *priv = NULL;
     int32_t len = 0;
 
-    this = THIS;
-    GF_ASSERT(this);
     priv = this->private;
     GF_ASSERT(priv);
     GF_ASSERT(volume_id);
@@ -236,6 +233,7 @@ build_volfile_path(char *volume_id, char *path, size_t path_len,
     if (volid_ptr) {
         volid_ptr = strchr(volid_ptr, '/');
         if (!volid_ptr) {
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_STRCHR_FAIL, NULL);
             ret = -1;
             goto out;
         }
@@ -256,6 +254,7 @@ build_volfile_path(char *volume_id, char *path, size_t path_len,
     if (volid_ptr) {
         volid_ptr = strchr(volid_ptr, '/');
         if (!volid_ptr) {
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_STRCHR_FAIL, NULL);
             ret = -1;
             goto out;
         }
@@ -271,6 +270,7 @@ build_volfile_path(char *volume_id, char *path, size_t path_len,
     if (volid_ptr) {
         volid_ptr = strchr(volid_ptr, '/');
         if (!volid_ptr) {
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_STRCHR_FAIL, NULL);
             ret = -1;
             goto out;
         }
@@ -292,6 +292,7 @@ build_volfile_path(char *volume_id, char *path, size_t path_len,
     if (volid_ptr) {
         volid_ptr = strchr(volid_ptr, '/');
         if (!volid_ptr) {
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_STRCHR_FAIL, NULL);
             ret = -1;
             goto out;
         }
@@ -312,6 +313,7 @@ build_volfile_path(char *volume_id, char *path, size_t path_len,
     if (volid_ptr) {
         volid_ptr = strchr(volid_ptr, '/');
         if (!volid_ptr) {
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_STRCHR_FAIL, NULL);
             ret = -1;
             goto out;
         }
@@ -366,6 +368,7 @@ build_volfile_path(char *volume_id, char *path, size_t path_len,
     if (volid_ptr) {
         volid_ptr = strchr(volid_ptr, '/');
         if (!volid_ptr) {
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_STRCHR_FAIL, NULL);
             ret = -1;
             goto out;
         }
@@ -386,6 +389,7 @@ build_volfile_path(char *volume_id, char *path, size_t path_len,
     if (volid_ptr) {
         volid_ptr = strchr(volid_ptr, '/');
         if (!volid_ptr) {
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_STRCHR_FAIL, NULL);
             ret = -1;
             goto out;
         }
@@ -402,6 +406,8 @@ build_volfile_path(char *volume_id, char *path, size_t path_len,
         /* Split the volume name */
         vol = strtok_r(dup_volname, ".", &save_ptr);
         if (!vol) {
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_SPLIT_FAIL,
+                    "Volume name=%s", dup_volname, NULL);
             ret = -1;
             goto out;
         }
@@ -446,18 +452,25 @@ build_volfile_path(char *volume_id, char *path, size_t path_len,
     if (ret) {
         dup_volname = gf_strdup(volid_ptr);
         if (!dup_volname) {
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_STRDUP_FAILED,
+                    "Volume name=%s", volid_ptr, NULL);
             ret = -1;
             goto out;
         }
         /* Split the volume name */
         vol = strtok_r(dup_volname, ".", &save_ptr);
         if (!vol) {
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_SPLIT_FAIL,
+                    "Volume name=%s", dup_volname, NULL);
             ret = -1;
             goto out;
         }
         ret = glusterd_volinfo_find(vol, &volinfo);
-        if (ret)
+        if (ret) {
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_VOLINFO_GET_FAIL,
+                    NULL);
             goto out;
+        }
     }
 
 gotvolinfo:
@@ -466,8 +479,10 @@ gotvolinfo:
 
     ret = snprintf(path, path_len, "%s/%s/%s.vol", path_prefix,
                    volinfo->volname, volid_ptr);
-    if (ret == -1)
+    if (ret == -1) {
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_COPY_FAIL, NULL);
         goto out;
+    }
 
     ret = sys_stat(path, &stbuf);
 
@@ -513,21 +528,21 @@ glusterd_get_args_from_dict(gf_getspec_req *args, peer_info_t *peerinfo,
     int client_max_op_version = 1;
     int client_min_op_version = 1;
     int32_t ret = -1;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     char *name = NULL;
 
-    this = THIS;
-    GF_ASSERT(this);
     GF_ASSERT(args);
     GF_ASSERT(peerinfo);
 
     if (!args->xdata.xdata_len) {
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_INVALID_ARGUMENT, NULL);
         ret = 0;
         goto out;
     }
 
     dict = dict_new();
     if (!dict) {
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_DICT_CREATE_FAIL, NULL);
         ret = -1;
         goto out;
     }
@@ -561,6 +576,8 @@ glusterd_get_args_from_dict(gf_getspec_req *args, peer_info_t *peerinfo,
     }
     *brick_name = gf_strdup(name);
     if (*brick_name == NULL) {
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_STRDUP_FAILED,
+                "Brick_name=%s", name, NULL);
         ret = -1;
         goto out;
     }
@@ -594,11 +611,9 @@ glusterd_create_missed_snap(glusterd_missed_snap_info *missed_snapinfo,
     uuid_t snap_uuid = {
         0,
     };
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     char *mnt_device = NULL;
 
-    this = THIS;
-    GF_ASSERT(this);
     priv = this->private;
     GF_ASSERT(priv);
     GF_ASSERT(missed_snapinfo);
@@ -768,10 +783,8 @@ glusterd_take_missing_brick_snapshots(char *brick_name)
     glusterd_snap_op_t *snap_opinfo = NULL;
     int32_t ret = -1;
     gf_boolean_t update_list = _gf_false;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
 
-    this = THIS;
-    GF_ASSERT(this);
     priv = this->private;
     GF_ASSERT(priv);
     GF_ASSERT(brick_name);
@@ -905,12 +918,19 @@ __server_getspec(rpcsvc_request_t *req)
     };
     char addrstr[RPCSVC_PEER_STRLEN] = {0};
     peer_info_t *peerinfo = NULL;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     dict_t *dict = NULL;
+    glusterd_peerinfo_t *peer = NULL;
+    glusterd_conf_t *conf = NULL;
+    int peer_cnt = 0;
+    char *peer_hosts = NULL;
+    char *tmp_str = NULL;
+    char portstr[10] = {
+        0,
+    };
+    int len = 0;
 
-    this = THIS;
-    GF_ASSERT(this);
-
+    conf = this->private;
     ret = xdr_to_generic(req->msg[0], &args, (xdrproc_t)xdr_gf_getspec_req);
     if (ret < 0) {
         // failed to decode msg;
@@ -945,7 +965,7 @@ __server_getspec(rpcsvc_request_t *req)
                        volume);
     if (ret < 0 || ret >= sizeof(peerinfo->volname)) {
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_VOLINFO_GET_FAIL,
-               "peerinfo->volname %s truncated or error occured: "
+               "peerinfo->volname %s truncated or error occurred: "
                "(ret: %d)",
                peerinfo->volname, ret);
         ret = -1;
@@ -966,6 +986,7 @@ __server_getspec(rpcsvc_request_t *req)
 
     dict = dict_new();
     if (!dict) {
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_DICT_CREATE_FAIL, NULL);
         ret = -ENOMEM;
         goto fail;
     }
@@ -990,7 +1011,7 @@ __server_getspec(rpcsvc_request_t *req)
      * meant for external users.
      * For unix domain socket, address will be empty.
      */
-    if (strlen(addrstr) == 0 || gf_is_local_addr(addrstr)) {
+    if (strlen(addrstr) == 0 || glusterd_gf_is_local_addr(addrstr)) {
         ret = build_volfile_path(volume, filename, sizeof(filename),
                                  TRUSTED_PREFIX, dict);
     } else {
@@ -998,18 +1019,53 @@ __server_getspec(rpcsvc_request_t *req)
                                  dict);
     }
 
+    RCU_READ_LOCK;
+    cds_list_for_each_entry_rcu(peer, &conf->peers, uuid_list)
+    {
+        if (!peer->connected)
+            continue;
+        if (!peer_hosts) {
+            if (peer->port) {
+                snprintf(portstr, sizeof(portstr), "%d", peer->port);
+            } else {
+                snprintf(portstr, sizeof(portstr), "%d", GLUSTERD_DEFAULT_PORT);
+            }
+            len = strlen(peer->hostname) + strlen(portstr) + 3;
+            tmp_str = GF_CALLOC(1, len, gf_gld_mt_char);
+            snprintf(tmp_str, len, "%s%s%s%s", peer->hostname, ":", portstr,
+                     " ");
+            peer_hosts = tmp_str;
+        } else {
+            len = strlen(peer_hosts) + strlen(peer->hostname) +
+                  strlen(portstr) + 3;
+            tmp_str = GF_CALLOC(1, len, gf_gld_mt_char);
+            snprintf(tmp_str, len, "%s%s%s%s%s", peer_hosts, peer->hostname,
+                     ":", portstr, " ");
+            GF_FREE(peer_hosts);
+            peer_hosts = tmp_str;
+        }
+        peer_cnt++;
+    }
+    RCU_READ_UNLOCK;
+    if (peer_cnt) {
+        op_ret = dict_set_str(dict, GLUSTERD_BRICK_SERVERS, peer_hosts);
+        if (op_ret) {
+            gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_SET_FAILED,
+                   "failed to set peer_host in dict");
+            ret = op_ret;
+            goto fail;
+        }
+    }
+
     if (ret == 0) {
         if (dict->count > 0) {
             ret = dict_allocate_and_serialize(dict, &rsp.xdata.xdata_val,
                                               &rsp.xdata.xdata_len);
             if (ret) {
-                gf_msg(this->name, GF_LOG_ERROR, 0,
-                       GD_MSG_DICT_SERL_LENGTH_GET_FAIL,
-                       "Failed to serialize dict "
-                       "to request buffer");
+                gf_smsg(this->name, GF_LOG_ERROR, errno,
+                        GD_MSG_DICT_ALLOC_AND_SERL_LENGTH_GET_FAIL, NULL);
                 goto fail;
             }
-            dict->extra_free = rsp.xdata.xdata_val;
         }
 
         /* to allocate the proper buffer to hold the file data */
@@ -1028,6 +1084,7 @@ __server_getspec(rpcsvc_request_t *req)
         }
         ret = file_len = stbuf.st_size;
     } else {
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_PEER_NOT_FOUND, NULL);
         op_errno = ENOENT;
         goto fail;
     }
@@ -1035,6 +1092,7 @@ __server_getspec(rpcsvc_request_t *req)
     if (file_len) {
         rsp.spec = CALLOC(file_len + 1, sizeof(char));
         if (!rsp.spec) {
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_NO_MEMORY, NULL);
             ret = -1;
             op_errno = ENOMEM;
             goto fail;
@@ -1061,9 +1119,12 @@ fail:
     GF_FREE(brick_name);
 
     rsp.op_ret = ret;
-    if (rsp.op_ret < 0)
+    if (rsp.op_ret < 0) {
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_MOUNT_REQ_FAIL,
                "Failed to mount the volume");
+        if (!op_errno)
+            op_errno = ENOENT;
+    }
 
     if (op_errno)
         rsp.op_errno = gf_errno_to_error(op_errno);
@@ -1076,12 +1137,17 @@ fail:
     free(args.key);  // malloced by xdr
     free(rsp.spec);
 
+    if (peer_hosts)
+        GF_FREE(peer_hosts);
     if (dict)
         dict_unref(dict);
 
     if (args.xdata.xdata_val)
         free(args.xdata.xdata_val);
 
+    if (rsp.xdata.xdata_val)
+        GF_FREE(rsp.xdata.xdata_val);
+
     return 0;
 }
 
@@ -1108,13 +1174,17 @@ __server_event_notify(rpcsvc_request_t *req)
                          (xdrproc_t)xdr_gf_event_notify_req);
     if (ret < 0) {
         req->rpc_err = GARBAGE_ARGS;
+        gf_smsg("glusterd", GF_LOG_ERROR, errno, GD_MSG_GARBAGE_ARGS, NULL);
         goto fail;
     }
 
     if (args.dict.dict_len) {
         dict = dict_new();
-        if (!dict)
+        if (!dict) {
+            gf_smsg("glusterd", GF_LOG_ERROR, errno, GD_MSG_DICT_CREATE_FAIL,
+                    NULL);
             return ret;
+        }
         ret = dict_unserialize(args.dict.dict_val, args.dict.dict_len, &dict);
         if (ret) {
             gf_msg("glusterd", GF_LOG_ERROR, 0, GD_MSG_DICT_UNSERIALIZE_FAIL,
@@ -1215,15 +1285,12 @@ gd_validate_mgmt_hndsk_req(rpcsvc_request_t *req, dict_t *dict)
         0,
     };
     glusterd_peerinfo_t *peer = NULL;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     char *uuid_str = NULL;
     uuid_t peer_uuid = {
         0,
     };
 
-    this = THIS;
-    GF_ASSERT(this);
-
     if (!glusterd_have_peers() && !glusterd_have_volumes())
         return _gf_true;
 
@@ -1286,7 +1353,7 @@ int
 __glusterd_mgmt_hndsk_versions(rpcsvc_request_t *req)
 {
     dict_t *dict = NULL;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_conf_t *conf = NULL;
     int ret = -1;
     int op_errno = EINVAL;
@@ -1300,13 +1367,13 @@ __glusterd_mgmt_hndsk_versions(rpcsvc_request_t *req)
     };
     dict_t *args_dict = NULL;
 
-    this = THIS;
     conf = this->private;
 
     ret = xdr_to_generic(req->msg[0], &args, (xdrproc_t)xdr_gf_mgmt_hndsk_req);
     if (ret < 0) {
         // failed to decode msg;
         req->rpc_err = GARBAGE_ARGS;
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_GARBAGE_ARGS, NULL);
         goto out;
     }
 
@@ -1320,8 +1387,10 @@ __glusterd_mgmt_hndsk_versions(rpcsvc_request_t *req)
     }
 
     dict = dict_new();
-    if (!dict)
+    if (!dict) {
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_DICT_CREATE_FAIL, NULL);
         goto out;
+    }
 
     ret = dict_set_int32(dict, GD_OP_VERSION_KEY, conf->op_version);
     if (ret) {
@@ -1386,7 +1455,7 @@ int
 __glusterd_mgmt_hndsk_versions_ack(rpcsvc_request_t *req)
 {
     dict_t *clnt_dict = NULL;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_conf_t *conf = NULL;
     int ret = -1;
     int op_errno = EINVAL;
@@ -1400,13 +1469,13 @@ __glusterd_mgmt_hndsk_versions_ack(rpcsvc_request_t *req)
         0,
     };
 
-    this = THIS;
     conf = this->private;
 
     ret = xdr_to_generic(req->msg[0], &args, (xdrproc_t)xdr_gf_mgmt_hndsk_req);
     if (ret < 0) {
         // failed to decode msg;
         req->rpc_err = GARBAGE_ARGS;
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_GARBAGE_ARGS, NULL);
         goto out;
     }
 
@@ -1479,22 +1548,24 @@ __server_get_volume_info(rpcsvc_request_t *req)
     char *volume_id_str = NULL;
     int32_t flags = 0;
 
+    xlator_t *this = THIS;
+
     ret = xdr_to_generic(req->msg[0], &vol_info_req,
                          (xdrproc_t)xdr_gf_get_volume_info_req);
     if (ret < 0) {
         /* failed to decode msg */
         req->rpc_err = GARBAGE_ARGS;
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_GARBAGE_ARGS, NULL);
         goto out;
     }
-    gf_msg("glusterd", GF_LOG_INFO, 0, GD_MSG_VOL_INFO_REQ_RECVD,
-           "Received get volume info req");
+    gf_smsg(this->name, GF_LOG_INFO, 0, GD_MSG_VOL_INFO_REQ_RECVD, NULL);
 
     if (vol_info_req.dict.dict_len) {
         /* Unserialize the dictionary */
         dict = dict_new();
         if (!dict) {
-            gf_msg("glusterd", GF_LOG_WARNING, ENOMEM, GD_MSG_NO_MEMORY,
-                   "Out of Memory");
+            gf_smsg(this->name, GF_LOG_WARNING, ENOMEM, GD_MSG_DICT_CREATE_FAIL,
+                    NULL);
             op_errno = ENOMEM;
             ret = -1;
             goto out;
@@ -1503,9 +1574,8 @@ __server_get_volume_info(rpcsvc_request_t *req)
         ret = dict_unserialize(vol_info_req.dict.dict_val,
                                vol_info_req.dict.dict_len, &dict);
         if (ret < 0) {
-            gf_msg("glusterd", GF_LOG_ERROR, 0, GD_MSG_DICT_UNSERIALIZE_FAIL,
-                   "failed to "
-                   "unserialize req-buffer to dictionary");
+            gf_smsg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_UNSERIALIZE_FAIL,
+                    NULL);
             op_errno = -ret;
             ret = -1;
             goto out;
@@ -1516,8 +1586,8 @@ __server_get_volume_info(rpcsvc_request_t *req)
 
     ret = dict_get_int32(dict, "flags", &flags);
     if (ret) {
-        gf_msg(THIS->name, GF_LOG_ERROR, -ret, GD_MSG_DICT_GET_FAILED,
-               "failed to get flags");
+        gf_smsg(this->name, GF_LOG_ERROR, -ret, GD_MSG_DICT_GET_FAILED,
+                "Key=flags", NULL);
         op_errno = -ret;
         ret = -1;
         goto out;
@@ -1525,13 +1595,15 @@ __server_get_volume_info(rpcsvc_request_t *req)
 
     if (!flags) {
         /* Nothing to query about. Just return success */
-        gf_msg(THIS->name, GF_LOG_ERROR, 0, GD_MSG_NO_FLAG_SET, "No flags set");
+        gf_smsg(this->name, GF_LOG_ERROR, 0, GD_MSG_NO_FLAG_SET, NULL);
         ret = 0;
         goto out;
     }
 
     ret = dict_get_str(dict, "volname", &volname);
     if (ret) {
+        gf_smsg(this->name, GF_LOG_ERROR, -ret, GD_MSG_DICT_GET_FAILED,
+                "Key=volname", NULL);
         op_errno = EINVAL;
         ret = -1;
         goto out;
@@ -1539,6 +1611,8 @@ __server_get_volume_info(rpcsvc_request_t *req)
 
     ret = glusterd_volinfo_find(volname, &volinfo);
     if (ret) {
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_VOLINFO_GET_FAIL,
+                "Volname=%s", volname, NULL);
         op_errno = EINVAL;
         ret = -1;
         goto out;
@@ -1547,6 +1621,8 @@ __server_get_volume_info(rpcsvc_request_t *req)
     if (flags & (int32_t)GF_GET_VOLUME_UUID) {
         volume_id_str = gf_strdup(uuid_utoa(volinfo->volume_id));
         if (!volume_id_str) {
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_STRDUP_FAILED,
+                    NULL);
             op_errno = ENOMEM;
             ret = -1;
             goto out;
@@ -1554,8 +1630,8 @@ __server_get_volume_info(rpcsvc_request_t *req)
 
         dict_rsp = dict_new();
         if (!dict_rsp) {
-            gf_msg("glusterd", GF_LOG_WARNING, ENOMEM, GD_MSG_NO_MEMORY,
-                   "Out of Memory");
+            gf_smsg(this->name, GF_LOG_WARNING, ENOMEM, GD_MSG_DICT_CREATE_FAIL,
+                    NULL);
             op_errno = ENOMEM;
             GF_FREE(volume_id_str);
             ret = -1;
@@ -1563,6 +1639,8 @@ __server_get_volume_info(rpcsvc_request_t *req)
         }
         ret = dict_set_dynstr(dict_rsp, "volume_id", volume_id_str);
         if (ret) {
+            gf_smsg(this->name, GF_LOG_ERROR, -ret, GD_MSG_DICT_SET_FAILED,
+                    "Key=volume_id", NULL);
             op_errno = -ret;
             ret = -1;
             goto out;
@@ -1571,6 +1649,8 @@ __server_get_volume_info(rpcsvc_request_t *req)
     ret = dict_allocate_and_serialize(dict_rsp, &vol_info_rsp.dict.dict_val,
                                       &vol_info_rsp.dict.dict_len);
     if (ret) {
+        gf_smsg(this->name, GF_LOG_ERROR, errno,
+                GD_MSG_DICT_ALLOC_AND_SERL_LENGTH_GET_FAIL, NULL);
         op_errno = -ret;
         ret = -1;
         goto out;
@@ -1636,6 +1716,8 @@ __server_get_snap_info(rpcsvc_request_t *req)
     if (snap_info_req.dict.dict_len) {
         dict = dict_new();
         if (!dict) {
+            gf_smsg("glusterd", GF_LOG_WARNING, ENOMEM, GD_MSG_DICT_CREATE_FAIL,
+                    NULL);
             op_errno = ENOMEM;
             ret = -1;
             goto out;
@@ -1658,7 +1740,7 @@ __server_get_snap_info(rpcsvc_request_t *req)
     ret = dict_get_str(dict, "volname", &volname);
     if (ret) {
         op_errno = EINVAL;
-        gf_msg("glusterd", GF_LOG_ERROR, EINVAL, GD_MSG_DICT_GET_FAILED,
+        gf_msg("glusterd", GF_LOG_ERROR, -ret, GD_MSG_DICT_GET_FAILED,
                "Failed to retrieve volname");
         ret = -1;
         goto out;
@@ -1666,6 +1748,8 @@ __server_get_snap_info(rpcsvc_request_t *req)
 
     dict_rsp = dict_new();
     if (!dict_rsp) {
+        gf_smsg("glusterd", GF_LOG_WARNING, ENOMEM, GD_MSG_DICT_CREATE_FAIL,
+                NULL);
         op_errno = ENOMEM;
         ret = -1;
         goto out;
@@ -1708,16 +1792,16 @@ server_get_snap_info(rpcsvc_request_t *req)
     return glusterd_big_locked_handler(req, __server_get_snap_info);
 }
 
-rpcsvc_actor_t gluster_handshake_actors[GF_HNDSK_MAXVALUE] = {
-    [GF_HNDSK_NULL] = {"NULL", GF_HNDSK_NULL, NULL, NULL, 0, DRC_NA},
-    [GF_HNDSK_GETSPEC] = {"GETSPEC", GF_HNDSK_GETSPEC, server_getspec, NULL, 0,
-                          DRC_NA},
-    [GF_HNDSK_EVENT_NOTIFY] = {"EVENTNOTIFY", GF_HNDSK_EVENT_NOTIFY,
-                               server_event_notify, NULL, 0, DRC_NA},
-    [GF_HNDSK_GET_VOLUME_INFO] = {"GETVOLUMEINFO", GF_HNDSK_GET_VOLUME_INFO,
-                                  server_get_volume_info, NULL, 0, DRC_NA},
-    [GF_HNDSK_GET_SNAPSHOT_INFO] = {"GETSNAPINFO", GF_HNDSK_GET_SNAPSHOT_INFO,
-                                    server_get_snap_info, NULL, 0, DRC_NA},
+static rpcsvc_actor_t gluster_handshake_actors[GF_HNDSK_MAXVALUE] = {
+    [GF_HNDSK_NULL] = {"NULL", NULL, NULL, GF_HNDSK_NULL, DRC_NA, 0},
+    [GF_HNDSK_GETSPEC] = {"GETSPEC", server_getspec, NULL, GF_HNDSK_GETSPEC,
+                          DRC_NA, 0},
+    [GF_HNDSK_EVENT_NOTIFY] = {"EVENTNOTIFY", server_event_notify, NULL,
+                               GF_HNDSK_EVENT_NOTIFY, DRC_NA, 0},
+    [GF_HNDSK_GET_VOLUME_INFO] = {"GETVOLUMEINFO", server_get_volume_info, NULL,
+                                  GF_HNDSK_GET_VOLUME_INFO, DRC_NA, 0},
+    [GF_HNDSK_GET_SNAPSHOT_INFO] = {"GETSNAPINFO", server_get_snap_info, NULL,
+                                    GF_HNDSK_GET_SNAPSHOT_INFO, DRC_NA, 0},
 };
 
 struct rpcsvc_program gluster_handshake_prog = {
@@ -1729,9 +1813,9 @@ struct rpcsvc_program gluster_handshake_prog = {
 };
 
 /* A minimal RPC program just for the cli getspec command */
-rpcsvc_actor_t gluster_cli_getspec_actors[GF_HNDSK_MAXVALUE] = {
-    [GF_HNDSK_GETSPEC] = {"GETSPEC", GF_HNDSK_GETSPEC, server_getspec, NULL, 0,
-                          DRC_NA},
+static rpcsvc_actor_t gluster_cli_getspec_actors[GF_HNDSK_MAXVALUE] = {
+    [GF_HNDSK_GETSPEC] = {"GETSPEC", server_getspec, NULL, GF_HNDSK_GETSPEC,
+                          DRC_NA, 0},
 };
 
 struct rpcsvc_program gluster_cli_getspec_prog = {
@@ -1742,26 +1826,26 @@ struct rpcsvc_program gluster_cli_getspec_prog = {
     .numactors = GF_HNDSK_MAXVALUE,
 };
 
-char *glusterd_dump_proc[GF_DUMP_MAXVALUE] = {
+static char *glusterd_dump_proc[GF_DUMP_MAXVALUE] = {
     [GF_DUMP_NULL] = "NULL",
     [GF_DUMP_DUMP] = "DUMP",
     [GF_DUMP_PING] = "PING",
 };
 
-rpc_clnt_prog_t glusterd_dump_prog = {
+static rpc_clnt_prog_t glusterd_dump_prog = {
     .progname = "GLUSTERD-DUMP",
     .prognum = GLUSTER_DUMP_PROGRAM,
     .progver = GLUSTER_DUMP_VERSION,
     .procnames = glusterd_dump_proc,
 };
 
-rpcsvc_actor_t glusterd_mgmt_hndsk_actors[GD_MGMT_HNDSK_MAXVALUE] = {
-    [GD_MGMT_HNDSK_NULL] = {"NULL", GD_MGMT_HNDSK_NULL, NULL, NULL, 0, DRC_NA},
-    [GD_MGMT_HNDSK_VERSIONS] = {"MGMT-VERS", GD_MGMT_HNDSK_VERSIONS,
-                                glusterd_mgmt_hndsk_versions, NULL, 0, DRC_NA},
-    [GD_MGMT_HNDSK_VERSIONS_ACK] = {"MGMT-VERS-ACK", GD_MGMT_HNDSK_VERSIONS_ACK,
-                                    glusterd_mgmt_hndsk_versions_ack, NULL, 0,
-                                    DRC_NA},
+static rpcsvc_actor_t glusterd_mgmt_hndsk_actors[GD_MGMT_HNDSK_MAXVALUE] = {
+    [GD_MGMT_HNDSK_NULL] = {"NULL", NULL, NULL, GD_MGMT_HNDSK_NULL, DRC_NA, 0},
+    [GD_MGMT_HNDSK_VERSIONS] = {"MGMT-VERS", glusterd_mgmt_hndsk_versions, NULL,
+                                GD_MGMT_HNDSK_VERSIONS, DRC_NA, 0},
+    [GD_MGMT_HNDSK_VERSIONS_ACK] = {"MGMT-VERS-ACK",
+                                    glusterd_mgmt_hndsk_versions_ack, NULL,
+                                    GD_MGMT_HNDSK_VERSIONS_ACK, DRC_NA, 0},
 };
 
 struct rpcsvc_program glusterd_mgmt_hndsk_prog = {
@@ -1772,13 +1856,13 @@ struct rpcsvc_program glusterd_mgmt_hndsk_prog = {
     .numactors = GD_MGMT_HNDSK_MAXVALUE,
 };
 
-char *glusterd_mgmt_hndsk_proc[GD_MGMT_HNDSK_MAXVALUE] = {
+static char *glusterd_mgmt_hndsk_proc[GD_MGMT_HNDSK_MAXVALUE] = {
     [GD_MGMT_HNDSK_NULL] = "NULL",
     [GD_MGMT_HNDSK_VERSIONS] = "MGMT-VERS",
     [GD_MGMT_HNDSK_VERSIONS_ACK] = "MGMT-VERS-ACK",
 };
 
-rpc_clnt_prog_t gd_clnt_mgmt_hndsk_prog = {
+static rpc_clnt_prog_t gd_clnt_mgmt_hndsk_prog = {
     .progname = "Gluster MGMT Handshake",
     .prognum = GD_MGMT_HNDSK_PROGRAM,
     .progver = GD_MGMT_HNDSK_VERSION,
@@ -1816,12 +1900,13 @@ glusterd_event_connected_inject(glusterd_peerctx_t *peerctx)
 
     peerinfo = glusterd_peerinfo_find_by_generation(peerctx->peerinfo_gen);
     if (!peerinfo) {
+        RCU_READ_UNLOCK;
         ret = -1;
         gf_msg(THIS->name, GF_LOG_ERROR, 0, GD_MSG_PEER_NOT_FOUND,
                "Could not find peer %s(%s)", peerctx->peername,
                uuid_utoa(peerctx->peerid));
         GF_FREE(ctx);
-        goto unlock;
+        goto out;
     }
     ctx->hostname = gf_strdup(peerinfo->hostname);
     ctx->port = peerinfo->port;
@@ -1834,13 +1919,13 @@ glusterd_event_connected_inject(glusterd_peerctx_t *peerctx)
 
     ret = glusterd_friend_sm_inject_event(event);
 
+    RCU_READ_UNLOCK;
+
     if (ret)
         gf_msg("glusterd", GF_LOG_ERROR, 0, GD_MSG_EVENT_INJECT_FAIL,
                "Unable to inject "
                "EVENT_CONNECTED ret = %d",
                ret);
-unlock:
-    RCU_READ_UNLOCK;
 
 out:
     gf_msg_debug("glusterd", 0, "returning %d", ret);
@@ -1857,22 +1942,39 @@ gd_validate_peer_op_version(xlator_t *this, glusterd_peerinfo_t *peerinfo,
     int32_t peer_min_op_version = 0;
     int32_t peer_max_op_version = 0;
 
-    if (!dict || !this || !peerinfo)
+    if (!dict) {
+        gf_smsg("glusterd", GF_LOG_WARNING, ENOMEM, GD_MSG_DICT_CREATE_FAIL,
+                NULL);
         goto out;
+    }
+
+    if (!peerinfo) {
+        gf_smsg("glusterd", GF_LOG_ERROR, errno, GD_MSG_INVALID_ARGUMENT, NULL);
+        goto out;
+    }
 
     conf = this->private;
 
     ret = dict_get_int32(dict, GD_OP_VERSION_KEY, &peer_op_version);
-    if (ret)
+    if (ret) {
+        gf_smsg("glusterd", GF_LOG_ERROR, -ret, GD_MSG_DICT_GET_FAILED,
+                "Key=%s", GD_OP_VERSION_KEY, NULL);
         goto out;
+    }
 
     ret = dict_get_int32(dict, GD_MAX_OP_VERSION_KEY, &peer_max_op_version);
-    if (ret)
+    if (ret) {
+        gf_smsg("glusterd", GF_LOG_ERROR, -ret, GD_MSG_DICT_GET_FAILED,
+                "Key=%s", GD_MAX_OP_VERSION_KEY, NULL);
         goto out;
+    }
 
     ret = dict_get_int32(dict, GD_MIN_OP_VERSION_KEY, &peer_min_op_version);
-    if (ret)
+    if (ret) {
+        gf_smsg("glusterd", GF_LOG_ERROR, -ret, GD_MSG_DICT_GET_FAILED,
+                "Key=%s", GD_MIN_OP_VERSION_KEY, NULL);
         goto out;
+    }
 
     ret = -1;
     /* Check if peer can support our op_version */
@@ -1889,8 +1991,8 @@ gd_validate_peer_op_version(xlator_t *this, glusterd_peerinfo_t *peerinfo,
     ret = 0;
 out:
     if (peerinfo)
-        gf_msg_debug((this ? this->name : "glusterd"), 0, "Peer %s %s",
-                     peerinfo->hostname, ((ret < 0) ? "rejected" : "accepted"));
+        gf_msg_debug(this->name, 0, "Peer %s %s", peerinfo->hostname,
+                     ((ret < 0) ? "rejected" : "accepted"));
     return ret;
 }
 
@@ -1902,7 +2004,7 @@ __glusterd_mgmt_hndsk_version_ack_cbk(struct rpc_req *req, struct iovec *iov,
     gf_mgmt_hndsk_rsp rsp = {
         0,
     };
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     call_frame_t *frame = NULL;
     glusterd_peerinfo_t *peerinfo = NULL;
     glusterd_peerctx_t *peerctx = NULL;
@@ -1910,7 +2012,6 @@ __glusterd_mgmt_hndsk_version_ack_cbk(struct rpc_req *req, struct iovec *iov,
         0,
     };
 
-    this = THIS;
     frame = myframe;
     peerctx = frame->local;
 
@@ -2007,7 +2108,7 @@ __glusterd_mgmt_hndsk_version_cbk(struct rpc_req *req, struct iovec *iov,
     gf_mgmt_hndsk_req arg = {{
         0,
     }};
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     call_frame_t *frame = NULL;
     glusterd_peerinfo_t *peerinfo = NULL;
     glusterd_peerctx_t *peerctx = NULL;
@@ -2018,7 +2119,6 @@ __glusterd_mgmt_hndsk_version_cbk(struct rpc_req *req, struct iovec *iov,
         0,
     };
 
-    this = THIS;
     conf = this->private;
     frame = myframe;
     peerctx = frame->local;
@@ -2138,19 +2238,25 @@ glusterd_mgmt_handshake(xlator_t *this, glusterd_peerctx_t *peerctx)
     int ret = -1;
 
     frame = create_frame(this, this->ctx->pool);
-    if (!frame)
+    if (!frame) {
+        gf_smsg("glusterd", GF_LOG_WARNING, errno, GD_MSG_FRAME_CREATE_FAIL,
+                NULL);
         goto out;
+    }
 
     frame->local = peerctx;
 
     req_dict = dict_new();
-    if (!req_dict)
+    if (!req_dict) {
+        gf_smsg("glusterd", GF_LOG_WARNING, ENOMEM, GD_MSG_DICT_CREATE_FAIL,
+                NULL);
         goto out;
+    }
 
     ret = dict_set_dynstr(req_dict, GD_PEER_ID_KEY,
                           gf_strdup(uuid_utoa(MY_UUID)));
     if (ret) {
-        gf_msg(this->name, GF_LOG_ERROR, errno, GD_MSG_DICT_SET_FAILED,
+        gf_msg(this->name, GF_LOG_ERROR, -ret, GD_MSG_DICT_SET_FAILED,
                "failed to set peer ID in dict");
         goto out;
     }
@@ -2162,18 +2268,21 @@ glusterd_mgmt_handshake(xlator_t *this, glusterd_peerctx_t *peerctx)
 
     peerinfo = glusterd_peerinfo_find_by_generation(peerctx->peerinfo_gen);
     if (!peerinfo) {
+        RCU_READ_UNLOCK;
         gf_msg_debug(THIS->name, 0, "Could not find peer %s(%s)",
                      peerctx->peername, uuid_utoa(peerctx->peerid));
-        goto unlock;
+        goto out;
     }
 
     ret = glusterd_submit_request(
         peerinfo->rpc, &req, frame, &gd_clnt_mgmt_hndsk_prog,
         GD_MGMT_HNDSK_VERSIONS, NULL, this, glusterd_mgmt_hndsk_version_cbk,
         (xdrproc_t)xdr_gf_mgmt_hndsk_req);
-    ret = 0;
-unlock:
+
     RCU_READ_UNLOCK;
+
+    ret = 0;
+
 out:
     if (req_dict)
         dict_unref(req_dict);
@@ -2275,7 +2384,7 @@ __glusterd_peer_dump_version_cbk(struct rpc_req *req, struct iovec *iov,
     gf_dump_rsp rsp = {
         0,
     };
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     gf_prog_detail *trav = NULL;
     gf_prog_detail *next = NULL;
     call_frame_t *frame = NULL;
@@ -2286,7 +2395,6 @@ __glusterd_peer_dump_version_cbk(struct rpc_req *req, struct iovec *iov,
         0,
     };
 
-    this = THIS;
     conf = this->private;
     frame = myframe;
     peerctx = frame->local;
@@ -2409,20 +2517,26 @@ glusterd_peer_dump_version(xlator_t *this, struct rpc_clnt *rpc,
     int ret = -1;
 
     frame = create_frame(this, this->ctx->pool);
-    if (!frame)
+    if (!frame) {
+        gf_smsg(this->name, GF_LOG_WARNING, errno, GD_MSG_FRAME_CREATE_FAIL,
+                NULL);
         goto out;
+    }
 
     frame->local = peerctx;
-    if (!peerctx)
+    if (!peerctx) {
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_INVALID_ARGUMENT, NULL);
         goto out;
+    }
 
     RCU_READ_LOCK;
 
     peerinfo = glusterd_peerinfo_find_by_generation(peerctx->peerinfo_gen);
     if (!peerinfo) {
+        RCU_READ_UNLOCK;
         gf_msg_debug(this->name, 0, "Couldn't find peer %s(%s)",
                      peerctx->peername, uuid_utoa(peerctx->peerid));
-        goto unlock;
+        goto out;
     }
 
     req.gfs_id = 0xcafe;
@@ -2430,7 +2544,7 @@ glusterd_peer_dump_version(xlator_t *this, struct rpc_clnt *rpc,
     ret = glusterd_submit_request(
         peerinfo->rpc, &req, frame, &glusterd_dump_prog, GF_DUMP_DUMP, NULL,
         this, glusterd_peer_dump_version_cbk, (xdrproc_t)xdr_gf_dump_req);
-unlock:
+
     RCU_READ_UNLOCK;
 out:
     if (ret && frame)
