@@ -81,6 +81,7 @@ parse_mount_pattern_desc(gf_mount_spec_t *mspec, char *pdesc)
     mspec->patterns = GF_CALLOC(mspec->len, sizeof(*mspec->patterns),
                                 gf_gld_mt_mount_pattern);
     if (!mspec->patterns) {
+        gf_smsg("glusterd", GF_LOG_ERROR, errno, GD_MSG_NO_MEMORY, NULL);
         ret = -1;
         goto out;
     }
@@ -233,7 +234,7 @@ const char *georep_mnt_desc_template =
     "user-map-root=%s "
     ")"
     "SUB+("
-    "log-file=" DEFAULT_LOG_FILE_DIRECTORY "/" GEOREP
+    "log-file=%s/" GEOREP
     "*/* "
     "log-level=* "
     "volfile-id=* "
@@ -242,21 +243,9 @@ const char *georep_mnt_desc_template =
     "%s"
     ")";
 
-const char *hadoop_mnt_desc_template =
-    "SUP("
-    "volfile-server=%s "
-    "client-pid=%d "
-    "volfile-id=%s "
-    "user-map-root=%s "
-    ")"
-    "SUB+("
-    "log-file=" DEFAULT_LOG_FILE_DIRECTORY "/" GHADOOP
-    "*/* "
-    "log-level=* "
-    ")";
-
 int
-make_georep_mountspec(gf_mount_spec_t *mspec, const char *volnames, char *user)
+make_georep_mountspec(gf_mount_spec_t *mspec, const char *volnames, char *user,
+                      char *logdir)
 {
     char *georep_mnt_desc = NULL;
     char *meetspec = NULL;
@@ -273,8 +262,11 @@ make_georep_mountspec(gf_mount_spec_t *mspec, const char *volnames, char *user)
     int ret = 0;
 
     vols = gf_strdup((char *)volnames);
-    if (!vols)
+    if (!vols) {
+        gf_smsg(THIS->name, GF_LOG_ERROR, errno, GD_MSG_STRDUP_FAILED,
+                "Volume name=%s", volnames, NULL);
         goto out;
+    }
 
     for (vc = 1, p = vols; *p; p++) {
         if (*p == ',')
@@ -282,8 +274,10 @@ make_georep_mountspec(gf_mount_spec_t *mspec, const char *volnames, char *user)
     }
     siz = strlen(volnames) + vc * SLEN("volfile-id=");
     meetspec = GF_CALLOC(1, siz + 1, gf_gld_mt_georep_meet_spec);
-    if (!meetspec)
+    if (!meetspec) {
+        gf_smsg(THIS->name, GF_LOG_ERROR, errno, GD_MSG_NO_MEMORY, NULL);
         goto out;
+    }
 
     for (p = vols;;) {
         vol = strtok_r(p, ",", &savetok);
@@ -299,7 +293,7 @@ make_georep_mountspec(gf_mount_spec_t *mspec, const char *volnames, char *user)
     }
 
     ret = gf_asprintf(&georep_mnt_desc, georep_mnt_desc_template,
-                      GF_CLIENT_PID_GSYNCD, user, meetspec);
+                      GF_CLIENT_PID_GSYNCD, user, logdir, meetspec);
     if (ret == -1) {
         georep_mnt_desc = NULL;
         goto out;
@@ -322,24 +316,6 @@ out:
     return ret;
 }
 
-int
-make_ghadoop_mountspec(gf_mount_spec_t *mspec, const char *volname, char *user,
-                       char *server)
-{
-    char *hadoop_mnt_desc = NULL;
-    int ret = 0;
-
-    ret = gf_asprintf(&hadoop_mnt_desc, hadoop_mnt_desc_template, server,
-                      GF_CLIENT_PID_HADOOP, volname, user);
-    if (ret == -1)
-        return ret;
-
-    ret = parse_mount_pattern_desc(mspec, hadoop_mnt_desc);
-    GF_FREE(hadoop_mnt_desc);
-
-    return ret;
-}
-
 static gf_boolean_t
 match_comp(char *str, char *patcomp)
 {
