@@ -7,10 +7,10 @@
    later), or the GNU General Public License, version 2 (GPLv2), in all
    cases as published by the Free Software Foundation.
 */
-#include "common-utils.h"
+#include <glusterfs/common-utils.h>
 #include "cli1-xdr.h"
 #include "xdr-generic.h"
-#include "glusterfs.h"
+#include <glusterfs/glusterfs.h>
 #include "glusterd.h"
 #include "glusterd-op-sm.h"
 #include "glusterd-geo-rep.h"
@@ -18,13 +18,12 @@
 #include "glusterd-utils.h"
 #include "glusterd-svc-mgmt.h"
 #include "glusterd-svc-helper.h"
-#include "glusterd-nfs-svc.h"
 #include "glusterd-volgen.h"
 #include "glusterd-messages.h"
 #include "glusterd-server-quorum.h"
 #include "glusterd-mgmt.h"
-#include "run.h"
-#include "syscall.h"
+#include <glusterfs/run.h>
+#include <glusterfs/syscall.h>
 
 #include <signal.h>
 
@@ -48,12 +47,10 @@ __glusterd_handle_replace_brick(rpcsvc_request_t *req)
     char msg[256] = {
         0,
     };
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_conf_t *conf = NULL;
 
     GF_ASSERT(req);
-    this = THIS;
-    GF_ASSERT(this);
     conf = this->private;
 
     ret = xdr_to_generic(req->msg[0], &cli_req, (xdrproc_t)xdr_gf_cli_req);
@@ -187,13 +184,10 @@ glusterd_op_stage_replace_brick(dict_t *dict, char **op_errstr,
     glusterd_brickinfo_t *dst_brickinfo = NULL;
     glusterd_conf_t *priv = NULL;
     char pidfile[PATH_MAX] = {0};
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     gf_boolean_t is_force = _gf_false;
     char *dup_dstbrick = NULL;
 
-    this = THIS;
-    GF_ASSERT(this);
-
     priv = this->private;
     GF_ASSERT(priv);
 
@@ -228,6 +222,20 @@ glusterd_op_stage_replace_brick(dict_t *dict, char **op_errstr,
         is_force = _gf_true;
     }
 
+    if (volinfo->snap_count > 0 || !cds_list_empty(&volinfo->snap_volumes)) {
+        snprintf(msg, sizeof(msg),
+                 "Volume %s  has %" PRIu64
+                 " snapshots. "
+                 "Changing the volume configuration will not effect snapshots."
+                 "But the snapshot brick mount should be intact to "
+                 "make them function.",
+                 volname, volinfo->snap_count);
+        gf_msg("glusterd", GF_LOG_WARNING, 0, GD_MSG_SNAP_WARN, "%s", msg);
+        msg[0] = '\0';
+    }
+
+    glusterd_add_peers_to_auth_list(volname);
+
     ret = glusterd_get_dst_brick_info(&dst_brick, volname, op_errstr,
                                       &dst_brickinfo, &host, dict,
                                       &dup_dstbrick);
@@ -259,7 +267,7 @@ glusterd_op_stage_replace_brick(dict_t *dict, char **op_errstr,
         goto out;
     }
 
-    if (gf_is_local_addr(host)) {
+    if (glusterd_gf_is_local_addr(host)) {
         ret = glusterd_validate_and_create_brickpath(
             dst_brickinfo, volinfo->volume_id, volinfo->volname, op_errstr,
             is_force, _gf_false);
@@ -267,35 +275,38 @@ glusterd_op_stage_replace_brick(dict_t *dict, char **op_errstr,
             goto out;
     }
 
-    if (!gf_is_local_addr(host)) {
-        rcu_read_lock();
+    if (!glusterd_gf_is_local_addr(host)) {
+        RCU_READ_LOCK;
 
         peerinfo = glusterd_peerinfo_find(NULL, host);
         if (peerinfo == NULL) {
+            RCU_READ_UNLOCK;
             ret = -1;
             snprintf(msg, sizeof(msg), "%s, is not a friend", host);
             *op_errstr = gf_strdup(msg);
+            goto out;
 
         } else if (!peerinfo->connected) {
+            RCU_READ_UNLOCK;
+            ret = -1;
             snprintf(msg, sizeof(msg),
                      "%s, is not connected at "
                      "the moment",
                      host);
             *op_errstr = gf_strdup(msg);
-            ret = -1;
+            goto out;
 
         } else if (GD_FRIEND_STATE_BEFRIENDED != peerinfo->state.state) {
+            RCU_READ_UNLOCK;
+            ret = -1;
             snprintf(msg, sizeof(msg),
                      "%s, is not befriended "
                      "at the moment",
                      host);
             *op_errstr = gf_strdup(msg);
-            ret = -1;
-        }
-        rcu_read_unlock();
-
-        if (ret)
             goto out;
+        }
+        RCU_READ_UNLOCK;
 
     } else if (priv->op_version >= GD_OP_VERSION_3_6_0) {
         /* A bricks mount dir is required only by snapshots which were
@@ -346,11 +357,12 @@ glusterd_op_perform_replace_brick(glusterd_volinfo_t *volinfo, char *old_brick,
     glusterd_brickinfo_t *old_brickinfo = NULL;
     glusterd_brickinfo_t *new_brickinfo = NULL;
     int32_t ret = -1;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_conf_t *conf = NULL;
+    struct statvfs brickstat = {
+        0,
+    };
 
-    this = THIS;
-    GF_ASSERT(this);
     GF_ASSERT(dict);
     GF_ASSERT(volinfo);
 
@@ -366,13 +378,28 @@ glusterd_op_perform_replace_brick(glusterd_volinfo_t *volinfo, char *old_brick,
     if (ret)
         goto out;
 
+    if (!gf_uuid_compare(new_brickinfo->uuid, MY_UUID)) {
+        ret = sys_statvfs(new_brickinfo->path, &brickstat);
+        if (ret) {
+            gf_msg(this->name, GF_LOG_ERROR, errno, GD_MSG_STATVFS_FAILED,
+                   "Failed to fetch disk utilization "
+                   "from the brick (%s:%s). Please check the health of "
+                   "the brick. Error code was %s",
+                   new_brickinfo->hostname, new_brickinfo->path,
+                   strerror(errno));
+
+            goto out;
+        }
+        new_brickinfo->statfs_fsid = brickstat.f_fsid;
+    }
+
     ret = glusterd_volume_brickinfo_get_by_brick(old_brick, volinfo,
                                                  &old_brickinfo, _gf_false);
     if (ret)
         goto out;
 
-    strncpy(new_brickinfo->brick_id, old_brickinfo->brick_id,
-            sizeof(new_brickinfo->brick_id));
+    (void)snprintf(new_brickinfo->brick_id, sizeof(new_brickinfo->brick_id),
+                   "%s", old_brickinfo->brick_id);
     new_brickinfo->port = old_brickinfo->port;
 
     /* A bricks mount dir is required only by snapshots which were
@@ -387,8 +414,8 @@ glusterd_op_perform_replace_brick(glusterd_volinfo_t *volinfo, char *old_brick,
                    "brick1.mount_dir not present");
             goto out;
         }
-        strncpy(new_brickinfo->mount_dir, brick_mount_dir,
-                sizeof(new_brickinfo->mount_dir));
+        (void)snprintf(new_brickinfo->mount_dir,
+                       sizeof(new_brickinfo->mount_dir), "%s", brick_mount_dir);
     }
 
     cds_list_add(&new_brickinfo->brick_list, &old_brickinfo->brick_list);
@@ -433,16 +460,13 @@ glusterd_op_replace_brick(dict_t *dict, dict_t *rsp_dict)
     char *replace_op = NULL;
     glusterd_volinfo_t *volinfo = NULL;
     char *volname = NULL;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_conf_t *priv = NULL;
     char *src_brick = NULL;
     char *dst_brick = NULL;
     glusterd_brickinfo_t *src_brickinfo = NULL;
     glusterd_brickinfo_t *dst_brickinfo = NULL;
 
-    this = THIS;
-    GF_ASSERT(this);
-
     priv = this->private;
     GF_ASSERT(priv);
 
@@ -530,8 +554,8 @@ glusterd_op_replace_brick(dict_t *dict, dict_t *rsp_dict)
         (void)glusterd_svcs_manager(volinfo);
         goto out;
     }
-
-    volinfo->rebal.defrag_status = 0;
+    if (volinfo->rebal.defrag_status != GF_DEFRAG_STATUS_NOT_STARTED)
+        volinfo->rebal.defrag_status = GF_DEFRAG_STATUS_RESET_DUE_REPLACE_BRC;
 
     ret = glusterd_svcs_manager(volinfo);
     if (ret) {
@@ -570,12 +594,10 @@ glusterd_mgmt_v3_initiate_replace_brick_cmd_phases(rpcsvc_request_t *req,
     dict_t *req_dict = NULL;
     dict_t *tmp_dict = NULL;
     uuid_t *originator_uuid = NULL;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_conf_t *conf = NULL;
     gf_boolean_t is_acquired = _gf_false;
 
-    this = THIS;
-    GF_ASSERT(this);
     GF_ASSERT(req);
     GF_ASSERT(dict);
     conf = this->private;
