@@ -89,6 +89,8 @@ glusterd_validate_quorum(xlator_t *this, glusterd_op_t op, dict_t *dict,
 
     ret = dict_get_str(dict, "volname", &volname);
     if (ret) {
+        gf_smsg(this->name, GF_LOG_ERROR, -ret, GD_MSG_DICT_GET_FAILED,
+                "Key=volname", NULL);
         ret = 0;
         goto out;
     }
@@ -143,9 +145,7 @@ glusterd_is_quorum_changed(dict_t *options, char *option, char *value)
     char *newquorum = NULL;
     char *oldratio = NULL;
     char *newratio = NULL;
-    xlator_t *this = NULL;
-
-    this = THIS;
+    xlator_t *this = THIS;
 
     if ((strcmp("all", option) != 0) && !glusterd_is_quorum_option(option))
         goto out;
@@ -252,8 +252,11 @@ glusterd_is_volume_in_server_quorum(glusterd_volinfo_t *volinfo)
     int ret = 0;
 
     ret = dict_get_str(volinfo->dict, GLUSTERD_QUORUM_TYPE_KEY, &quorum_type);
-    if (ret)
+    if (ret) {
+        gf_smsg(THIS->name, GF_LOG_DEBUG, -ret, GD_MSG_DICT_GET_FAILED,
+                "Key=%s", GLUSTERD_QUORUM_TYPE_KEY, NULL);
         goto out;
+    }
 
     if (strcmp(quorum_type, GLUSTERD_SERVER_QUORUM) == 0)
         res = _gf_true;
@@ -287,8 +290,11 @@ does_gd_meet_server_quorum(xlator_t *this)
 
     ret = glusterd_get_quorum_cluster_counts(this, &active_count,
                                              &quorum_count);
-    if (ret)
+    if (ret) {
+        gf_smsg(this->name, GF_LOG_ERROR, errno,
+                GD_MSG_QUORUM_CLUSTER_COUNT_GET_FAIL, NULL);
         goto out;
+    }
 
     if (!does_quorum_meet(active_count, quorum_count)) {
         goto out;
@@ -418,7 +424,7 @@ out:
 int
 glusterd_do_quorum_action()
 {
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_conf_t *conf = NULL;
     glusterd_volinfo_t *volinfo = NULL;
     int ret = 0;
@@ -426,7 +432,6 @@ glusterd_do_quorum_action()
     int quorum_count = 0;
     gf_boolean_t meets = _gf_false;
 
-    this = THIS;
     conf = this->private;
 
     conf->pending_quorum_action = _gf_true;
