@@ -106,6 +106,8 @@ glusterd_destroy_friend_req_ctx(glusterd_friend_req_ctx_t *ctx)
 
     if (ctx->vols)
         dict_unref(ctx->vols);
+    if (ctx->peer_ver)
+        dict_unref(ctx->peer_ver);
     GF_FREE(ctx->hostname);
     GF_FREE(ctx);
 }
@@ -124,7 +126,7 @@ glusterd_broadcast_friend_delete(char *hostname, uuid_t uuid)
 {
     int ret = 0;
     rpc_clnt_procedure_t *proc = NULL;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_friend_update_ctx_t ctx = {
         {0},
     };
@@ -137,7 +139,6 @@ glusterd_broadcast_friend_delete(char *hostname, uuid_t uuid)
     int keylen;
     int32_t count = 0;
 
-    this = THIS;
     priv = this->private;
 
     GF_ASSERT(priv);
@@ -146,22 +147,33 @@ glusterd_broadcast_friend_delete(char *hostname, uuid_t uuid)
     ctx.op = GD_FRIEND_UPDATE_DEL;
 
     friends = dict_new();
-    if (!friends)
+    if (!friends) {
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_DICT_CREATE_FAIL, NULL);
         goto out;
+    }
 
     keylen = snprintf(key, sizeof(key), "op");
     ret = dict_set_int32n(friends, key, keylen, ctx.op);
-    if (ret)
+    if (ret) {
+        gf_smsg(this->name, GF_LOG_ERROR, -ret, GD_MSG_DICT_SET_FAILED,
+                "Key=%s", key, NULL);
         goto out;
+    }
 
     keylen = snprintf(key, sizeof(key), "hostname");
     ret = dict_set_strn(friends, key, keylen, hostname);
-    if (ret)
+    if (ret) {
+        gf_smsg(this->name, GF_LOG_ERROR, -ret, GD_MSG_DICT_SET_FAILED,
+                "Key=%s", key, NULL);
         goto out;
+    }
 
     ret = dict_set_int32n(friends, "count", SLEN("count"), count);
-    if (ret)
+    if (ret) {
+        gf_smsg(this->name, GF_LOG_ERROR, -ret, GD_MSG_DICT_SET_FAILED,
+                "Key=%s", key, NULL);
         goto out;
+    }
 
     RCU_READ_LOCK;
     cds_list_for_each_entry_rcu(peerinfo, &priv->peers, uuid_list)
@@ -175,9 +187,10 @@ glusterd_broadcast_friend_delete(char *hostname, uuid_t uuid)
          */
         ret = dict_set_static_ptr(friends, "peerinfo", peerinfo);
         if (ret) {
+            RCU_READ_UNLOCK;
             gf_msg("glusterd", GF_LOG_ERROR, 0, GD_MSG_DICT_SET_FAILED,
                    "failed to set peerinfo");
-            goto unlock;
+            goto out;
         }
 
         proc = &peerinfo->peer->proctable[GLUSTERD_FRIEND_UPDATE];
@@ -185,15 +198,13 @@ glusterd_broadcast_friend_delete(char *hostname, uuid_t uuid)
             ret = proc->fn(NULL, this, friends);
         }
     }
-unlock:
+
     RCU_READ_UNLOCK;
-
-    gf_msg_debug("glusterd", 0, "Returning with %d", ret);
-
 out:
     if (friends)
         dict_unref(friends);
 
+    gf_msg_debug("glusterd", 0, "Returning with %d", ret);
     return ret;
 }
 
@@ -229,29 +240,32 @@ glusterd_ac_reverse_probe_begin(glusterd_friend_sm_event_t *event, void *ctx)
     GF_ASSERT(event);
     GF_ASSERT(ctx);
 
+    new_ev_ctx = GF_CALLOC(1, sizeof(*new_ev_ctx), gf_gld_mt_probe_ctx_t);
+
     RCU_READ_LOCK;
 
     peerinfo = glusterd_peerinfo_find(event->peerid, event->peername);
     if (!peerinfo) {
+        RCU_READ_UNLOCK;
+        ret = -1;
         gf_msg(THIS->name, GF_LOG_ERROR, 0, GD_MSG_PEER_NOT_FOUND,
                "Could not find peer %s(%s)", event->peername,
                uuid_utoa(event->peerid));
-        ret = -1;
         goto out;
     }
 
     ret = glusterd_friend_sm_new_event(GD_FRIEND_EVENT_PROBE, &new_event);
 
     if (ret) {
+        RCU_READ_UNLOCK;
         gf_msg("glusterd", GF_LOG_ERROR, 0, GD_MSG_EVENT_NEW_GET_FAIL,
                "Unable to get new new_event");
         ret = -1;
         goto out;
     }
 
-    new_ev_ctx = GF_CALLOC(1, sizeof(*new_ev_ctx), gf_gld_mt_probe_ctx_t);
-
     if (!new_ev_ctx) {
+        RCU_READ_UNLOCK;
         ret = -1;
         goto out;
     }
@@ -266,6 +280,8 @@ glusterd_ac_reverse_probe_begin(glusterd_friend_sm_event_t *event, void *ctx)
 
     ret = glusterd_friend_sm_inject_event(new_event);
 
+    RCU_READ_UNLOCK;
+
     if (ret) {
         gf_msg("glusterd", GF_LOG_ERROR, 0, GD_MSG_EVENT_INJECT_FAIL,
                "Unable to inject new_event %d, "
@@ -274,8 +290,6 @@ glusterd_ac_reverse_probe_begin(glusterd_friend_sm_event_t *event, void *ctx)
     }
 
 out:
-    RCU_READ_UNLOCK;
-
     if (ret) {
         if (new_event)
             GF_FREE(new_event->peername);
@@ -296,11 +310,10 @@ glusterd_ac_friend_add(glusterd_friend_sm_event_t *event, void *ctx)
     rpc_clnt_procedure_t *proc = NULL;
     call_frame_t *frame = NULL;
     glusterd_conf_t *conf = NULL;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
 
     GF_ASSERT(event);
 
-    this = THIS;
     conf = this->private;
 
     GF_ASSERT(conf);
@@ -309,27 +322,30 @@ glusterd_ac_friend_add(glusterd_friend_sm_event_t *event, void *ctx)
 
     peerinfo = glusterd_peerinfo_find(event->peerid, event->peername);
     if (!peerinfo) {
+        RCU_READ_UNLOCK;
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_PEER_NOT_FOUND,
                "Could not find peer %s(%s)", event->peername,
                uuid_utoa(event->peerid));
         goto out;
     }
 
-    if (!peerinfo->peer)
+    if (!peerinfo->peer) {
+        RCU_READ_UNLOCK;
         goto out;
+    }
     proc = &peerinfo->peer->proctable[GLUSTERD_FRIEND_ADD];
     if (proc->fn) {
         frame = create_frame(this, this->ctx->pool);
         if (!frame) {
+            RCU_READ_UNLOCK;
             goto out;
         }
         frame->local = ctx;
         ret = proc->fn(frame, this, event);
     }
 
-out:
     RCU_READ_UNLOCK;
-
+out:
     if (ret && frame)
         STACK_DESTROY(frame->root);
 
@@ -344,7 +360,7 @@ glusterd_ac_friend_probe(glusterd_friend_sm_event_t *event, void *ctx)
     rpc_clnt_procedure_t *proc = NULL;
     call_frame_t *frame = NULL;
     glusterd_conf_t *conf = NULL;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_probe_ctx_t *probe_ctx = NULL;
     glusterd_peerinfo_t *peerinfo = NULL;
     dict_t *dict = NULL;
@@ -353,10 +369,6 @@ glusterd_ac_friend_probe(glusterd_friend_sm_event_t *event, void *ctx)
 
     probe_ctx = ctx;
 
-    this = THIS;
-
-    GF_ASSERT(this);
-
     conf = this->private;
 
     GF_ASSERT(conf);
@@ -365,30 +377,45 @@ glusterd_ac_friend_probe(glusterd_friend_sm_event_t *event, void *ctx)
     peerinfo = glusterd_peerinfo_find(NULL, probe_ctx->hostname);
     if (peerinfo == NULL) {
         // We should not reach this state ideally
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_PEER_NOT_FOUND, NULL);
         ret = -1;
-        goto out;
+        goto unlock;
     }
 
-    if (!peerinfo->peer)
-        goto out;
+    if (!peerinfo->peer) {
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_PEER_ADDRESS_GET_FAIL,
+                NULL);
+        goto unlock;
+    }
     proc = &peerinfo->peer->proctable[GLUSTERD_PROBE_QUERY];
     if (proc->fn) {
         frame = create_frame(this, this->ctx->pool);
         if (!frame) {
-            goto out;
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_FRAME_CREATE_FAIL,
+                    NULL);
+            goto unlock;
         }
         frame->local = ctx;
         dict = dict_new();
-        if (!dict)
-            goto out;
+        if (!dict) {
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_DICT_CREATE_FAIL,
+                    NULL);
+            goto unlock;
+        }
         ret = dict_set_strn(dict, "hostname", SLEN("hostname"),
                             probe_ctx->hostname);
-        if (ret)
-            goto out;
+        if (ret) {
+            gf_smsg(this->name, GF_LOG_ERROR, -ret, GD_MSG_DICT_SET_FAILED,
+                    "Key=hostname", NULL);
+            goto unlock;
+        }
 
         ret = dict_set_int32n(dict, "port", SLEN("port"), probe_ctx->port);
-        if (ret)
-            goto out;
+        if (ret) {
+            gf_smsg(this->name, GF_LOG_ERROR, -ret, GD_MSG_DICT_SET_FAILED,
+                    "Key=port", NULL);
+            goto unlock;
+        }
 
         /* The peerinfo reference being set here is going to be used
          * only within this critical section, in glusterd_rpc_probe
@@ -396,6 +423,7 @@ glusterd_ac_friend_probe(glusterd_friend_sm_event_t *event, void *ctx)
          */
         ret = dict_set_static_ptr(dict, "peerinfo", peerinfo);
         if (ret) {
+            RCU_READ_UNLOCK;
             gf_msg("glusterd", GF_LOG_ERROR, 0, GD_MSG_DICT_SET_FAILED,
                    "failed to set peerinfo");
             goto out;
@@ -403,11 +431,11 @@ glusterd_ac_friend_probe(glusterd_friend_sm_event_t *event, void *ctx)
 
         ret = proc->fn(frame, this, dict);
         if (ret)
-            goto out;
+            goto unlock;
     }
-
-out:
+unlock:
     RCU_READ_UNLOCK;
+out:
 
     if (dict)
         dict_unref(dict);
@@ -428,14 +456,13 @@ glusterd_ac_send_friend_remove_req(glusterd_friend_sm_event_t *event,
     rpc_clnt_procedure_t *proc = NULL;
     call_frame_t *frame = NULL;
     glusterd_conf_t *conf = NULL;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_friend_sm_event_type_t event_type = GD_FRIEND_EVENT_NONE;
     glusterd_probe_ctx_t *ctx = NULL;
     glusterd_friend_sm_event_t *new_event = NULL;
 
     GF_ASSERT(event);
 
-    this = THIS;
     conf = this->private;
 
     GF_ASSERT(conf);
@@ -444,6 +471,8 @@ glusterd_ac_send_friend_remove_req(glusterd_friend_sm_event_t *event,
 
     peerinfo = glusterd_peerinfo_find(event->peerid, event->peername);
     if (!peerinfo) {
+        RCU_READ_UNLOCK;
+        ret = -1;
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_PEER_NOT_FOUND,
                "Could not find peer %s(%s)", event->peername,
                uuid_utoa(event->peerid));
@@ -471,23 +500,29 @@ glusterd_ac_send_friend_remove_req(glusterd_friend_sm_event_t *event,
             glusterd_broadcast_friend_delete(ctx->hostname, NULL);
             glusterd_destroy_probe_ctx(ctx);
         }
-        goto out;
+        goto unlock;
     }
 
-    if (!peerinfo->peer)
-        goto out;
+    if (!peerinfo->peer) {
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_PEER_ADDRESS_GET_FAIL,
+                NULL);
+        goto unlock;
+    }
     proc = &peerinfo->peer->proctable[GLUSTERD_FRIEND_REMOVE];
     if (proc->fn) {
         frame = create_frame(this, this->ctx->pool);
         if (!frame) {
-            goto out;
+            gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_FRAME_CREATE_FAIL,
+                    NULL);
+            goto unlock;
         }
         frame->local = data;
         ret = proc->fn(frame, this, event);
     }
 
-out:
+unlock:
     RCU_READ_UNLOCK;
+out:
 
     gf_msg_debug("glusterd", 0, "Returning with %d", ret);
 
@@ -501,13 +536,11 @@ static gf_boolean_t
 glusterd_should_update_peer(glusterd_peerinfo_t *peerinfo,
                             glusterd_peerinfo_t *cur_peerinfo)
 {
-    gf_boolean_t is_valid = _gf_false;
-
     if ((peerinfo == cur_peerinfo) ||
         (peerinfo->state.state == GD_FRIEND_STATE_BEFRIENDED))
-        is_valid = _gf_true;
+        return _gf_true;
 
-    return is_valid;
+    return _gf_false;
 }
 
 static int
@@ -517,7 +550,7 @@ glusterd_ac_send_friend_update(glusterd_friend_sm_event_t *event, void *ctx)
     glusterd_peerinfo_t *cur_peerinfo = NULL;
     glusterd_peerinfo_t *peerinfo = NULL;
     rpc_clnt_procedure_t *proc = NULL;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_friend_update_ctx_t ev_ctx = {{0}};
     glusterd_conf_t *priv = NULL;
     dict_t *friends = NULL;
@@ -529,32 +562,37 @@ glusterd_ac_send_friend_update(glusterd_friend_sm_event_t *event, void *ctx)
 
     GF_ASSERT(event);
 
-    this = THIS;
     priv = this->private;
 
     GF_ASSERT(priv);
 
+    keylen = snprintf(key, sizeof(key), "op");
+    friends = dict_new();
+
     RCU_READ_LOCK;
 
     cur_peerinfo = glusterd_peerinfo_find(event->peerid, event->peername);
     if (!cur_peerinfo) {
+        RCU_READ_UNLOCK;
+        ret = -1;
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_PEER_NOT_FOUND,
                "Could not find peer %s(%s)", event->peername,
                uuid_utoa(event->peerid));
-        ret = -1;
         goto out;
     }
 
+    if (!friends) {
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_DICT_CREATE_FAIL, NULL);
+        goto unlock;
+    }
+
     ev_ctx.op = GD_FRIEND_UPDATE_ADD;
-
-    friends = dict_new();
-    if (!friends)
-        goto out;
-
-    keylen = snprintf(key, sizeof(key), "op");
     ret = dict_set_int32n(friends, key, keylen, ev_ctx.op);
-    if (ret)
-        goto out;
+    if (ret) {
+        gf_smsg(this->name, GF_LOG_ERROR, -ret, GD_MSG_DICT_SET_FAILED,
+                "Key=%s", key, NULL);
+        goto unlock;
+    }
 
     cds_list_for_each_entry_rcu(peerinfo, &priv->peers, uuid_list)
     {
@@ -566,12 +604,15 @@ glusterd_ac_send_friend_update(glusterd_friend_sm_event_t *event, void *ctx)
         snprintf(key, sizeof(key), "friend%d", count);
         ret = gd_add_friend_to_dict(peerinfo, friends, key);
         if (ret)
-            goto out;
+            goto unlock;
     }
 
     ret = dict_set_int32n(friends, "count", SLEN("count"), count);
-    if (ret)
-        goto out;
+    if (ret) {
+        gf_smsg(this->name, GF_LOG_ERROR, -ret, GD_MSG_DICT_SET_FAILED,
+                "Key=count", NULL);
+        goto unlock;
+    }
 
     cds_list_for_each_entry_rcu(peerinfo, &priv->peers, uuid_list)
     {
@@ -583,6 +624,7 @@ glusterd_ac_send_friend_update(glusterd_friend_sm_event_t *event, void *ctx)
 
         ret = dict_set_static_ptr(friends, "peerinfo", peerinfo);
         if (ret) {
+            RCU_READ_UNLOCK;
             gf_msg("glusterd", GF_LOG_ERROR, 0, GD_MSG_DICT_SET_FAILED,
                    "failed to set peerinfo");
             goto out;
@@ -594,14 +636,14 @@ glusterd_ac_send_friend_update(glusterd_friend_sm_event_t *event, void *ctx)
         }
     }
 
-    gf_msg_debug("glusterd", 0, "Returning with %d", ret);
-
-out:
+unlock:
     RCU_READ_UNLOCK;
+out:
 
     if (friends)
         dict_unref(friends);
 
+    gf_msg_debug("glusterd", 0, "Returning with %d", ret);
     return ret;
 }
 
@@ -615,7 +657,7 @@ glusterd_ac_update_friend(glusterd_friend_sm_event_t *event, void *ctx)
     glusterd_peerinfo_t *cur_peerinfo = NULL;
     glusterd_peerinfo_t *peerinfo = NULL;
     rpc_clnt_procedure_t *proc = NULL;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_friend_update_ctx_t ev_ctx = {{0}};
     glusterd_conf_t *priv = NULL;
     dict_t *friends = NULL;
@@ -627,19 +669,22 @@ glusterd_ac_update_friend(glusterd_friend_sm_event_t *event, void *ctx)
 
     GF_ASSERT(event);
 
-    this = THIS;
     priv = this->private;
 
     GF_ASSERT(priv);
 
+    friends = dict_new();
+    keylen = snprintf(key, sizeof(key), "op");
+
     RCU_READ_LOCK;
 
     cur_peerinfo = glusterd_peerinfo_find(event->peerid, event->peername);
     if (!cur_peerinfo) {
+        RCU_READ_UNLOCK;
+        ret = -1;
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_PEER_NOT_FOUND,
                "Could not find peer %s(%s)", event->peername,
                uuid_utoa(event->peerid));
-        ret = -1;
         goto out;
     }
 
@@ -649,19 +694,21 @@ glusterd_ac_update_friend(glusterd_friend_sm_event_t *event, void *ctx)
      */
     if (!cur_peerinfo->connected || !cur_peerinfo->peer) {
         ret = 0;
+        goto unlock;
+    }
+
+    if (!friends) {
+        gf_smsg(this->name, GF_LOG_ERROR, errno, GD_MSG_DICT_CREATE_FAIL, NULL);
         goto out;
     }
 
     ev_ctx.op = GD_FRIEND_UPDATE_ADD;
-
-    friends = dict_new();
-    if (!friends)
-        goto out;
-
-    keylen = snprintf(key, sizeof(key), "op");
     ret = dict_set_int32n(friends, key, keylen, ev_ctx.op);
-    if (ret)
-        goto out;
+    if (ret) {
+        gf_smsg(this->name, GF_LOG_ERROR, -ret, GD_MSG_DICT_SET_FAILED,
+                "Key=%s", key, NULL);
+        goto unlock;
+    }
 
     cds_list_for_each_entry_rcu(peerinfo, &priv->peers, uuid_list)
     {
@@ -673,15 +720,19 @@ glusterd_ac_update_friend(glusterd_friend_sm_event_t *event, void *ctx)
         snprintf(key, sizeof(key), "friend%d", count);
         ret = gd_add_friend_to_dict(peerinfo, friends, key);
         if (ret)
-            goto out;
+            goto unlock;
     }
 
     ret = dict_set_int32n(friends, "count", SLEN("count"), count);
-    if (ret)
-        goto out;
+    if (ret) {
+        gf_smsg(this->name, GF_LOG_ERROR, -ret, GD_MSG_DICT_SET_FAILED,
+                "Key=count", NULL);
+        goto unlock;
+    }
 
     ret = dict_set_static_ptr(friends, "peerinfo", cur_peerinfo);
     if (ret) {
+        RCU_READ_UNLOCK;
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_DICT_SET_FAILED,
                "failed to set peerinfo");
         goto out;
@@ -693,8 +744,9 @@ glusterd_ac_update_friend(glusterd_friend_sm_event_t *event, void *ctx)
 
     gf_msg_debug(this->name, 0, "Returning with %d", ret);
 
-out:
+unlock:
     RCU_READ_UNLOCK;
+out:
 
     if (friends)
         dict_unref(friends);
@@ -846,18 +898,18 @@ glusterd_ac_friend_remove(glusterd_friend_sm_event_t *event, void *ctx)
 
     peerinfo = glusterd_peerinfo_find(event->peerid, event->peername);
     if (!peerinfo) {
+        RCU_READ_UNLOCK;
         gf_msg(THIS->name, GF_LOG_ERROR, 0, GD_MSG_PEER_NOT_FOUND,
                "Could not find peer %s(%s)", event->peername,
                uuid_utoa(event->peerid));
-        RCU_READ_UNLOCK;
         goto out;
     }
     ret = glusterd_friend_remove_cleanup_vols(peerinfo->uuid);
+    RCU_READ_UNLOCK;
     if (ret)
         gf_msg(THIS->name, GF_LOG_WARNING, 0, GD_MSG_VOL_CLEANUP_FAIL,
                "Volumes cleanup failed");
 
-    RCU_READ_UNLOCK;
     /* Exiting read critical section as glusterd_peerinfo_cleanup calls
      * synchronize_rcu before freeing the peerinfo
      */
@@ -895,12 +947,9 @@ glusterd_ac_handle_friend_add_req(glusterd_friend_sm_event_t *event, void *ctx)
     int status = 0;
     int32_t op_ret = -1;
     int32_t op_errno = 0;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     char *hostname = NULL;
 
-    this = THIS;
-    GF_ASSERT(this);
-
     GF_ASSERT(ctx);
     ev_ctx = ctx;
     gf_uuid_copy(uuid, ev_ctx->uuid);
@@ -908,11 +957,11 @@ glusterd_ac_handle_friend_add_req(glusterd_friend_sm_event_t *event, void *ctx)
     RCU_READ_LOCK;
     peerinfo = glusterd_peerinfo_find(event->peerid, event->peername);
     if (!peerinfo) {
+        RCU_READ_UNLOCK;
+        ret = -1;
         gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_PEER_NOT_FOUND,
                "Could not find peer %s(%s)", event->peername,
                uuid_utoa(event->peerid));
-        ret = -1;
-        RCU_READ_UNLOCK;
         goto out;
     }
 
@@ -936,8 +985,8 @@ glusterd_ac_handle_friend_add_req(glusterd_friend_sm_event_t *event, void *ctx)
     // Build comparison logic here.
     pthread_mutex_lock(&conf->import_volumes);
     {
-        ret = glusterd_compare_friend_data(ev_ctx->vols, &status,
-                                           event->peername);
+        ret = glusterd_compare_friend_data(ev_ctx->vols, ev_ctx->peer_ver,
+                                           &status, event->peername);
         if (ret) {
             pthread_mutex_unlock(&conf->import_volumes);
             goto out;
@@ -1050,6 +1099,7 @@ glusterd_friend_sm_transition_state(uuid_t peerid, char *peername,
     RCU_READ_LOCK;
     peerinfo = glusterd_peerinfo_find(peerid, peername);
     if (!peerinfo) {
+        gf_smsg(THIS->name, GF_LOG_ERROR, errno, GD_MSG_PEER_NOT_FOUND, NULL);
         goto out;
     }
 
@@ -1399,11 +1449,9 @@ glusterd_friend_sm()
     gf_boolean_t is_await_conn = _gf_false;
     gf_boolean_t quorum_action = _gf_false;
     glusterd_friend_sm_state_t old_state = GD_FRIEND_STATE_DEFAULT;
-    xlator_t *this = NULL;
+    xlator_t *this = THIS;
     glusterd_conf_t *priv = NULL;
 
-    this = THIS;
-    GF_ASSERT(this);
     priv = this->private;
     GF_ASSERT(priv);
 
@@ -1417,21 +1465,20 @@ glusterd_friend_sm()
 
             peerinfo = glusterd_peerinfo_find(event->peerid, event->peername);
             if (!peerinfo) {
+                RCU_READ_UNLOCK;
                 gf_msg("glusterd", GF_LOG_CRITICAL, 0, GD_MSG_PEER_NOT_FOUND,
                        "Received"
                        " event %s with empty peer info",
                        glusterd_friend_sm_event_name_get(event_type));
 
                 GF_FREE(event);
-                RCU_READ_UNLOCK;
                 continue;
             }
+            old_state = peerinfo->state.state;
+            RCU_READ_UNLOCK;
             gf_msg_debug("glusterd", 0, "Dequeued event of type: '%s'",
                          glusterd_friend_sm_event_name_get(event_type));
 
-            old_state = peerinfo->state.state;
-
-            RCU_READ_UNLOCK;
             /* Giving up read-critical section here as we only need
              * the current state to call the handler.
              *
@@ -1514,11 +1561,11 @@ glusterd_friend_sm()
             }
 
             ret = glusterd_store_peerinfo(peerinfo);
+            RCU_READ_UNLOCK;
             if (ret) {
                 gf_msg(this->name, GF_LOG_ERROR, 0, GD_MSG_PEERINFO_CREATE_FAIL,
                        "Failed to store peerinfo");
             }
-            RCU_READ_UNLOCK;
 
             glusterd_destroy_friend_event_context(event);
             GF_FREE(event);
