@@ -11,13 +11,13 @@
 #define _GLUSTERD_UTILS_H
 
 #include <pthread.h>
-#include "compat-uuid.h"
+#include <glusterfs/compat-uuid.h>
 
-#include "glusterfs.h"
-#include "xlator.h"
-#include "logging.h"
-#include "call-stub.h"
-#include "byte-order.h"
+#include <glusterfs/glusterfs.h>
+#include <glusterfs/xlator.h>
+#include <glusterfs/logging.h>
+#include <glusterfs/call-stub.h>
+#include <glusterfs/byte-order.h>
 #include "glusterd.h"
 #include "rpc-clnt.h"
 #include "protocol-common.h"
@@ -32,38 +32,43 @@
                 brickid);                                                      \
     } while (0)
 
+#define GLUSTERD_ASSIGN_BRICKID_TO_TA_BRICKINFO(ta_brickinfo, volinfo,         \
+                                                brickid)                       \
+    do {                                                                       \
+        sprintf(ta_brickinfo->brick_id, "%s-ta-%d", volinfo->volname,          \
+                brickid);                                                      \
+    } while (0)
+
 #define ALL_VOLUME_OPTION_CHECK(volname, get_opt, key, ret, op_errstr, label)  \
     do {                                                                       \
         gf_boolean_t _all = !strcmp("all", volname);                           \
+        gf_boolean_t _key_all = !strcmp(key, "all");                           \
         gf_boolean_t _is_valid_opt = _gf_false;                                \
         int32_t i = 0;                                                         \
                                                                                \
-        if (!get_opt && (!strcmp(key, "all") ||                                \
-                         !strcmp(key, GLUSTERD_MAX_OP_VERSION_KEY))) {         \
+        if (!get_opt &&                                                        \
+            (_key_all || !strcmp(key, GLUSTERD_MAX_OP_VERSION_KEY))) {         \
             ret = -1;                                                          \
             *op_errstr = gf_strdup("Not a valid option to set");               \
             goto out;                                                          \
         }                                                                      \
-                                                                               \
-        for (i = 0; valid_all_vol_opts[i].option; i++) {                       \
-            if (!strcmp(key, "all") ||                                         \
-                !strcmp(key, valid_all_vol_opts[i].option)) {                  \
-                _is_valid_opt = _gf_true;                                      \
-                break;                                                         \
+        if (_key_all) {                                                        \
+            _is_valid_opt = _gf_true;                                          \
+        } else {                                                               \
+            for (i = 0; valid_all_vol_opts[i].option; i++) {                   \
+                if (!strcmp(key, valid_all_vol_opts[i].option)) {              \
+                    _is_valid_opt = _gf_true;                                  \
+                    break;                                                     \
+                }                                                              \
             }                                                                  \
         }                                                                      \
-                                                                               \
         if (_all && !_is_valid_opt) {                                          \
             ret = -1;                                                          \
-            *op_errstr = gf_strdup(                                            \
-                "Not a valid option for all "                                  \
-                "volumes");                                                    \
+            *op_errstr = gf_strdup("Not a valid option for all volumes");      \
             goto label;                                                        \
         } else if (!_all && _is_valid_opt) {                                   \
             ret = -1;                                                          \
-            *op_errstr = gf_strdup(                                            \
-                "Not a valid option for "                                      \
-                "single volume");                                              \
+            *op_errstr = gf_strdup("Not a valid option for single volume");    \
             goto label;                                                        \
         }                                                                      \
     } while (0)
@@ -81,6 +86,11 @@ typedef struct glusterd_dict_ctx_ {
     char *prefix;
 } glusterd_dict_ctx_t;
 
+typedef struct glusterd_hostname_ {
+    char *hostname;
+    struct list_head hostname_list;
+} glusterd_hostname_t;
+
 gf_boolean_t
 is_brick_mx_enabled(void);
 
@@ -145,9 +155,6 @@ glusterd_auth_set_password(glusterd_volinfo_t *volinfo, char *password);
 void
 glusterd_auth_cleanup(glusterd_volinfo_t *volinfo);
 
-gf_boolean_t
-glusterd_check_volume_exists(char *volname);
-
 int32_t
 glusterd_brickprocess_new(glusterd_brick_proc_t **brickprocess);
 
@@ -162,6 +169,9 @@ glusterd_brickinfo_new_from_brick(char *brick, glusterd_brickinfo_t **brickinfo,
 int32_t
 glusterd_volinfo_find(const char *volname, glusterd_volinfo_t **volinfo);
 
+gf_boolean_t
+glusterd_volume_exists(const char *volname);
+
 int
 glusterd_volinfo_find_by_volume_id(uuid_t volume_id,
                                    glusterd_volinfo_t **volinfo);
@@ -181,7 +191,8 @@ int32_t
 glusterd_resolve_brick(glusterd_brickinfo_t *brickinfo);
 
 int
-glusterd_brick_process_add_brick(glusterd_brickinfo_t *brickinfo);
+glusterd_brick_process_add_brick(glusterd_brickinfo_t *brickinfo,
+                                 glusterd_brickinfo_t *parent_brickinfo);
 
 int
 glusterd_brick_process_remove_brick(glusterd_brickinfo_t *brickinfo,
@@ -203,7 +214,7 @@ glusterd_volume_stop_glusterfs(glusterd_volinfo_t *volinfo,
 int
 send_attach_req(xlator_t *this, struct rpc_clnt *rpc, char *path,
                 glusterd_brickinfo_t *brick, glusterd_brickinfo_t *other_brick,
-                int op);
+                int op, gf_boolean_t graceful_cleanup);
 
 glusterd_volinfo_t *
 glusterd_volinfo_ref(glusterd_volinfo_t *volinfo);
@@ -226,10 +237,11 @@ glusterd_volume_brickinfo_get_by_brick(char *brick, glusterd_volinfo_t *volinfo,
                                        gf_boolean_t construct_real_path);
 
 int32_t
-glusterd_add_volumes_to_export_dict(dict_t **peer_data);
+glusterd_add_volumes_to_export_dict(dict_t *peer_data, char **buf,
+                                    u_int *length);
 
 int32_t
-glusterd_compare_friend_data(dict_t *peer_data, int32_t *status,
+glusterd_compare_friend_data(dict_t *peer_data, dict_t *cmp, int32_t *status,
                              char *hostname);
 
 int
@@ -286,17 +298,6 @@ int
 glusterd_brick_stop(glusterd_volinfo_t *volinfo,
                     glusterd_brickinfo_t *brickinfo, gf_boolean_t del_brick);
 
-gf_boolean_t
-glusterd_is_tier_daemon_running(glusterd_volinfo_t *volinfo);
-
-int32_t
-glusterd_add_tierd_to_dict(glusterd_volinfo_t *volinfo, dict_t *dict,
-                           int32_t count);
-
-int
-glusterd_op_tier_status(dict_t *dict, char **op_errstr, dict_t *rsp_dict,
-                        glusterd_op_t op);
-
 int
 glusterd_is_defrag_on(glusterd_volinfo_t *volinfo);
 
@@ -369,7 +370,7 @@ int
 glusterd_restart_gsyncds(glusterd_conf_t *conf);
 
 int
-glusterd_start_gsync(glusterd_volinfo_t *master_vol, char *slave,
+glusterd_start_gsync(glusterd_volinfo_t *primary_vol, char *secondary,
                      char *path_list, char *conf_path, char *glusterd_uuid_str,
                      char **op_errstr, gf_boolean_t is_pause);
 int
@@ -407,8 +408,10 @@ glusterd_brick_terminate(glusterd_volinfo_t *volinfo,
                          glusterd_brickinfo_t *brickinfo, char *options,
                          int option_cnt, char **op_errstr);
 
+#ifdef BUILD_GNFS
 int
 glusterd_nfs_statedump(char *options, int option_cnt, char **op_errstr);
+#endif
 
 int
 glusterd_client_statedump(char *volname, char *options, int option_cnt,
@@ -443,11 +446,6 @@ glusterd_get_trusted_client_filepath(char *filepath,
 int
 glusterd_restart_rebalance(glusterd_conf_t *conf);
 
-int32_t
-glusterd_create_sub_tier_volinfo(glusterd_volinfo_t *volinfo,
-                                 glusterd_volinfo_t **dup_volinfo,
-                                 gf_boolean_t is_hot_tier,
-                                 const char *new_name);
 int
 glusterd_restart_rebalance_for_volume(glusterd_volinfo_t *volinfo);
 
@@ -463,9 +461,6 @@ int
 glusterd_add_node_to_dict(char *server, dict_t *dict, int count,
                           dict_t *vol_opts);
 
-int
-glusterd_calc_dist_leaf_count(int rcount, int scount);
-
 int
 glusterd_get_dist_leaf_count(glusterd_volinfo_t *volinfo);
 
@@ -508,8 +503,6 @@ glusterd_volume_status_copy_to_op_ctx_dict(dict_t *aggr, dict_t *rsp_dict);
 int
 glusterd_volume_rebalance_use_rsp_dict(dict_t *aggr, dict_t *rsp_dict);
 int
-glusterd_volume_tier_use_rsp_dict(dict_t *aggr, dict_t *rsp_dict);
-int
 glusterd_volume_heal_use_rsp_dict(dict_t *aggr, dict_t *rsp_dict);
 int
 glusterd_use_rsp_dict(dict_t *aggr, dict_t *rsp_dict);
@@ -581,17 +574,19 @@ glusterd_remove_brick_validate_bricks(gf1_op_commands cmd, int32_t brick_count,
                                       dict_t *dict, glusterd_volinfo_t *volinfo,
                                       char **errstr, gf_cli_defrag_type);
 int
-glusterd_get_slave_details_confpath(glusterd_volinfo_t *volinfo, dict_t *dict,
-                                    char **slave_url, char **slave_host,
-                                    char **slave_vol, char **conf_path,
-                                    char **op_errstr);
+glusterd_get_secondary_details_confpath(glusterd_volinfo_t *volinfo,
+                                        dict_t *dict, char **secondary_url,
+                                        char **secondary_host,
+                                        char **secondary_vol, char **conf_path,
+                                        char **op_errstr);
 
 int
-glusterd_get_slave_info(char *slave, char **slave_url, char **hostname,
-                        char **slave_vol, char **op_errstr);
+glusterd_get_secondary_info(char *secondary, char **secondary_url,
+                            char **hostname, char **secondary_vol,
+                            char **op_errstr);
 
 int
-glusterd_get_statefile_name(glusterd_volinfo_t *volinfo, char *slave,
+glusterd_get_statefile_name(glusterd_volinfo_t *volinfo, char *secondary,
                             char *conf_path, char **statefile,
                             gf_boolean_t *is_template_in_use);
 
@@ -599,17 +594,18 @@ int
 glusterd_gsync_read_frm_status(char *path, char *buf, size_t blen);
 
 int
-glusterd_create_status_file(char *master, char *slave, char *slave_url,
-                            char *slave_vol, char *status);
+glusterd_create_status_file(char *primary, char *secondary, char *secondary_url,
+                            char *secondary_vol, char *status);
 
 int
-glusterd_check_restart_gsync_session(glusterd_volinfo_t *volinfo, char *slave,
-                                     dict_t *resp_dict, char *path_list,
-                                     char *conf_path, gf_boolean_t is_force);
+glusterd_check_restart_gsync_session(glusterd_volinfo_t *volinfo,
+                                     char *secondary, dict_t *resp_dict,
+                                     char *path_list, char *conf_path,
+                                     gf_boolean_t is_force);
 
 int
-glusterd_check_gsync_running_local(char *master, char *slave, char *conf_path,
-                                   gf_boolean_t *is_run);
+glusterd_check_gsync_running_local(char *primary, char *secondary,
+                                   char *conf_path, gf_boolean_t *is_run);
 
 gf_boolean_t
 glusterd_is_status_tasks_op(glusterd_op_t op, dict_t *dict);
@@ -617,12 +613,6 @@ glusterd_is_status_tasks_op(glusterd_op_t op, dict_t *dict);
 gf_boolean_t
 gd_should_i_start_rebalance(glusterd_volinfo_t *volinfo);
 
-int
-glusterd_is_tierd_enabled(glusterd_volinfo_t *volinfo);
-
-int
-glusterd_is_tierd_supposed_to_be_enabled(glusterd_volinfo_t *volinfo);
-
 int
 glusterd_is_volume_quota_enabled(glusterd_volinfo_t *volinfo);
 
@@ -679,7 +669,7 @@ int32_t
 glusterd_take_lvm_snapshot(glusterd_brickinfo_t *brickinfo,
                            char *origin_brick_path);
 
-void
+int
 glusterd_launch_synctask(synctask_fn_t fn, void *opaque);
 
 int
@@ -779,6 +769,10 @@ glusterd_volume_get_type_str(glusterd_volinfo_t *volinfo, char **vol_type_str);
 int
 glusterd_volume_get_status_str(glusterd_volinfo_t *volinfo, char *status_str);
 
+void
+glusterd_brick_get_status_str(glusterd_brickinfo_t *brickinfo,
+                              char *status_str);
+
 int
 glusterd_volume_get_transport_type_str(glusterd_volinfo_t *volinfo,
                                        char *transport_type_str);
@@ -791,21 +785,10 @@ int
 glusterd_volume_get_rebalance_status_str(glusterd_volinfo_t *volinfo,
                                          char *rebal_status_str);
 
-int
-glusterd_volume_get_hot_tier_type_str(glusterd_volinfo_t *volinfo,
-                                      char **hot_tier_type_str);
-
-int
-glusterd_volume_get_cold_tier_type_str(glusterd_volinfo_t *volinfo,
-                                       char **cold_tier_type_str);
-
 void
 glusterd_list_add_order(struct cds_list_head *new, struct cds_list_head *head,
                         int (*compare)(struct cds_list_head *,
                                        struct cds_list_head *));
-int
-glusterd_disallow_op_for_tier(glusterd_volinfo_t *volinfo, glusterd_op_t op,
-                              int cmd);
 
 struct rpc_clnt *
 glusterd_defrag_rpc_get(glusterd_defrag_info_t *defrag);
@@ -873,4 +856,22 @@ glusterd_get_volinfo_from_brick(char *brick, glusterd_volinfo_t **volinfo);
 gf_boolean_t
 glusterd_is_profile_on(glusterd_volinfo_t *volinfo);
 
+char *
+search_brick_path_from_proc(pid_t brick_pid, char *brickpath);
+
+int32_t
+glusterd_add_shd_to_dict(glusterd_volinfo_t *volinfo, dict_t *dict,
+                         int32_t count);
+int32_t
+glusterd_check_brick_order(dict_t *dict, char *err_str, int32_t type,
+                           char **volname, char **bricks, int32_t *brick_count,
+                           int32_t sub_count, int flag);
+gf_boolean_t
+glusterd_gf_is_local_addr(char *hostname);
+
+int
+glusterd_defrag_ref(glusterd_defrag_info_t *defrag);
+
+int
+glusterd_defrag_unref(glusterd_defrag_info_t *defrag);
 #endif
