@@ -11,7 +11,7 @@
 #ifndef __FUSE_MEM_TYPES_H__
 #define __FUSE_MEM_TYPES_H__
 
-#include "mem-types.h"
+#include <glusterfs/mem-types.h>
 
 enum gf_fuse_mem_types_ {
     gf_fuse_mt_iovec = gf_common_mt_end + 1,
@@ -24,6 +24,8 @@ enum gf_fuse_mem_types_ {
     gf_fuse_mt_gids_t,
     gf_fuse_mt_invalidate_node_t,
     gf_fuse_mt_pthread_t,
+    gf_fuse_mt_timed_message_t,
+    gf_fuse_mt_interrupt_record_t,
     gf_fuse_mt_end
 };
 #endif
