@@ -9,16 +9,16 @@
 */
 
 #include "rpcsvc.h"
-#include "dict.h"
-#include "xlator.h"
+#include <glusterfs/dict.h>
+#include <glusterfs/xlator.h>
 #include "xdr-nfs3.h"
 #include "msg-nfs3.h"
-#include "iobuf.h"
+#include <glusterfs/iobuf.h>
 #include "nfs-common.h"
 #include "nfs-fops.h"
 #include "nfs-mem-types.h"
 #include "rpcsvc.h"
-#include "iatt.h"
+#include <glusterfs/iatt.h>
 #include "nfs-messages.h"
 
 #include <libgen.h>
@@ -272,11 +272,8 @@ err:
 int
 nfs_root_loc_fill(inode_table_t *itable, loc_t *loc)
 {
-    uuid_t rootgfid = {
-        0,
-    };
+    static uuid_t rootgfid = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1};
 
-    rootgfid[15] = 1;
     return nfs_gfid_loc_fill(itable, rootgfid, loc, NFS_RESOLVE_EXIST);
 }
 
@@ -432,7 +429,7 @@ nfs_fix_generation(xlator_t *this, inode_t *inode)
     priv = this->private;
 
     if (inode_ctx_get(inode, this, &raw_ctx) == 0) {
-        ictx = (struct nfs_inode_ctx *)raw_ctx;
+        ictx = (struct nfs_inode_ctx *)(uintptr_t)raw_ctx;
         ictx->generation = priv->generation;
     } else {
         ictx = GF_CALLOC(1, sizeof(struct nfs_inode_ctx), gf_nfs_mt_inode_ctx);
@@ -443,7 +440,7 @@ nfs_fix_generation(xlator_t *this, inode_t *inode)
         }
         INIT_LIST_HEAD(&ictx->shares);
         ictx->generation = priv->generation;
-        ret = inode_ctx_put(inode, this, (uint64_t)ictx);
+        ret = inode_ctx_put(inode, this, (uint64_t)(uintptr_t)ictx);
         if (ret) {
             gf_msg(this->name, GF_LOG_ERROR, 0, NFS_MSG_INODE_CTX_STORE_FAIL,
                    "could not store nfs inode ctx");
