@@ -11,10 +11,10 @@
 #ifndef _NFS_FOPS_H_
 #define _NFS_FOPS_H_
 
-#include "dict.h"
-#include "xlator.h"
-#include "iobuf.h"
-#include "call-stub.h"
+#include <glusterfs/dict.h>
+#include <glusterfs/xlator.h>
+#include <glusterfs/iobuf.h>
+#include <glusterfs/call-stub.h>
 #include "nfs.h"
 #include "nfs-common.h"
 #include "nfs-messages.h"
