@@ -10,7 +10,7 @@
 #ifndef _NFS_MESSAGES_H_
 #define _NFS_MESSAGES_H_
 
-#include "glfs-message-id.h"
+#include <glusterfs/glfs-message-id.h>
 
 /* To add new message IDs, append new identifiers at the end of the list.
  *
