@@ -1609,7 +1609,7 @@ mem_acct_init(xlator_t *this)
     if (!this)
         return ret;
 
-    ret = xlator_mem_acct_init(this, gf_ioc_mt_end + 1);
+    ret = xlator_mem_acct_init(this, gf_ioc_mt_end);
 
     if (ret != 0) {
         gf_smsg(this->name, GF_LOG_ERROR, ENOMEM,
@@ -1673,7 +1673,7 @@ reconfigure(xlator_t *this, dict_t *options)
         GF_OPTION_RECONF("pass-through", this->pass_through, options, bool,
                          unlock);
 
-        GF_OPTION_RECONF("cache-timeout", table->cache_timeout, options, int32,
+        GF_OPTION_RECONF("cache-timeout", table->cache_timeout, options, time,
                          unlock);
 
         data = dict_get(options, "priority");
@@ -1765,7 +1765,7 @@ init(xlator_t *this)
 
     GF_OPTION_INIT("cache-size", table->cache_size, size_uint64, out);
 
-    GF_OPTION_INIT("cache-timeout", table->cache_timeout, int32, out);
+    GF_OPTION_INIT("cache-timeout", table->cache_timeout, time, out);
 
     GF_OPTION_INIT("min-file-size", table->min_file_size, size_uint64, out);
 
@@ -2056,7 +2056,7 @@ ioc_priv_dump(xlator_t *this)
         gf_proc_dump_write("cache_size", "%" PRIu64, priv->cache_size);
         gf_proc_dump_write("cache_used", "%" PRIu64, priv->cache_used);
         gf_proc_dump_write("inode_count", "%u", priv->inode_count);
-        gf_proc_dump_write("cache_timeout", "%u", priv->cache_timeout);
+        gf_proc_dump_write("cache_timeout", "%ld", priv->cache_timeout);
         gf_proc_dump_write("min-file-size", "%" PRIu64, priv->min_file_size);
         gf_proc_dump_write("max-file-size", "%" PRIu64, priv->max_file_size);
     }
