@@ -909,7 +909,7 @@ mem_acct_init(xlator_t *this)
     if (!this)
         return ret;
 
-    ret = xlator_mem_acct_init(this, gf_iot_mt_end + 1);
+    ret = xlator_mem_acct_init(this, gf_iot_mt_end);
 
     if (ret != 0) {
         gf_smsg(this->name, GF_LOG_ERROR, ENOMEM, IO_THREADS_MSG_NO_MEMORY,
@@ -942,7 +942,7 @@ iot_priv_dump(xlator_t *this)
     gf_proc_dump_write("maximum_threads_count", "%d", conf->max_count);
     gf_proc_dump_write("current_threads_count", "%d", conf->curr_count);
     gf_proc_dump_write("sleep_count", "%d", conf->sleep_count);
-    gf_proc_dump_write("idle_time", "%d", conf->idle_time);
+    gf_proc_dump_write("idle_time", "%ld", conf->idle_time);
     gf_proc_dump_write("stack_size", "%zd", conf->stack_size);
     gf_proc_dump_write("max_high_priority_threads", "%d",
                        conf->ac_iot_limit[GF_FOP_PRI_HI]);
@@ -1235,7 +1235,7 @@ init(xlator_t *this)
     GF_OPTION_INIT("least-prio-threads", conf->ac_iot_limit[GF_FOP_PRI_LEAST],
                    int32, out);
 
-    GF_OPTION_INIT("idle-time", conf->idle_time, int32, out);
+    GF_OPTION_INIT("idle-time", conf->idle_time, time, out);
 
     GF_OPTION_INIT("enable-least-priority", conf->least_priority, bool, out);
 
@@ -1477,61 +1477,54 @@ struct volume_options options[] = {
      .type = GF_OPTION_TYPE_INT,
      .min = IOT_MIN_THREADS,
      .max = IOT_MAX_THREADS,
-     .default_value = "16",
+     .default_value = TOSTRING(IOT_DEFAULT_THREADS),
      .op_version = {1},
-     .flags = OPT_FLAG_SETTABLE | OPT_FLAG_DOC,
+     .flags = OPT_FLAG_SETTABLE | OPT_FLAG_DOC | OPT_FLAG_RANGE,
      .tags = {"io-threads"},
-     /*.option = "thread-count"*/
      .description = "Number of threads in IO threads translator which "
-                    "perform concurrent IO operations"
+                    "perform concurrent IO operations."
 
     },
     {.key = {"high-prio-threads"},
      .type = GF_OPTION_TYPE_INT,
      .min = IOT_MIN_THREADS,
      .max = IOT_MAX_THREADS,
-     .default_value = "16",
+     .default_value = TOSTRING(IOT_DEFAULT_THREADS),
      .op_version = {1},
-     .flags = OPT_FLAG_SETTABLE | OPT_FLAG_DOC,
+     .flags = OPT_FLAG_SETTABLE | OPT_FLAG_DOC | OPT_FLAG_RANGE,
      .tags = {"io-threads"},
      .description = "Max number of threads in IO threads translator which "
-                    "perform high priority IO operations at a given time"
-
-    },
+                    "perform high priority IO operations at a given time."},
     {.key = {"normal-prio-threads"},
      .type = GF_OPTION_TYPE_INT,
      .min = IOT_MIN_THREADS,
      .max = IOT_MAX_THREADS,
-     .default_value = "16",
+     .default_value = TOSTRING(IOT_DEFAULT_THREADS),
      .op_version = {1},
-     .flags = OPT_FLAG_SETTABLE | OPT_FLAG_DOC,
+     .flags = OPT_FLAG_SETTABLE | OPT_FLAG_DOC | OPT_FLAG_RANGE,
      .tags = {"io-threads"},
      .description = "Max number of threads in IO threads translator which "
-                    "perform normal priority IO operations at a given time"
-
-    },
+                    "perform normal priority IO operations at a given time."},
     {.key = {"low-prio-threads"},
      .type = GF_OPTION_TYPE_INT,
      .min = IOT_MIN_THREADS,
      .max = IOT_MAX_THREADS,
-     .default_value = "16",
+     .default_value = TOSTRING(IOT_DEFAULT_THREADS),
      .op_version = {1},
-     .flags = OPT_FLAG_SETTABLE | OPT_FLAG_DOC,
+     .flags = OPT_FLAG_SETTABLE | OPT_FLAG_DOC | OPT_FLAG_RANGE,
      .tags = {"io-threads"},
      .description = "Max number of threads in IO threads translator which "
-                    "perform low priority IO operations at a given time"
-
-    },
+                    "perform low priority IO operations at a given time."},
     {.key = {"least-prio-threads"},
      .type = GF_OPTION_TYPE_INT,
      .min = IOT_MIN_THREADS,
      .max = IOT_MAX_THREADS,
-     .default_value = "1",
+     .default_value = TOSTRING(IOT_MIN_THREADS),
      .op_version = {1},
-     .flags = OPT_FLAG_SETTABLE | OPT_FLAG_DOC,
+     .flags = OPT_FLAG_SETTABLE | OPT_FLAG_DOC | OPT_FLAG_RANGE,
      .tags = {"io-threads"},
      .description = "Max number of threads in IO threads translator which "
-                    "perform least priority IO operations at a given time"},
+                    "perform least priority IO operations at a given time."},
     {.key = {"enable-least-priority"},
      .type = GF_OPTION_TYPE_BOOL,
      .default_value = SITE_H_ENABLE_LEAST_PRIORITY,
