@@ -11,7 +11,7 @@
 #ifndef __RDA_MEM_TYPES_H__
 #define __RDA_MEM_TYPES_H__
 
-#include "mem-types.h"
+#include <glusterfs/mem-types.h>
 
 enum gf_rda_mem_types_ {
     gf_rda_mt_rda_local = gf_common_mt_end + 1,
