@@ -343,7 +343,7 @@ client_pre_readv(xlator_t *this, gfs3_read_req *req, fd_t *fd, size_t size,
     int op_errno = ESTALE;
 
     CLIENT_GET_REMOTE_FD(this, fd, FALLBACK_TO_ANON_FD, remote_fd, op_errno,
-                         out);
+                         GFS3_OP_READ, out);
 
     req->size = size;
     req->offset = offset;
@@ -368,7 +368,7 @@ client_pre_writev(xlator_t *this, gfs3_write_req *req, fd_t *fd, size_t size,
     int op_errno = ESTALE;
 
     CLIENT_GET_REMOTE_FD(this, fd, FALLBACK_TO_ANON_FD, remote_fd, op_errno,
-                         out);
+                         GFS3_OP_WRITE, out);
 
     req->size = size;
     req->offset = offset;
@@ -429,7 +429,8 @@ client_pre_flush(xlator_t *this, gfs3_flush_req *req, fd_t *fd, dict_t *xdata)
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FLUSH, out);
 
     req->fd = remote_fd;
     memcpy(req->gfid, fd->inode->gfid, 16);
@@ -449,7 +450,8 @@ client_pre_fsync(xlator_t *this, gfs3_fsync_req *req, fd_t *fd, int32_t flags,
     int64_t remote_fd = -1;
     int op_errno = 0;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, FALLBACK_TO_ANON_FD, remote_fd, op_errno,
+                         GFS3_OP_FSYNC, out);
 
     req->fd = remote_fd;
     req->data = flags;
@@ -590,7 +592,8 @@ client_pre_fsyncdir(xlator_t *this, gfs3_fsyncdir_req *req, fd_t *fd,
     int32_t op_errno = ESTALE;
     int64_t remote_fd = -1;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FSYNCDIR, out);
 
     req->fd = remote_fd;
     req->data = flags;
@@ -667,7 +670,8 @@ client_pre_ftruncate(xlator_t *this, gfs3_ftruncate_req *req, fd_t *fd,
     int64_t remote_fd = -1;
     int op_errno = EINVAL;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FTRUNCATE, out);
 
     req->offset = offset;
     req->fd = remote_fd;
@@ -686,7 +690,8 @@ client_pre_fstat(xlator_t *this, gfs3_fstat_req *req, fd_t *fd, dict_t *xdata)
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FSTAT, out);
 
     req->fd = remote_fd;
     memcpy(req->gfid, fd->inode->gfid, 16);
@@ -709,13 +714,14 @@ client_pre_lk(xlator_t *this, gfs3_lk_req *req, int32_t cmd,
     int32_t gf_type = 0;
     int ret = 0;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_LK, out);
 
     ret = client_cmd_to_gf_cmd(cmd, &gf_cmd);
     if (ret) {
         op_errno = EINVAL;
-        gf_msg(this->name, GF_LOG_WARNING, EINVAL, PC_MSG_INVALID_ENTRY,
-               "Unknown cmd (%d)!", gf_cmd);
+        gf_smsg(this->name, GF_LOG_WARNING, EINVAL, PC_MSG_UNKNOWN_CMD,
+                "gf_cmd=%d", gf_cmd, NULL);
         goto out;
     }
 
@@ -786,7 +792,8 @@ client_pre_readdir(xlator_t *this, gfs3_readdir_req *req, fd_t *fd, size_t size,
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_READDIR, out);
 
     req->size = size;
     req->offset = offset;
@@ -827,8 +834,8 @@ client_pre_inodelk(xlator_t *this, gfs3_inodelk_req *req, loc_t *loc, int cmd,
     else if (cmd == F_SETLKW || cmd == F_SETLKW64)
         gf_cmd = GF_LK_SETLKW;
     else {
-        gf_msg(this->name, GF_LOG_WARNING, EINVAL, PC_MSG_INVALID_ENTRY,
-               "Unknown cmd (%d)!", gf_cmd);
+        gf_smsg(this->name, GF_LOG_WARNING, EINVAL, PC_MSG_UNKNOWN_CMD,
+                "gf_cmd=%d", gf_cmd, NULL);
         op_errno = EINVAL;
         goto out;
     }
@@ -868,7 +875,7 @@ client_pre_finodelk(xlator_t *this, gfs3_finodelk_req *req, fd_t *fd, int cmd,
     int32_t gf_cmd = 0;
 
     CLIENT_GET_REMOTE_FD(this, fd, FALLBACK_TO_ANON_FD, remote_fd, op_errno,
-                         out);
+                         GFS3_OP_FINODELK, out);
 
     if (cmd == F_GETLK || cmd == F_GETLK64)
         gf_cmd = GF_LK_GETLK;
@@ -877,8 +884,8 @@ client_pre_finodelk(xlator_t *this, gfs3_finodelk_req *req, fd_t *fd, int cmd,
     else if (cmd == F_SETLKW || cmd == F_SETLKW64)
         gf_cmd = GF_LK_SETLKW;
     else {
-        gf_msg(this->name, GF_LOG_WARNING, EINVAL, PC_MSG_INVALID_ENTRY,
-               "Unknown cmd (%d)!", gf_cmd);
+        gf_smsg(this->name, GF_LOG_WARNING, EINVAL, PC_MSG_UNKNOWN_CMD,
+                "gf_cmd=%d", gf_cmd, NULL);
         goto out;
     }
 
@@ -951,7 +958,8 @@ client_pre_fentrylk(xlator_t *this, gfs3_fentrylk_req *req, fd_t *fd,
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FENTRYLK, out);
 
     req->fd = remote_fd;
     req->cmd = cmd_entrylk;
@@ -1012,7 +1020,7 @@ client_pre_fxattrop(xlator_t *this, gfs3_fxattrop_req *req, fd_t *fd,
     int64_t remote_fd = -1;
 
     CLIENT_GET_REMOTE_FD(this, fd, FALLBACK_TO_ANON_FD, remote_fd, op_errno,
-                         out);
+                         GFS3_OP_FXATTROP, out);
 
     req->fd = remote_fd;
     req->flags = flags;
@@ -1038,7 +1046,8 @@ client_pre_fgetxattr(xlator_t *this, gfs3_fgetxattr_req *req, fd_t *fd,
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FGETXATTR, out);
 
     req->namelen = 1; /* Use it as a flag */
     req->fd = remote_fd;
@@ -1064,7 +1073,8 @@ client_pre_fsetxattr(xlator_t *this, gfs3_fsetxattr_req *req, fd_t *fd,
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FSETXATTR, out);
 
     req->fd = remote_fd;
     req->flags = flags;
@@ -1090,7 +1100,8 @@ client_pre_rchecksum(xlator_t *this, gfs3_rchecksum_req *req, fd_t *fd,
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_RCHECKSUM, out);
 
     req->len = len;
     req->offset = offset;
@@ -1140,7 +1151,8 @@ client_pre_fsetattr(xlator_t *this, gfs3_fsetattr_req *req, fd_t *fd,
     int op_errno = ESTALE;
     int64_t remote_fd = -1;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FSETATTR, out);
 
     req->fd = remote_fd;
     req->valid = valid;
@@ -1160,7 +1172,8 @@ client_pre_readdirp(xlator_t *this, gfs3_readdirp_req *req, fd_t *fd,
     int op_errno = ESTALE;
     int64_t remote_fd = -1;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_READDIRP, out);
 
     req->size = size;
     req->offset = offset;
@@ -1186,7 +1199,8 @@ client_pre_fremovexattr(xlator_t *this, gfs3_fremovexattr_req *req, fd_t *fd,
     if (!(fd && fd->inode))
         goto out;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FREMOVEXATTR, out);
 
     memcpy(req->gfid, fd->inode->gfid, 16);
     req->name = (char *)name;
@@ -1207,7 +1221,8 @@ client_pre_fallocate(xlator_t *this, gfs3_fallocate_req *req, fd_t *fd,
     int op_errno = ESTALE;
     int64_t remote_fd = -1;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FALLOCATE, out);
 
     req->fd = remote_fd;
     req->flags = flags;
@@ -1229,7 +1244,8 @@ client_pre_discard(xlator_t *this, gfs3_discard_req *req, fd_t *fd,
     int op_errno = ESTALE;
     int64_t remote_fd = -1;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_DISCARD, out);
 
     req->fd = remote_fd;
     req->offset = offset;
@@ -1250,7 +1266,8 @@ client_pre_zerofill(xlator_t *this, gfs3_zerofill_req *req, fd_t *fd,
     int op_errno = ESTALE;
     int64_t remote_fd = -1;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_ZEROFILL, out);
 
     req->fd = remote_fd;
     req->offset = offset;
@@ -1285,7 +1302,8 @@ client_pre_seek(xlator_t *this, gfs3_seek_req *req, fd_t *fd, off_t offset,
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_SEEK, out);
 
     memcpy(req->gfid, fd->inode->gfid, 16);
     req->fd = remote_fd;
@@ -2507,7 +2525,7 @@ client_pre_readv_v2(xlator_t *this, gfx_read_req *req, fd_t *fd, size_t size,
     int op_errno = ESTALE;
 
     CLIENT_GET_REMOTE_FD(this, fd, FALLBACK_TO_ANON_FD, remote_fd, op_errno,
-                         out);
+                         GFS3_OP_READ, out);
 
     req->size = size;
     req->offset = offset;
@@ -2531,7 +2549,7 @@ client_pre_writev_v2(xlator_t *this, gfx_write_req *req, fd_t *fd, size_t size,
     int op_errno = ESTALE;
 
     CLIENT_GET_REMOTE_FD(this, fd, FALLBACK_TO_ANON_FD, remote_fd, op_errno,
-                         out);
+                         GFS3_OP_WRITE, out);
 
     req->size = size;
     req->offset = offset;
@@ -2566,10 +2584,10 @@ client_pre_copy_file_range_v2(xlator_t *this, gfx_copy_file_range_req *req,
     int op_errno = ESTALE;
 
     CLIENT_GET_REMOTE_FD(this, fd_in, FALLBACK_TO_ANON_FD, remote_fd_in,
-                         op_errno, out);
+                         op_errno, GFS3_OP_COPY_FILE_RANGE, out);
 
     CLIENT_GET_REMOTE_FD(this, fd_out, FALLBACK_TO_ANON_FD, remote_fd_out,
-                         op_errno, out);
+                         op_errno, GFS3_OP_COPY_FILE_RANGE, out);
     req->size = size;
     req->off_in = off_in;
     req->off_out = off_out;
@@ -2622,7 +2640,8 @@ client_pre_flush_v2(xlator_t *this, gfx_flush_req *req, fd_t *fd, dict_t *xdata)
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FLUSH, out);
 
     req->fd = remote_fd;
     memcpy(req->gfid, fd->inode->gfid, 16);
@@ -2641,7 +2660,8 @@ client_pre_fsync_v2(xlator_t *this, gfx_fsync_req *req, fd_t *fd, int32_t flags,
     int64_t remote_fd = -1;
     int op_errno = 0;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, FALLBACK_TO_ANON_FD, remote_fd, op_errno,
+                         GFS3_OP_FSYNC, out);
 
     req->fd = remote_fd;
     req->data = flags;
@@ -2776,7 +2796,8 @@ client_pre_fsyncdir_v2(xlator_t *this, gfx_fsyncdir_req *req, fd_t *fd,
     int32_t op_errno = ESTALE;
     int64_t remote_fd = -1;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FSYNCDIR, out);
 
     req->fd = remote_fd;
     req->data = flags;
@@ -2850,7 +2871,8 @@ client_pre_ftruncate_v2(xlator_t *this, gfx_ftruncate_req *req, fd_t *fd,
     int64_t remote_fd = -1;
     int op_errno = EINVAL;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FTRUNCATE, out);
 
     req->offset = offset;
     req->fd = remote_fd;
@@ -2868,7 +2890,8 @@ client_pre_fstat_v2(xlator_t *this, gfx_fstat_req *req, fd_t *fd, dict_t *xdata)
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FSTAT, out);
 
     req->fd = remote_fd;
     memcpy(req->gfid, fd->inode->gfid, 16);
@@ -2890,13 +2913,14 @@ client_pre_lk_v2(xlator_t *this, gfx_lk_req *req, int32_t cmd,
     int32_t gf_type = 0;
     int ret = 0;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_LK, out);
 
     ret = client_cmd_to_gf_cmd(cmd, &gf_cmd);
     if (ret) {
         op_errno = EINVAL;
-        gf_msg(this->name, GF_LOG_WARNING, EINVAL, PC_MSG_INVALID_ENTRY,
-               "Unknown cmd (%d)!", gf_cmd);
+        gf_smsg(this->name, GF_LOG_WARNING, EINVAL, PC_MSG_UNKNOWN_CMD,
+                "gf_cmd=%d", gf_cmd, NULL);
         goto out;
     }
 
@@ -2965,7 +2989,8 @@ client_pre_readdir_v2(xlator_t *this, gfx_readdir_req *req, fd_t *fd,
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_READDIR, out);
 
     req->size = size;
     req->offset = offset;
@@ -3005,8 +3030,8 @@ client_pre_inodelk_v2(xlator_t *this, gfx_inodelk_req *req, loc_t *loc, int cmd,
     else if (cmd == F_SETLKW || cmd == F_SETLKW64)
         gf_cmd = GF_LK_SETLKW;
     else {
-        gf_msg(this->name, GF_LOG_WARNING, EINVAL, PC_MSG_INVALID_ENTRY,
-               "Unknown cmd (%d)!", gf_cmd);
+        gf_smsg(this->name, GF_LOG_WARNING, EINVAL, PC_MSG_UNKNOWN_CMD,
+                "gf_cmd=%d", gf_cmd, NULL);
         op_errno = EINVAL;
         goto out;
     }
@@ -3046,7 +3071,7 @@ client_pre_finodelk_v2(xlator_t *this, gfx_finodelk_req *req, fd_t *fd, int cmd,
     int32_t gf_cmd = 0;
 
     CLIENT_GET_REMOTE_FD(this, fd, FALLBACK_TO_ANON_FD, remote_fd, op_errno,
-                         out);
+                         GFS3_OP_FINODELK, out);
 
     if (cmd == F_GETLK || cmd == F_GETLK64)
         gf_cmd = GF_LK_GETLK;
@@ -3055,8 +3080,8 @@ client_pre_finodelk_v2(xlator_t *this, gfx_finodelk_req *req, fd_t *fd, int cmd,
     else if (cmd == F_SETLKW || cmd == F_SETLKW64)
         gf_cmd = GF_LK_SETLKW;
     else {
-        gf_msg(this->name, GF_LOG_WARNING, EINVAL, PC_MSG_INVALID_ENTRY,
-               "Unknown cmd (%d)!", gf_cmd);
+        gf_smsg(this->name, GF_LOG_WARNING, EINVAL, PC_MSG_UNKNOWN_CMD,
+                "gf_cmd=%d", gf_cmd, NULL);
         goto out;
     }
 
@@ -3127,7 +3152,8 @@ client_pre_fentrylk_v2(xlator_t *this, gfx_fentrylk_req *req, fd_t *fd,
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FENTRYLK, out);
 
     req->fd = remote_fd;
     req->cmd = cmd_entrylk;
@@ -3183,7 +3209,7 @@ client_pre_fxattrop_v2(xlator_t *this, gfx_fxattrop_req *req, fd_t *fd,
     int64_t remote_fd = -1;
 
     CLIENT_GET_REMOTE_FD(this, fd, FALLBACK_TO_ANON_FD, remote_fd, op_errno,
-                         out);
+                         GFS3_OP_FXATTROP, out);
 
     req->fd = remote_fd;
     req->flags = flags;
@@ -3205,7 +3231,8 @@ client_pre_fgetxattr_v2(xlator_t *this, gfx_fgetxattr_req *req, fd_t *fd,
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FGETXATTR, out);
 
     req->namelen = 1; /* Use it as a flag */
     req->fd = remote_fd;
@@ -3230,7 +3257,8 @@ client_pre_fsetxattr_v2(xlator_t *this, gfx_fsetxattr_req *req, fd_t *fd,
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FSETXATTR, out);
 
     req->fd = remote_fd;
     req->flags = flags;
@@ -3254,7 +3282,8 @@ client_pre_rchecksum_v2(xlator_t *this, gfx_rchecksum_req *req, fd_t *fd,
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_RCHECKSUM, out);
 
     req->len = len;
     req->offset = offset;
@@ -3302,7 +3331,8 @@ client_pre_fsetattr_v2(xlator_t *this, gfx_fsetattr_req *req, fd_t *fd,
     int op_errno = ESTALE;
     int64_t remote_fd = -1;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FSETATTR, out);
 
     memcpy(req->gfid, fd->inode->gfid, 16);
     req->fd = remote_fd;
@@ -3322,7 +3352,8 @@ client_pre_readdirp_v2(xlator_t *this, gfx_readdirp_req *req, fd_t *fd,
     int op_errno = ESTALE;
     int64_t remote_fd = -1;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_READDIRP, out);
 
     req->size = size;
     req->offset = offset;
@@ -3347,7 +3378,8 @@ client_pre_fremovexattr_v2(xlator_t *this, gfx_fremovexattr_req *req, fd_t *fd,
     if (!(fd && fd->inode))
         goto out;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FREMOVEXATTR, out);
 
     memcpy(req->gfid, fd->inode->gfid, 16);
     req->name = (char *)name;
@@ -3367,7 +3399,8 @@ client_pre_fallocate_v2(xlator_t *this, gfx_fallocate_req *req, fd_t *fd,
     int op_errno = ESTALE;
     int64_t remote_fd = -1;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_FALLOCATE, out);
 
     req->fd = remote_fd;
     req->flags = flags;
@@ -3388,7 +3421,8 @@ client_pre_discard_v2(xlator_t *this, gfx_discard_req *req, fd_t *fd,
     int op_errno = ESTALE;
     int64_t remote_fd = -1;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_DISCARD, out);
 
     req->fd = remote_fd;
     req->offset = offset;
@@ -3408,7 +3442,8 @@ client_pre_zerofill_v2(xlator_t *this, gfx_zerofill_req *req, fd_t *fd,
     int op_errno = ESTALE;
     int64_t remote_fd = -1;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_ZEROFILL, out);
 
     req->fd = remote_fd;
     req->offset = offset;
@@ -3437,7 +3472,8 @@ client_pre_seek_v2(xlator_t *this, gfx_seek_req *req, fd_t *fd, off_t offset,
     int64_t remote_fd = -1;
     int op_errno = ESTALE;
 
-    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno, out);
+    CLIENT_GET_REMOTE_FD(this, fd, DEFAULT_REMOTE_FD, remote_fd, op_errno,
+                         GFS3_OP_SEEK, out);
 
     memcpy(req->gfid, fd->inode->gfid, 16);
     req->fd = remote_fd;
@@ -3585,3 +3621,24 @@ client_post_rename_v2(xlator_t *this, gfx_rename_rsp *rsp, struct iatt *stbuf,
 
     return xdr_to_dict(&rsp->xdata, xdata);
 }
+
+void
+set_fd_reopen_status(xlator_t *this, dict_t *xdata,
+                     enum gf_fd_reopen_status fd_reopen_status)
+{
+    clnt_conf_t *conf = NULL;
+
+    conf = this->private;
+    if (!conf) {
+        gf_msg_debug(this->name, ENOMEM, "Failed to get client conf");
+        return;
+    }
+
+    if (!conf->strict_locks)
+        fd_reopen_status = FD_REOPEN_ALLOWED;
+
+    if (dict_set_int32(xdata, "fd-reopen-status", fd_reopen_status))
+        gf_smsg(this->name, GF_LOG_WARNING, ENOMEM, PC_MSG_NO_MEM, NULL);
+
+    return;
+}
