@@ -12,13 +12,13 @@
 #include "server-helpers.h"
 #include "rpc-common-xdr.h"
 #include "glusterfs3-xdr.h"
-#include "compat-errno.h"
+#include <glusterfs/compat-errno.h>
 #include "glusterfs3.h"
 #include "authenticate.h"
 #include "server-messages.h"
-#include "syscall.h"
-#include "events.h"
-#include "syncop.h"
+#include <glusterfs/syscall.h>
+#include <glusterfs/events.h>
+#include <glusterfs/syncop.h>
 
 struct __get_xl_struct {
     const char *name;
@@ -36,7 +36,6 @@ gf_compare_client_version(rpcsvc_request_t *req, int fop_prognum,
     return ret;
 }
 
-
 int
 server_getspec(rpcsvc_request_t *req)
 {
@@ -94,7 +93,7 @@ do_path_lookup(xlator_t *xl, dict_t *dict, inode_t *parinode, char *basename)
     };
     inode_t *inode = NULL;
 
-    loc.parent = parinode;
+    loc.parent = inode_ref(parinode);
     loc_touchup(&loc, basename);
     loc.inode = inode_new(xl->itable);
 
@@ -123,6 +122,7 @@ do_path_lookup(xlator_t *xl, dict_t *dict, inode_t *parinode, char *basename)
     inode_ref(inode);
 
 out:
+    loc_wipe(&loc);
     return inode;
 }
 
@@ -232,6 +232,7 @@ server_setvolume(rpcsvc_request_t *req)
     dict_t *config_params = NULL;
     dict_t *params = NULL;
     char *name = NULL;
+    char *volume_id = NULL;
     char *client_uid = NULL;
     char *clnt_version = NULL;
     xlator_t *xl = NULL;
@@ -240,7 +241,6 @@ server_setvolume(rpcsvc_request_t *req)
     int32_t ret = -1;
     int32_t op_ret = -1;
     int32_t op_errno = EINVAL;
-    char *buf = NULL;
     uint32_t opversion = 0;
     rpc_transport_t *xprt = NULL;
     int32_t fop_version = 0;
@@ -250,6 +250,7 @@ server_setvolume(rpcsvc_request_t *req)
     char *subdir_mount = NULL;
     char *client_name = NULL;
     gf_boolean_t cleanup_starting = _gf_false;
+    gf_boolean_t xlator_in_graph = _gf_true;
 
     params = dict_new();
     reply = dict_new();
@@ -267,18 +268,11 @@ server_setvolume(rpcsvc_request_t *req)
      */
     config_params = dict_copy_with_ref(this->options, NULL);
 
-    buf = memdup(args.dict.dict_val, args.dict.dict_len);
-    if (buf == NULL) {
-        op_ret = -1;
-        op_errno = ENOMEM;
-        goto fail;
-    }
-
-    ret = dict_unserialize(buf, args.dict.dict_len, &params);
+    ret = dict_unserialize(args.dict.dict_val, args.dict.dict_len, &params);
     if (ret < 0) {
-        ret = dict_set_str(reply, "ERROR",
-                           "Internal error: failed to unserialize "
-                           "request dictionary");
+        ret = dict_set_sizen_str_sizen(reply, "ERROR",
+                                       "Internal error: failed to unserialize "
+                                       "request dictionary");
         if (ret < 0)
             gf_msg_debug(this->name, 0,
                          "failed to set error "
@@ -291,9 +285,6 @@ server_setvolume(rpcsvc_request_t *req)
         goto fail;
     }
 
-    params->extra_free = buf;
-    buf = NULL;
-
     ret = dict_get_str(params, "remote-subvolume", &name);
     if (ret < 0) {
         ret = dict_set_str(reply, "ERROR",
@@ -311,11 +302,18 @@ server_setvolume(rpcsvc_request_t *req)
     LOCK(&ctx->volfile_lock);
     {
         xl = get_xlator_by_name(this, name);
-        if (!xl)
+        if (!xl) {
+            xlator_in_graph = _gf_false;
             xl = this;
+        }
+        if (ctx->cleanup_starting) {
+            cleanup_starting = _gf_true;
+            op_ret = -1;
+            op_errno = ENOENT;
+        }
     }
     UNLOCK(&ctx->volfile_lock);
-    if (xl == NULL) {
+    if (!xl || cleanup_starting) {
         ret = gf_asprintf(&msg, "remote-subvolume \"%s\" is not found", name);
         if (-1 == ret) {
             gf_msg(this->name, GF_LOG_ERROR, 0, PS_MSG_ASPRINTF_FAILED,
@@ -354,6 +352,7 @@ server_setvolume(rpcsvc_request_t *req)
         goto fail;
     }
 
+    pthread_mutex_lock(&conf->mutex);
     list_for_each_entry(tmp, &conf->child_status->status_list, status_list)
     {
         if (strcmp(tmp->name, name) == 0)
@@ -361,10 +360,8 @@ server_setvolume(rpcsvc_request_t *req)
     }
 
     if (!tmp->name) {
-        gf_msg(this->name, GF_LOG_ERROR, 0, PS_MSG_CHILD_STATUS_FAILED,
-               "No xlator %s is found in "
-               "child status list",
-               name);
+        gf_msg(this->name, GF_LOG_INFO, 0, PS_MSG_CHILD_STATUS_FAILED,
+               "No xlator %s is found in child status list", name);
     } else {
         ret = dict_set_int32(reply, "child_up", tmp->child_up);
         if (ret < 0)
@@ -372,7 +369,21 @@ server_setvolume(rpcsvc_request_t *req)
                    "Failed to set 'child_up' for xlator %s "
                    "in the reply dict",
                    tmp->name);
+        if (!tmp->child_up) {
+            ret = dict_set_str(reply, "ERROR",
+                               "Not received child_up for this xlator");
+            if (ret < 0)
+                gf_msg_debug(this->name, 0, "failed to set error msg");
+
+            gf_msg(this->name, GF_LOG_ERROR, 0, PS_MSG_CHILD_STATUS_FAILED,
+                   "Not received child_up for this xlator %s", name);
+            op_ret = -1;
+            op_errno = EAGAIN;
+            pthread_mutex_unlock(&conf->mutex);
+            goto fail;
+        }
     }
+    pthread_mutex_unlock(&conf->mutex);
 
     ret = dict_get_str(params, "process-uuid", &client_uid);
     if (ret < 0) {
@@ -396,6 +407,25 @@ server_setvolume(rpcsvc_request_t *req)
         client_name = "unknown";
     }
 
+    /* If any value is set, the first element will be non-0.
+       It would be '0', but not '\0' :-) */
+    if (xl->graph->volume_id[0]) {
+        ret = dict_get_str_sizen(params, "volume-id", &volume_id);
+        if (!ret && strcmp(xl->graph->volume_id, volume_id)) {
+            ret = dict_set_str(reply, "ERROR",
+                               "Volume-ID different, possible case "
+                               "of same brick re-used in another volume");
+            if (ret < 0)
+                gf_msg_debug(this->name, 0, "failed to set error msg");
+
+            op_ret = -1;
+            op_errno = EINVAL;
+            goto fail;
+        }
+        ret = dict_set_str(reply, "volume-id", tmp->volume_id);
+        if (ret)
+            gf_msg_debug(this->name, 0, "failed to set 'volume-id'");
+    }
     client = gf_client_get(this, &req->cred, client_uid, subdir_mount);
     if (client == NULL) {
         op_ret = -1;
@@ -533,8 +563,8 @@ server_setvolume(rpcsvc_request_t *req)
         req->trans->clnt_options = dict_ref(params);
 
         gf_msg(this->name, GF_LOG_INFO, 0, PS_MSG_CLIENT_ACCEPTED,
-               "accepted client from %s (version: %s)", client->client_uid,
-               (clnt_version) ? clnt_version : "old");
+               "accepted client from %s (version: %s) with subvol %s",
+               client->client_uid, (clnt_version) ? clnt_version : "old", name);
 
         gf_event(EVENT_CLIENT_CONNECT,
                  "client_uid=%s;"
@@ -555,20 +585,30 @@ server_setvolume(rpcsvc_request_t *req)
                          "failed to set error "
                          "msg");
     } else {
-        gf_event(EVENT_CLIENT_AUTH_REJECT,
-                 "client_uid=%s;"
-                 "client_identifier=%s;server_identifier=%s;"
-                 "brick_path=%s",
-                 client->client_uid, req->trans->peerinfo.identifier,
-                 req->trans->myinfo.identifier, name);
-        gf_msg(this->name, GF_LOG_ERROR, EACCES, PS_MSG_AUTHENTICATE_ERROR,
-               "Cannot authenticate client"
-               " from %s %s",
-               client->client_uid, (clnt_version) ? clnt_version : "old");
-
         op_ret = -1;
-        op_errno = EACCES;
-        ret = dict_set_str(reply, "ERROR", "Authentication failed");
+        if (!xlator_in_graph) {
+            gf_msg(this->name, GF_LOG_ERROR, ENOENT, PS_MSG_AUTHENTICATE_ERROR,
+                   "Cannot authenticate client"
+                   " from %s %s because brick is not attached in graph",
+                   client->client_uid, (clnt_version) ? clnt_version : "old");
+
+            op_errno = ENOENT;
+            ret = dict_set_str(reply, "ERROR", "Brick not found");
+        } else {
+            gf_event(EVENT_CLIENT_AUTH_REJECT,
+                     "client_uid=%s;"
+                     "client_identifier=%s;server_identifier=%s;"
+                     "brick_path=%s",
+                     client->client_uid, req->trans->peerinfo.identifier,
+                     req->trans->myinfo.identifier, name);
+            gf_msg(this->name, GF_LOG_ERROR, EACCES, PS_MSG_AUTHENTICATE_ERROR,
+                   "Cannot authenticate client"
+                   " from %s %s",
+                   client->client_uid, (clnt_version) ? clnt_version : "old");
+
+            op_errno = EACCES;
+            ret = dict_set_str(reply, "ERROR", "Authentication failed");
+        }
         if (ret < 0)
             gf_msg_debug(this->name, 0,
                          "failed to set error "
@@ -603,7 +643,7 @@ server_setvolume(rpcsvc_request_t *req)
 
             /* TODO: what is this ? */
             client->bound_xl->itable = inode_table_new(conf->inode_lru_limit,
-                                                       client->bound_xl);
+                                                       client->bound_xl, 0, 0);
         }
     }
     UNLOCK(&conf->itable_lock);
@@ -638,6 +678,8 @@ fail:
             ret = dict_set_str(reply, "ERROR",
                                "cleanup flag is set for xlator "
                                "before call first_lookup Try again later");
+            /* quisce coverity about UNUSED_VALUE ret */
+            (void)(ret);
         } else {
             op_ret = server_first_lookup(this, client, reply);
             if (op_ret == -1)
@@ -649,21 +691,16 @@ fail:
     GF_ASSERT(rsp);
 
     rsp->op_ret = 0;
-    rsp->dict.dict_len = dict_serialized_length(reply);
-    if (rsp->dict.dict_len) {
-        rsp->dict.dict_val = GF_CALLOC(1, rsp->dict.dict_len,
-                                       gf_server_mt_rsp_buf_t);
-        if (rsp->dict.dict_val) {
-            ret = dict_serialize(reply, rsp->dict.dict_val);
-            if (ret < 0) {
-                gf_msg_debug("server-handshake", 0,
-                             "failed "
-                             "to serialize reply dict");
-                op_ret = -1;
-                op_errno = -ret;
-            }
-        }
+
+    ret = dict_allocate_and_serialize(reply, (char **)&rsp->dict.dict_val,
+                                      &rsp->dict.dict_len);
+    if (ret != 0) {
+        ret = -1;
+        gf_msg_debug("server-handshake", 0, "failed to serialize reply dict");
+        op_ret = -1;
+        op_errno = -ret;
     }
+
     rsp->op_ret = op_ret;
     rsp->op_errno = gf_errno_to_error(op_errno);
 
@@ -698,8 +735,6 @@ fail:
         dict_unref(config_params);
     }
 
-    GF_FREE(buf);
-
     return 0;
 }
 
@@ -747,15 +782,15 @@ fail:
     return 0;
 }
 
-rpcsvc_actor_t gluster_handshake_actors[GF_HNDSK_MAXVALUE] = {
-    [GF_HNDSK_NULL] = {"NULL", GF_HNDSK_NULL, server_null, NULL, 0, DRC_NA},
-    [GF_HNDSK_SETVOLUME] = {"SETVOLUME", GF_HNDSK_SETVOLUME, server_setvolume,
-                            NULL, 0, DRC_NA},
-    [GF_HNDSK_GETSPEC] = {"GETSPEC", GF_HNDSK_GETSPEC, server_getspec, NULL, 0,
-                          DRC_NA},
-    [GF_HNDSK_PING] = {"PING", GF_HNDSK_PING, server_ping, NULL, 0, DRC_NA},
-    [GF_HNDSK_SET_LK_VER] = {"SET_LK_VER", GF_HNDSK_SET_LK_VER,
-                             server_set_lk_version, NULL, 0, DRC_NA},
+static rpcsvc_actor_t gluster_handshake_actors[GF_HNDSK_MAXVALUE] = {
+    [GF_HNDSK_NULL] = {"NULL", server_null, NULL, GF_HNDSK_NULL, DRC_NA, 0},
+    [GF_HNDSK_SETVOLUME] = {"SETVOLUME", server_setvolume, NULL,
+                            GF_HNDSK_SETVOLUME, DRC_NA, 0},
+    [GF_HNDSK_GETSPEC] = {"GETSPEC", server_getspec, NULL, GF_HNDSK_GETSPEC,
+                          DRC_NA, 0},
+    [GF_HNDSK_PING] = {"PING", server_ping, NULL, GF_HNDSK_PING, DRC_NA, 0},
+    [GF_HNDSK_SET_LK_VER] = {"SET_LK_VER", server_set_lk_version, NULL,
+                             GF_HNDSK_SET_LK_VER, DRC_NA, 0},
 };
 
 struct rpcsvc_program gluster_handshake_prog = {
