@@ -236,7 +236,7 @@ mem_acct_init(xlator_t *this)
     if (!this)
         return ret;
 
-    ret = xlator_mem_acct_init(this, gf_posix_mt_end + 1);
+    ret = xlator_mem_acct_init(this, gf_posix_mt_end);
 
     if (ret != 0) {
         return ret;
@@ -441,9 +441,9 @@ posix_reconfigure(xlator_t *this, dict_t *options)
     }
 
     GF_OPTION_RECONF("health-check-interval", priv->health_check_interval,
-                     options, uint32, out);
+                     options, time, out);
     GF_OPTION_RECONF("health-check-timeout", priv->health_check_timeout,
-                     options, uint32, out);
+                     options, time, out);
     if (priv->health_check_interval) {
         ret = posix_spawn_health_check_thread(this);
         if (ret)
@@ -720,17 +720,8 @@ posix_init(xlator_t *this)
         _private->arrdfd[i] = -1;
 
     ret = dict_get_str(this->options, "hostname", &_private->hostname);
-    if (ret) {
-        _private->hostname = GF_CALLOC(256, sizeof(char), gf_common_mt_char);
-        if (!_private->hostname) {
-            goto out;
-        }
-        ret = gethostname(_private->hostname, 256);
-        if (ret < 0) {
-            gf_msg(this->name, GF_LOG_WARNING, errno, P_MSG_HOSTNAME_MISSING,
-                   "could not find hostname ");
-        }
-    }
+    if (ret)
+        _private->hostname = gf_gethostname();
 
     /* Check for Extended attribute support, if not present, log it */
     size = sys_lgetxattr(dir_data->data, "user.x", &value, sizeof(value));
@@ -981,7 +972,7 @@ posix_init(xlator_t *this)
     ret = 0;
 
     GF_OPTION_INIT("janitor-sleep-duration", _private->janitor_sleep_duration,
-                   int32, out);
+                   time, out);
 
     /* performing open dir on brick dir locks the brick dir
      * and prevents it from being unmounted
@@ -1138,9 +1129,9 @@ posix_init(xlator_t *this)
 
     _private->health_check_active = _gf_false;
     GF_OPTION_INIT("health-check-interval", _private->health_check_interval,
-                   uint32, out);
-    GF_OPTION_INIT("health-check-timeout", _private->health_check_timeout,
-                   uint32, out);
+                   time, out);
+    GF_OPTION_INIT("health-check-timeout", _private->health_check_timeout, time,
+                   out);
     if (_private->health_check_interval) {
         ret = posix_spawn_health_check_thread(this);
         if (ret)
@@ -1254,8 +1245,6 @@ out:
 
             GF_FREE(_private->base_path);
 
-            GF_FREE(_private->hostname);
-
             GF_FREE(_private->trash_path);
 
             GF_FREE(_private);
@@ -1356,7 +1345,6 @@ posix_fini(xlator_t *this)
     pthread_cond_destroy(&priv->fsync_cond);
     pthread_mutex_destroy(&priv->janitor_mutex);
     pthread_cond_destroy(&priv->janitor_cond);
-    GF_FREE(priv->hostname);
     GF_FREE(priv->trash_path);
     GF_FREE(priv);
     this->private = NULL;
