@@ -18,93 +18,14 @@
 #include "posix-gfid-path.h"
 #include "posix.h"
 
-int32_t
-posix_set_gfid2path_xattr(xlator_t *this, const char *path, uuid_t pgfid,
-                          const char *bname)
-{
-    char xxh64[GF_XXH64_DIGEST_LENGTH * 2 + 1] = {
-        0,
-    };
-    char pgfid_bname[1024] = {
-        0,
-    };
-    char *key = NULL;
-    char *val = NULL;
-    size_t key_size = 0;
-    size_t val_size = 0;
-    int ret = 0;
-
-    GF_VALIDATE_OR_GOTO("posix", this, err);
-
-    snprintf(pgfid_bname, sizeof(pgfid_bname), "%s/%s", uuid_utoa(pgfid),
-             bname);
-    gf_xxh64_wrapper((unsigned char *)pgfid_bname, strlen(pgfid_bname),
-                     GF_XXHSUM64_DEFAULT_SEED, xxh64);
-    key_size = GFID2PATH_XATTR_KEY_PREFIX_LENGTH + GF_XXH64_DIGEST_LENGTH * 2 +
-               1;
-    key = alloca(key_size);
-    snprintf(key, key_size, GFID2PATH_XATTR_KEY_PREFIX "%s", xxh64);
-
-    val_size = UUID_CANONICAL_FORM_LEN + NAME_MAX + 2;
-    val = alloca(val_size);
-    snprintf(val, val_size, "%s/%s", uuid_utoa(pgfid), bname);
-
-    ret = sys_lsetxattr(path, key, val, strlen(val), XATTR_CREATE);
-    if (ret == -1) {
-        gf_msg(this->name, GF_LOG_WARNING, errno, P_MSG_PGFID_OP,
-               "setting gfid2path xattr failed on %s: key = %s ", path, key);
-        goto err;
-    }
-    return 0;
-err:
-    return -1;
-}
-
-int32_t
-posix_remove_gfid2path_xattr(xlator_t *this, const char *path, uuid_t pgfid,
-                             const char *bname)
-{
-    char xxh64[GF_XXH64_DIGEST_LENGTH * 2 + 1] = {
-        0,
-    };
-    char pgfid_bname[1024] = {
-        0,
-    };
-    int ret = 0;
-    char *key = NULL;
-    size_t key_size = 0;
-
-    GF_VALIDATE_OR_GOTO("posix", this, err);
-
-    snprintf(pgfid_bname, sizeof(pgfid_bname), "%s/%s", uuid_utoa(pgfid),
-             bname);
-    gf_xxh64_wrapper((unsigned char *)pgfid_bname, strlen(pgfid_bname),
-                     GF_XXHSUM64_DEFAULT_SEED, xxh64);
-    key_size = GFID2PATH_XATTR_KEY_PREFIX_LENGTH + GF_XXH64_DIGEST_LENGTH * 2 +
-               1;
-    key = alloca(key_size);
-    snprintf(key, key_size, GFID2PATH_XATTR_KEY_PREFIX "%s", xxh64);
-
-    ret = sys_lremovexattr(path, key);
-    if (ret == -1) {
-        gf_msg(this->name, GF_LOG_WARNING, errno, P_MSG_PGFID_OP,
-               "removing gfid2path xattr failed on %s: key = %s", path, key);
-        goto err;
-    }
-    return 0;
-err:
-    return -1;
-}
-
 gf_boolean_t
 posix_is_gfid2path_xattr(const char *name)
 {
     if (name && strncmp(GFID2PATH_XATTR_KEY_PREFIX, name,
-                        GFID2PATH_XATTR_KEY_PREFIX_LENGTH) == 0) {
+                        GFID2PATH_XATTR_KEY_PREFIX_LENGTH) == 0)
         return _gf_true;
-    } else {
-        return _gf_false;
-    }
+
+    return _gf_false;
 }
 
 static int gf_posix_xattr_enotsup_log;
@@ -196,8 +117,10 @@ posix_get_gfid2path(xlator_t *this, inode_t *inode, const char *real_path,
             if (size == 0)
                 goto done;
         }
-        list = alloca(size);
+
+        list = GF_MALLOC(size, gf_posix_mt_char);
         if (!list) {
+            ret = -1;
             *op_errno = errno;
             goto err;
         }
@@ -214,7 +137,8 @@ posix_get_gfid2path(xlator_t *this, inode_t *inode, const char *real_path,
         remaining_size = size;
         list_offset = 0;
         while (remaining_size > 0) {
-            snprintf(keybuffer, sizeof(keybuffer), "%s", list + list_offset);
+            len = snprintf(keybuffer, sizeof(keybuffer), "%s",
+                           list + list_offset);
 
             if (!posix_is_gfid2path_xattr(keybuffer)) {
                 goto ignore;
@@ -244,7 +168,6 @@ posix_get_gfid2path(xlator_t *this, inode_t *inode, const char *real_path,
             i++;
 
         ignore:
-            len = strlen(keybuffer);
             remaining_size -= (len + 1);
             list_offset += (len + 1);
         } /* while (remaining_size > 0) */
@@ -310,6 +233,7 @@ done:
             GF_FREE(paths[j]);
     }
     ret = 0;
+    GF_FREE(list);
     return ret;
 err:
     if (path)
@@ -318,5 +242,6 @@ err:
         if (paths[j])
             GF_FREE(paths[j]);
     }
+    GF_FREE(list);
     return ret;
 }
