@@ -135,8 +135,9 @@ posix_handle_georep_xattrs(call_frame_t *frame, const char *name, int *op_errno,
     int pid = 1;
     gf_boolean_t filter_xattr = _gf_true;
     static const char *georep_xattr[] = {
-        "*.glusterfs.*.stime", "*.glusterfs.*.xtime",
-        "*.glusterfs.*.entry_stime", "*.glusterfs.volume-mark.*", NULL};
+        "*.glusterfs.*.stime",       "*.glusterfs.*.xtime",
+        "*.glusterfs.lockinfo",      "*.glusterfs.*.entry_stime",
+        "*.glusterfs.volume-mark.*", NULL};
 
     if (!name) {
         /* No need to do anything here */
@@ -347,7 +348,7 @@ _posix_get_marker_all_contributions(posix_xattr_filler_t *filler)
         goto out;
     }
 
-    list = alloca(size);
+    list = GF_MALLOC(size, gf_posix_mt_char);
     if (!list) {
         goto out;
     }
@@ -376,6 +377,7 @@ _posix_get_marker_all_contributions(posix_xattr_filler_t *filler)
     ret = 0;
 
 out:
+    GF_FREE(list);
     return ret;
 }
 
@@ -826,8 +828,7 @@ posix_pstat(xlator_t *this, inode_t *inode, uuid_t gfid, const char *path,
             errno = op_errno; /*gf_msg could have changed errno*/
         } else {
             op_errno = errno;
-            gf_msg_debug(this->name, 0, "lstat failed on %s (%s)", path,
-                         strerror(errno));
+            gf_msg_debug(this->name, errno, "lstat failed on %s ", path);
             errno = op_errno; /*gf_msg could have changed errno*/
         }
         goto out;
@@ -1228,7 +1229,9 @@ posix_handle_pair(xlator_t *this, loc_t *loc, const char *real_path, char *key,
                         SLEN(POSIX_ACL_ACCESS_XATTR)) &&
                stbuf && IS_DHT_LINKFILE_MODE(stbuf)) {
         goto out;
-    } else if (!strncmp(key, GF_INTERNAL_CTX_KEY, SLEN(GF_INTERNAL_CTX_KEY))) {
+    } else if (!strncmp(key, GF_INTERNAL_CTX_KEY, SLEN(GF_INTERNAL_CTX_KEY)) ||
+               !strncmp(key, GF_FORCE_REPLACE_KEY,
+                        SLEN(GF_FORCE_REPLACE_KEY))) {
         /* ignore this key value pair */
         ret = 0;
         goto out;
@@ -1313,10 +1316,7 @@ posix_fhandle_pair(call_frame_t *frame, xlator_t *this, int fd, char *key,
         } else {
 #ifdef GF_DARWIN_HOST_OS
             if (errno == EINVAL) {
-                gf_msg_debug(this->name, 0,
-                             "fd=%d: key:%s "
-                             "error:%s",
-                             fd, key, strerror(errno));
+                gf_msg_debug(this->name, errno, "fd=%d: key:%s error", fd, key);
             } else {
                 gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_XATTR_FAILED,
                        "fd=%d: key:%s", fd, key);
@@ -1361,10 +1361,7 @@ del_stale_dir_handle(xlator_t *this, uuid_t gfid)
     /* check that it is valid directory handle */
     size = sys_lstat(hpath, &stbuf);
     if (size < 0) {
-        gf_msg_debug(this->name, 0,
-                     "%s: Handle stat failed: "
-                     "%s",
-                     hpath, strerror(errno));
+        gf_msg_debug(this->name, errno, "%s: Handle stat failed", hpath);
         goto out;
     }
 
@@ -1378,7 +1375,7 @@ del_stale_dir_handle(xlator_t *this, uuid_t gfid)
     size = posix_handle_path(this, gfid, NULL, newpath, sizeof(newpath));
     if (size <= 0) {
         if (errno == ENOENT) {
-            gf_msg_debug(this->name, 0, "%s: %s", newpath, strerror(ENOENT));
+            gf_msg_debug(this->name, ENOENT, "Failed for path: %s", newpath);
             stale = _gf_true;
         }
         goto out;
@@ -1386,7 +1383,7 @@ del_stale_dir_handle(xlator_t *this, uuid_t gfid)
 
     size = sys_lgetxattr(newpath, GFID_XATTR_KEY, gfid_curr, 16);
     if (size < 0 && errno == ENOENT) {
-        gf_msg_debug(this->name, 0, "%s: %s", newpath, strerror(ENOENT));
+        gf_msg_debug(this->name, ENOENT, "Failed for path: %s", newpath);
         stale = _gf_true;
     } else if (size == 16 && gf_uuid_compare(gfid, gfid_curr)) {
         gf_msg_debug(this->name, 0,
@@ -1505,7 +1502,7 @@ posix_janitor_task(void *data)
     if (!priv)
         goto out;
 
-    time(&now);
+    now = gf_time();
     if ((now - priv->last_landfill_check) > priv->janitor_sleep_duration) {
         if (priv->disable_landfill_purge) {
             gf_msg_debug(this->name, 0,
@@ -1592,16 +1589,108 @@ unlock:
     return;
 }
 
-static int
-is_fresh_file(int64_t sec, int64_t ns)
+static struct posix_fd *
+janitor_get_next_fd(glusterfs_ctx_t *ctx)
 {
-    struct timeval tv;
+    struct posix_fd *pfd = NULL;
+
+    while (list_empty(&ctx->janitor_fds)) {
+        if (ctx->pxl_count == 0) {
+            return NULL;
+        }
+
+        pthread_cond_wait(&ctx->fd_cond, &ctx->fd_lock);
+    }
+
+    pfd = list_first_entry(&ctx->janitor_fds, struct posix_fd, list);
+    list_del_init(&pfd->list);
+
+    return pfd;
+}
+
+static void
+posix_close_pfd(xlator_t *xl, struct posix_fd *pfd)
+{
+    THIS = xl;
+
+    if (pfd->dir == NULL) {
+        gf_msg_trace(xl->name, 0, "janitor: closing file fd=%d", pfd->fd);
+        sys_close(pfd->fd);
+    } else {
+        gf_msg_debug(xl->name, 0, "janitor: closing dir fd=%p", pfd->dir);
+        sys_closedir(pfd->dir);
+    }
+
+    GF_FREE(pfd);
+}
+
+static void *
+posix_ctx_janitor_thread_proc(void *data)
+{
+    xlator_t *xl;
+    struct posix_fd *pfd;
+    glusterfs_ctx_t *ctx = NULL;
+    struct posix_private *priv_fd;
+
+    ctx = data;
+
+    pthread_mutex_lock(&ctx->fd_lock);
+
+    while ((pfd = janitor_get_next_fd(ctx)) != NULL) {
+        pthread_mutex_unlock(&ctx->fd_lock);
+
+        xl = pfd->xl;
+        posix_close_pfd(xl, pfd);
+
+        pthread_mutex_lock(&ctx->fd_lock);
+
+        priv_fd = xl->private;
+        priv_fd->rel_fdcount--;
+        if (!priv_fd->rel_fdcount)
+            pthread_cond_signal(&priv_fd->fd_cond);
+    }
+
+    pthread_mutex_unlock(&ctx->fd_lock);
+
+    return NULL;
+}
+
+int
+posix_spawn_ctx_janitor_thread(xlator_t *this)
+{
+    int ret = 0;
+    glusterfs_ctx_t *ctx = NULL;
+
+    ctx = this->ctx;
+
+    pthread_mutex_lock(&ctx->fd_lock);
+    {
+        if (ctx->pxl_count++ == 0) {
+            ret = gf_thread_create(&ctx->janitor, NULL,
+                                   posix_ctx_janitor_thread_proc, ctx,
+                                   "posixctxjan");
+
+            if (ret) {
+                gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_THREAD_FAILED,
+                       "spawning janitor thread failed");
+                ctx->pxl_count--;
+            }
+        }
+    }
+    pthread_mutex_unlock(&ctx->fd_lock);
+
+    return ret;
+}
+
+static int
+is_fresh_file(struct timespec *ts)
+{
+    struct timespec now;
     int64_t elapsed;
 
-    gettimeofday(&tv, NULL);
+    timespec_now_realtime(&now);
+    elapsed = gf_tsdiff(ts, &now);
 
-    elapsed = (tv.tv_sec - sec) * 1000000L;
-    elapsed += tv.tv_usec - (ns / 1000L);
     if (elapsed < 0) {
         /* The file has been modified in the future !!!
          * Is it fresh ? previous implementation considered this as a
@@ -1610,11 +1699,7 @@ is_fresh_file(int64_t sec, int64_t ns)
     }
 
     /* If the file is newer than a second, we consider it fresh. */
-    if (elapsed < 1000000) {
-        return 1;
-    }
-
-    return 0;
+    return elapsed < 1000000000;
 }
 
 int
@@ -1677,7 +1762,9 @@ posix_gfid_heal(xlator_t *this, const char *path, loc_t *loc, dict_t *xattr_req)
         if (ret != 16) {
             /* TODO: This is a very hacky way of doing this, and very prone to
              *       errors and unexpected behavior. This should be changed. */
-            if (is_fresh_file(stbuf.ia_ctime, stbuf.ia_ctime_nsec)) {
+            struct timespec ts = {.tv_sec = stbuf.ia_ctime,
+                                  .tv_nsec = stbuf.ia_ctime_nsec};
+            if (is_fresh_file(&ts)) {
                 gf_msg(this->name, GF_LOG_ERROR, ENOENT, P_MSG_FRESHFILE,
                        "Fresh file: %s", path);
                 return -ENOENT;
@@ -1691,7 +1778,7 @@ posix_gfid_heal(xlator_t *this, const char *path, loc_t *loc, dict_t *xattr_req)
         if (ret != 16) {
             /* TODO: This is a very hacky way of doing this, and very prone to
              *       errors and unexpected behavior. This should be changed. */
-            if (is_fresh_file(stat.st_ctim.tv_sec, stat.st_ctim.tv_nsec)) {
+            if (is_fresh_file(&stat.st_ctim)) {
                 gf_msg(this->name, GF_LOG_ERROR, ENOENT, P_MSG_FRESHFILE,
                        "Fresh file: %s", path);
                 return -ENOENT;
@@ -1703,51 +1790,6 @@ posix_gfid_heal(xlator_t *this, const char *path, loc_t *loc, dict_t *xattr_req)
     return 0;
 }
 
-int
-posix_acl_xattr_set(xlator_t *this, const char *path, dict_t *xattr_req)
-{
-    int ret = 0;
-    data_t *data = NULL;
-    struct stat stat = {
-        0,
-    };
-
-    if (!xattr_req)
-        goto out;
-
-    if (sys_lstat(path, &stat) != 0)
-        goto out;
-
-    data = dict_get(xattr_req, POSIX_ACL_ACCESS_XATTR);
-    if (data) {
-        ret = sys_lsetxattr(path, POSIX_ACL_ACCESS_XATTR, data->data, data->len,
-                            0);
-#ifdef __FreeBSD__
-        if (ret != -1) {
-            ret = 0;
-        }
-#endif /* __FreeBSD__ */
-        if (ret != 0)
-            goto out;
-    }
-
-    data = dict_get(xattr_req, POSIX_ACL_DEFAULT_XATTR);
-    if (data) {
-        ret = sys_lsetxattr(path, POSIX_ACL_DEFAULT_XATTR, data->data,
-                            data->len, 0);
-#ifdef __FreeBSD__
-        if (ret != -1) {
-            ret = 0;
-        }
-#endif /* __FreeBSD__ */
-        if (ret != 0)
-            goto out;
-    }
-
-out:
-    return ret;
-}
-
 static int
 _handle_entry_create_keyvalue_pair(dict_t *d, char *k, data_t *v, void *tmp)
 {
@@ -1924,7 +1966,7 @@ posix_fs_health_check(xlator_t *this, char *file_path)
 {
     struct posix_private *priv = NULL;
     int ret = -1;
-    char timestamp[256] = {
+    char timestamp[GF_TIMESTR_SIZE] = {
         0,
     };
     int fd = -1;
@@ -1936,7 +1978,7 @@ posix_fs_health_check(xlator_t *this, char *file_path)
     char *op = NULL;
     int op_errno = 0;
     int cnt;
-    int timeout = 0;
+    time_t timeout = 0;
     struct aiocb aiocb;
 
     priv = this->private;
@@ -1950,7 +1992,7 @@ posix_fs_health_check(xlator_t *this, char *file_path)
         goto out;
     }
 
-    time_sec = time(NULL);
+    time_sec = gf_time();
     gf_time_fmt(timestamp, sizeof timestamp, time_sec, gf_timefmt_FT);
     timelen = strlen(timestamp);
 
@@ -2044,7 +2086,7 @@ out:
             ret = 0;
         } else {
             gf_event(EVENT_POSIX_HEALTH_CHECK_FAILED,
-                     "op=%s;path=%s;error=%s;brick=%s:%s timeout is %d", op,
+                     "op=%s;path=%s;error=%s;brick=%s:%s timeout is %ld", op,
                      file_path, strerror(op_errno), priv->hostname,
                      priv->base_path, timeout);
         }
@@ -2185,7 +2227,13 @@ posix_spawn_health_check_thread(xlator_t *xl)
     {
         /* cancel the running thread  */
         if (priv->health_check_active == _gf_true) {
-            pthread_cancel(priv->health_check);
+            ret = pthread_cancel(priv->health_check);
+            if (ret != 0) {
+                gf_msg(xl->name, GF_LOG_ERROR, ret, P_MSG_PTHREAD_CANCEL_FAILED,
+                       "Failed to send cancellation to health-check thread");
+                ret = -1;
+                goto unlock;
+            }
             priv->health_check_active = _gf_false;
         }
 
@@ -2211,9 +2259,8 @@ unlock:
 }
 
 void
-posix_disk_space_check(xlator_t *this)
+posix_disk_space_check(struct posix_private *priv)
 {
-    struct posix_private *priv = NULL;
     char *subvol_path = NULL;
     int op_ret = 0;
     double size = 0;
@@ -2222,16 +2269,14 @@ posix_disk_space_check(xlator_t *this)
     double totsz = 0;
     double freesz = 0;
 
-    GF_VALIDATE_OR_GOTO("posix-helpers", this, out);
-    priv = this->private;
-    GF_VALIDATE_OR_GOTO(this->name, priv, out);
+    GF_VALIDATE_OR_GOTO("posix-helpers", priv, out);
 
     subvol_path = priv->base_path;
 
     op_ret = sys_statvfs(subvol_path, &buf);
 
     if (op_ret == -1) {
-        gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_STATVFS_FAILED,
+        gf_msg("posix-disk", GF_LOG_ERROR, errno, P_MSG_STATVFS_FAILED,
                "statvfs failed on %s", subvol_path);
         goto out;
     }
@@ -2255,78 +2300,102 @@ out:
 }
 
 static void *
-posix_disk_space_check_thread_proc(void *data)
+posix_ctx_disk_thread_proc(void *data)
 {
-    xlator_t *this = NULL;
     struct posix_private *priv = NULL;
+    glusterfs_ctx_t *ctx = NULL;
     uint32_t interval = 0;
-    int ret = -1;
-
-    this = data;
-    priv = this->private;
+    struct posix_diskxl *pthis = NULL;
+    xlator_t *this = NULL;
+    struct timespec sleep_till = {
+        0,
+    };
 
+    ctx = data;
     interval = 5;
-    gf_msg_debug(this->name, 0,
-                 "disk-space thread started, "
+
+    gf_msg_debug("glusterfs_ctx", 0,
+                 "Ctx disk-space thread started, "
                  "interval = %d seconds",
                  interval);
-    while (1) {
-        /* aborting sleep() is a request to exit this thread, sleep()
-         * will normally not return when cancelled */
-        ret = sleep(interval);
-        if (ret > 0)
-            break;
-        /* prevent thread errors while doing the health-check(s) */
-        pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
 
-        /* Do the disk-check.*/
-        posix_disk_space_check(this);
-        if (!priv->disk_space_check_active)
-            goto out;
-        pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
-    }
-
-out:
-    gf_msg_debug(this->name, 0, "disk space check thread exiting");
-    LOCK(&priv->lock);
+    pthread_mutex_lock(&ctx->xl_lock);
     {
-        priv->disk_space_check_active = _gf_false;
+        while (ctx->diskxl_count > 0) {
+            list_for_each_entry(pthis, &ctx->diskth_xl, list)
+            {
+                pthis->is_use = _gf_true;
+                pthread_mutex_unlock(&ctx->xl_lock);
+
+                THIS = this = pthis->xl;
+                priv = this->private;
+
+                posix_disk_space_check(priv);
+
+                pthread_mutex_lock(&ctx->xl_lock);
+                pthis->is_use = _gf_false;
+                /* Send a signal to posix_notify function */
+                if (pthis->detach_notify)
+                    pthread_cond_signal(&pthis->cond);
+            }
+
+            timespec_now_realtime(&sleep_till);
+            sleep_till.tv_sec += 5;
+            (void)pthread_cond_timedwait(&ctx->xl_cond, &ctx->xl_lock,
+                                         &sleep_till);
+        }
     }
-    UNLOCK(&priv->lock);
+    pthread_mutex_unlock(&ctx->xl_lock);
 
     return NULL;
 }
 
 int
-posix_spawn_disk_space_check_thread(xlator_t *xl)
+posix_spawn_disk_space_check_thread(xlator_t *this)
 {
-    struct posix_private *priv = NULL;
-    int ret = -1;
+    int ret = 0;
+    glusterfs_ctx_t *ctx = this->ctx;
+    struct posix_diskxl *pxl = NULL;
+    struct posix_private *priv = this->private;
 
-    priv = xl->private;
+    pxl = GF_CALLOC(1, sizeof(struct posix_diskxl), gf_posix_mt_diskxl_t);
+    if (!pxl) {
+        ret = -ENOMEM;
+        gf_log(this->name, GF_LOG_ERROR,
+               "Calloc is failed to allocate "
+               "memory for diskxl object");
+        goto out;
+    }
+    pthread_cond_init(&pxl->cond, NULL);
 
-    LOCK(&priv->lock);
+    pthread_mutex_lock(&ctx->xl_lock);
     {
-        /* cancel the running thread  */
-        if (priv->disk_space_check_active == _gf_true) {
-            pthread_cancel(priv->disk_space_check);
-            priv->disk_space_check_active = _gf_false;
+        if (ctx->diskxl_count++ == 0) {
+            ret = gf_thread_create(&ctx->disk_space_check, NULL,
+                                   posix_ctx_disk_thread_proc, ctx,
+                                   "posixctxres");
+
+            if (ret) {
+                gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_THREAD_FAILED,
+                       "spawning disk space check thread failed");
+                ctx->diskxl_count--;
+                pthread_mutex_unlock(&ctx->xl_lock);
+                goto out;
+            }
         }
+        pxl->xl = this;
+        priv->pxl = (void *)pxl;
+        list_add_tail(&pxl->list, &ctx->diskth_xl);
+    }
+    pthread_mutex_unlock(&ctx->xl_lock);
 
-        ret = gf_thread_create(&priv->disk_space_check, NULL,
-                               posix_disk_space_check_thread_proc, xl,
-                               "posix_reserve");
-        if (ret) {
-            priv->disk_space_check_active = _gf_false;
-            gf_msg(xl->name, GF_LOG_ERROR, errno, P_MSG_DISK_SPACE_CHECK_FAILED,
-                   "unable to setup disk space check thread");
-            goto unlock;
+out:
+    if (ret) {
+        if (pxl) {
+            pthread_cond_destroy(&pxl->cond);
+            GF_FREE(pxl);
         }
-
-        priv->disk_space_check_active = _gf_true;
     }
-unlock:
-    UNLOCK(&priv->lock);
     return ret;
 }
 
@@ -2395,23 +2464,8 @@ posix_fsyncer_syncfs(xlator_t *this, struct list_head *head)
 
     stub = list_entry(head->prev, call_stub_t, list);
     ret = posix_fd_ctx_get(stub->args.fd, this, &pfd, NULL);
-    if (ret)
-        return;
-
-#ifdef GF_LINUX_HOST_OS
-        /* syncfs() is not "declared" in RHEL's glibc even though
-           the kernel has support.
-        */
-#include <sys/syscall.h>
-#include <unistd.h>
-#ifdef SYS_syncfs
-    syscall(SYS_syncfs, pfd->fd);
-#else
-    sync();
-#endif
-#else
-    sync();
-#endif
+    if (!ret)
+        (void)gf_syncfs(pfd->fd);
 }
 
 void *
@@ -3503,6 +3557,7 @@ posix_is_layout_stale(dict_t *xdata, char *par_path, xlator_t *this)
     gf_boolean_t have_val = _gf_false;
     data_t *arg_data = NULL;
     char *xattr_name = NULL;
+    size_t xattr_len = 0;
     gf_boolean_t is_stale = _gf_false;
 
     op_ret = dict_get_str_sizen(xdata, GF_PREOP_PARENT_KEY, &xattr_name);
@@ -3511,7 +3566,8 @@ posix_is_layout_stale(dict_t *xdata, char *par_path, xlator_t *this)
         return is_stale;
     }
 
-    arg_data = dict_get(xdata, xattr_name);
+    xattr_len = strlen(xattr_name);
+    arg_data = dict_getn(xdata, xattr_name, xattr_len);
     if (!arg_data) {
         op_ret = 0;
         dict_del_sizen(xdata, GF_PREOP_PARENT_KEY);
@@ -3559,7 +3615,7 @@ posix_is_layout_stale(dict_t *xdata, char *par_path, xlator_t *this)
     }
 
 out:
-    dict_del_sizen(xdata, xattr_name);
+    dict_deln(xdata, xattr_name, xattr_len);
     dict_del_sizen(xdata, GF_PREOP_PARENT_KEY);
 
     if (op_ret == -1) {
@@ -3568,3 +3624,22 @@ out:
 
     return is_stale;
 }
+
+/* Delete user xattr from the file at the file-path specified by data and from
+ * dict */
+int
+posix_delete_user_xattr(dict_t *dict, char *k, data_t *v, void *data)
+{
+    int ret;
+    char *real_path = data;
+
+    ret = sys_lremovexattr(real_path, k);
+    if (ret) {
+        gf_msg("posix-helpers", GF_LOG_ERROR, P_MSG_XATTR_NOT_REMOVED, errno,
+               "removexattr failed. key %s path %s", k, real_path);
+    }
+
+    dict_del(dict, k);
+
+    return ret;
+}
