@@ -23,7 +23,6 @@
 #define TRASH_DIR "landfill"
 
 #define UUID0_STR "00000000-0000-0000-0000-000000000000"
-#define SLEN(str) (sizeof(str) - 1)
 
 #define LOC_HAS_ABSPATH(loc) (loc && (loc->path) && (loc->path[0] == '/'))
 #define LOC_IS_DIR(loc)                                                        \
@@ -44,14 +43,13 @@
 
 #define MAKE_HANDLE_PATH(var, this, gfid, base)                                \
     do {                                                                       \
-        int __len;                                                             \
-        __len = posix_handle_path(this, gfid, base, NULL, 0);                  \
-        if (__len <= 0)                                                        \
-            break;                                                             \
-        var = alloca(__len);                                                   \
-        __len = posix_handle_path(this, gfid, base, var, __len);               \
-        if (__len <= 0)                                                        \
+        int __len = 0;                                                         \
+        int tot = PATH_MAX;                                                    \
+        var = alloca(tot);                                                     \
+        __len = posix_handle_path(this, gfid, base, var, tot);                 \
+        if (__len <= 0) {                                                      \
             var = NULL;                                                        \
+        }                                                                      \
     } while (0)
 
 /* TODO: it is not a good idea to change a variable which
