@@ -38,6 +38,11 @@
 #include "posix-aio.h"
 #endif
 
+#ifdef HAVE_LIBURING
+#include <liburing.h>
+#include "posix-io-uring.h"
+#endif
+
 #define VECTOR_SIZE 64 * 1024 /* vector size 64KB*/
 #define MAX_NO_VECT 1024
 
@@ -125,10 +130,18 @@ struct posix_fd {
     off_t dir_eof;         /* offset at dir EOF */
     struct list_head list; /* to add to the janitor list */
     int odirect;
-
+    xlator_t *xl;
     char _pad[4]; /* manual padding */
 };
 
+struct posix_diskxl {
+    pthread_cond_t cond;
+    struct list_head list;
+    xlator_t *xl;
+    gf_boolean_t detach_notify;
+    gf_boolean_t is_use;
+};
+
 struct posix_private {
     char *base_path;
     int32_t base_path_length;
@@ -137,10 +150,6 @@ struct posix_private {
     gf_lock_t lock;
 
     char *hostname;
-    /* Statistics, provides activity of the server */
-
-    struct timeval prev_fetch_time;
-    struct timeval init_time;
 
     time_t last_landfill_check;
 
@@ -170,8 +179,10 @@ struct posix_private {
     pthread_cond_t fsync_cond;
     pthread_mutex_t janitor_mutex;
     pthread_cond_t janitor_cond;
+    pthread_cond_t fd_cond;
+    pthread_cond_t disk_cond;
     int fsync_queue_count;
-    int32_t janitor_sleep_duration;
+    time_t janitor_sleep_duration;
 
     enum {
         BATCH_NONE = 0,
@@ -185,9 +196,9 @@ struct posix_private {
     char gfid2path_sep[8];
 
     /* seconds to sleep between health checks */
-    uint32_t health_check_interval;
+    time_t health_check_interval;
     /* seconds to sleep to wait for aio write finish for health checks */
-    uint32_t health_check_timeout;
+    time_t health_check_timeout;
     pthread_t health_check;
 
     double disk_reserve;
@@ -224,7 +235,6 @@ struct posix_private {
     gf_boolean_t ctime;
     gf_boolean_t janitor_task_stop;
 
-    gf_boolean_t disk_space_check_active;
     char disk_unit;
     gf_boolean_t health_check_active;
     gf_boolean_t update_pgfid_nlinks;
@@ -254,8 +264,20 @@ struct posix_private {
     gf_boolean_t aio_configured;
     gf_boolean_t aio_init_done;
     gf_boolean_t aio_capable;
+    uint32_t rel_fdcount;
 
-    char _pad[4]; /* manual padding */
+    /*io_uring related.*/
+    gf_boolean_t io_uring_configured;
+#ifdef HAVE_LIBURING
+    struct io_uring ring;
+    gf_boolean_t io_uring_init_done;
+    gf_boolean_t io_uring_capable;
+    gf_boolean_t uring_thread_exit;
+    pthread_t uring_thread;
+    pthread_mutex_t sq_mutex;
+    pthread_mutex_t cq_mutex;
+#endif
+    void *pxl;
 };
 
 typedef struct {
@@ -346,8 +368,6 @@ posix_fhandle_pair(call_frame_t *frame, xlator_t *this, int fd, char *key,
 void
 posix_janitor_timer_start(xlator_t *this);
 int
-posix_acl_xattr_set(xlator_t *this, const char *path, dict_t *xattr_req);
-int
 posix_gfid_heal(xlator_t *this, const char *path, loc_t *loc,
                 dict_t *xattr_req);
 int
@@ -662,10 +682,16 @@ posix_cs_maintenance(xlator_t *this, fd_t *fd, loc_t *loc, int *pfd,
 int
 posix_check_dev_file(xlator_t *this, inode_t *inode, char *fop, int *op_errno);
 
+int
+posix_spawn_ctx_janitor_thread(xlator_t *this);
+
 void
 posix_update_iatt_buf(struct iatt *buf, int fd, char *loc, dict_t *xdata);
 
 gf_boolean_t
 posix_is_layout_stale(dict_t *xdata, char *par_path, xlator_t *this);
 
+int
+posix_delete_user_xattr(dict_t *dict, char *k, data_t *v, void *data);
+
 #endif /* _POSIX_H */
